package bingo.util;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;
import javax.swing.tree.*;
import bingo.base.*;


/**
 *  Die Klasse dient dem Generieren von Trainingsdaten durch Parsen von
 *  Netscape-Bookmarkdateien durch Verwendung der Klasse
 *  HTMLEditorKit.ParserCallback. Die spezielle Struktur dieser Dokumente
 *  erlaubt das Extrahieren der darin enhaltenen Links und Erzeugung einer
 *  Baumhierarchie.
 *
 *@author     T
 *@created    15. Juni 2001
 */
public class BookmarkParser {

  private Vector vURLs;
  private BaseURL baseURL;
  private String sURL;
  private String sPath;


  /**
   *  Initialisiert diesen BookmarkParser mit der URL der zu parsenden
   *  Bookmarkdatei.
   *
   *@param  sURL  Description of Parameter
   */
  public BookmarkParser( String sURL ) {
    this.sURL = sURL;
    vURLs = new Vector();
  }


  /**
   *  Die Bookmarkdatei wird geparst und ein Hierarchiebaum erzeugt. Die
   *  enthaltenen Links werden mit entsprechendem Kategorieeintrag versehen als
   *  Vector von Trainingsdaten zurckgegeben.
   *
   *@return    Der Vector, der die BaseURLs enthlt.
   */
  public Vector parseBookmarks() {
    final DefaultMutableTreeNode[] nodes =
        new DefaultMutableTreeNode[SessionBuffer.maxDepth];
    HTMLEditorKit.ParserCallback callback =
      new HTMLEditorKit.ParserCallback() {

        int iLevel = 0;
        boolean bStart = false;

        /**
         *  Description of the Field
         */
        public DefaultMutableTreeNode temp = null;
        /**
         *  Description of the Field
         */
        public String lastTag = new String();


        /**
         *  Description of the Method
         *
         *@param  data  Description of Parameter
         *@param  pos   Description of Parameter
         */
        public void handleText( char[] data, int pos ) {
          temp = new DefaultMutableTreeNode( new String( data ) );
          if ( !bStart ) {
            return;
          }
          if ( lastTag.equals( "h3" ) ) {
            nodes[iLevel].add( temp );
            iLevel++;
            nodes[iLevel] = temp;
          } else if ( lastTag.equals( "a" ) ) {
            baseURL = new BaseURL( sURL, nodes[iLevel] );
            vURLs.addElement( baseURL );
          }
        }


        public void handleStartTag( HTML.Tag t, MutableAttributeSet a, int pos ) {
          lastTag = t.toString();
          if ( t.toString().equals( "title" ) ) {
            nodes[0] = new DefaultMutableTreeNode( "ROOT" );
          } else if ( t.toString().equals( "a" ) ) {
            sURL = hrefToken( a.toString() );
          }
        }


        public void handleEndTag( HTML.Tag t, int pos ) {
          if ( t.toString().equals( "dl" ) ) {
            if ( iLevel > 0 ) {
              iLevel--;
            }
          }
          if ( t.toString().equals( "title" ) ) {
            bStart = true;
          }
        }


        public void handleSimpleTag( HTML.Tag t, MutableAttributeSet a, int pos ) { }
      };
    try {
      Reader f = new FileReader( sURL );
      new ParserDelegator().parse( f, callback, false );
    } catch ( Exception anyException ) {
      System.err.println( anyException );
    }
    return vURLs;
  }


  private String hrefToken( String sInput ) {
    StringTokenizer st = new StringTokenizer( sInput );
    while ( st.hasMoreTokens() ) {
      String s = st.nextToken();
      if ( s.startsWith( "href=" ) ) {
        return s.replace( '|', ':' ).substring( 5 );
      }
    }
    System.err.println( "Cannot parse URL from Attribute: " + sInput );
    return new String();
  }
}

