package bingo.util;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.GeneralPath;
import java.awt.event.*;


/**
 *  Der Login-Dialog zur Einstellung des Hosts und User-Accounts fr die
 *  Oracle-Datenbank.
 *
 *@author     T
 *@created    18. Juni 2001
 */
public class LoginDialog extends JDialog implements ActionListener, ItemListener {
  
  KeyAdapter keyAdapter =
  new KeyAdapter() {
    public void keyPressed( KeyEvent e ) {
      if ( e.getKeyCode() == KeyEvent.VK_ENTER ) {
        ok();
      } else if ( e.getKeyCode() == KeyEvent.VK_ESCAPE ) {
        cancel();
      }
    }
  };
  private JComboBox connectBox;
  private JTextField hostField;
  private JTextField sidField;
  private JTextField userField;
  private JPasswordField passwordField;
  private JCheckBox passive;
  private String host;
  private String sid;
  private String user;
  private String password;
  private boolean isOK;
  private JButton ok;
  private JButton cancel;
  private String[] users;
  private String[] passes;
  private String[] hosts;
  private String[] sids;
  private String[] connects;
  
  private static String sLogoFile = System.getProperty( "user.dir" ) + "/bingo/data/bingo.jpg";
  
  
  /**
   *  Konstruktor fr das LoginDialog Objekt
   *
   *@param  comp      Das Start-Component.
   *@param  user      Der User.
   *@param  password  Das Passwort.
   */
  public LoginDialog() {
    super( new JFrame(), "Oracle-Login", true );
    setSize( 280, 380 );
    Container content = this.getContentPane();
    content.setLayout( new BorderLayout() );
    
    Container cont = new Container();
    cont.setLayout( new GridLayout( 2, 1, 5, 5 ) );
        
    JPanel panel = new ImagePanel( sLogoFile, new Dimension( 280, 80 ) );
    cont.add( panel );
    
    panel = createFieldPanel();
    cont.add( panel );
    
    panel = new JPanel();
    panel.setBorder( new EmptyBorder( 6, 0, 6, 12 ) );
    
    ok = new JButton( "Ok" );
    ok.addActionListener( this );
    ok.addKeyListener( keyAdapter );
    panel.add( ok );
    cancel = new JButton( "Abbruch" );
    cancel.addActionListener( this );
    panel.add( cancel );
    
    content.add( cont, BorderLayout.CENTER );
    content.add( panel, BorderLayout.SOUTH );
    
    show();
    userField.selectAll();
  }
  
  
  /**
   *  Liefert das isOk-Attribut.
   *
   *@return    Das isOK-Attribut.
   */
  public boolean isOK() {
    return isOK;
  }
  
  
  /**
   *  Liefert die Host-Eingabe oder null, falls Abbruch gewhlt wurde.
   *
   *@return    Der Host.
   */
  public String getHost() {
    return host;
  }
  
  
  /**
   *  Liefert den Service Name.
   *
   *@return    Der Service Name.
   */
  public String getSID() {
    return sid;
  }
  
  
  /**
   *  Liefert die User-Eingabe.
   *
   *@return    Der User.
   */
  public String getUser() {
    return user;
  }
  
  
  /**
   *  Liefert die Passwort-Eingabe.
   *
   *@return    Das Passwort.
   */
  public String getPassword() {
    return password;
  }
  
  
  /**
   *  Ok-Aktion.
   */
  public void ok() {

    if ( connectBox.getSelectedIndex() == connectBox.getItemCount() -1 && userField.getText().length() == 0 && passwordField.getPassword().length == 0 ) {
      user = null;
      password = null;
      sid = null;
      host = null;
      isOK = false;
      dispose();
    } else if ( hostField.hasFocus() && userField.getText().length() == 0 ) {
      userField.requestFocus();
      isOK = false;
    } else if ( userField.hasFocus() && passwordField.getPassword().length == 0 ) {
      passwordField.requestFocus();
      isOK = false;
    } else {
      user = userField.getText ();
      password = new String ( passwordField.getPassword () );
      host = hostField.getText();
      sid = sidField.getText();
      isOK = true;
      dispose();
    }
  }
  
  
  /**
   *  Abbruch-Aktion.
   */
  public void cancel() {
    isOK = false;
    dispose();
  }
  
  
  /**
   *  berwacht die Buttons.
   *
   *@param  evt  Das Event.
   */
  public void actionPerformed( ActionEvent evt ) {
    Object source = evt.getSource();
    if ( source == ok ) {
      ok();
    } else if ( source == cancel ) {
      cancel();
    }
  }
  
  
  /**
   *  berwacht das Textfeld.
   *
   *@param  evt  Das Event.
   */
  public void itemStateChanged( ItemEvent evt ) {
    sidField.setText( sids[connectBox.getSelectedIndex()] );
    passwordField.setText( passes[connectBox.getSelectedIndex()] );
    userField.setText( users[connectBox.getSelectedIndex()] );
    hostField.setText( hosts[connectBox.getSelectedIndex()] );
  }
  
  
  // erstellt Eingabefelder
  private JPanel createFieldPanel() {
    Vector accounts = readAccounts();
    users = new String[accounts.size()];
    passes = new String[accounts.size()];
    hosts = new String[accounts.size()];
    sids = new String[accounts.size()];
    connects = new String[accounts.size()];
    int i = 0;
    for ( Enumeration enum = accounts.elements(); enum.hasMoreElements(); i++ ) {
      Account ac = (Account)enum.nextElement();
      users[i] = ac.user;
      passes[i] = ac.pass;
      hosts[i] = ac.host;
      sids[i] = ac.nsid;
      if ( i < accounts.size() -1 ) {
        connects[i] = users[i]+"@"+sids[i];
      } else {
        connects[i] = "<Keine Datenbank>";
      }
    }

    JPanel container = new JPanel( new BorderLayout() );
    
    GridLayout layout = new GridLayout( 4, 2, 10, 10 );
    JPanel panel = new JPanel( layout );

    connectBox = new JComboBox( connects );
    connectBox.addKeyListener( keyAdapter );
    connectBox.addItemListener( this );
    
    
    JLabel label = new JLabel( "     User", SwingConstants.LEFT );
    panel.add( label );
    
    userField = new JTextField( "", 30 );
    userField.addKeyListener( keyAdapter );
    panel.add( userField );
    
    label = new JLabel( "     Password", SwingConstants.LEFT );
    panel.add( label );
    
    passwordField = new JPasswordField( "", 30 );
    passwordField.addKeyListener( keyAdapter );
    panel.add( passwordField );
    
    label = new JLabel( "     Host", SwingConstants.LEFT );
    panel.add( label );
    
    hostField = new JTextField( "", 30 );
    hostField.addKeyListener( keyAdapter );
    panel.add( hostField );
    
    label = new JLabel( "     Service Name", SwingConstants.LEFT );
    panel.add( label );
    
    sidField = new JTextField( "", 30 );
    sidField.addKeyListener( keyAdapter );
    panel.add( sidField );
    
    hostField.setText( hosts[connectBox.getSelectedIndex()] );
    sidField.setText( sids[connectBox.getSelectedIndex()] );
    passwordField.setText( passes[connectBox.getSelectedIndex()] );
    userField.setText( users[connectBox.getSelectedIndex()] );

    container.add( connectBox, BorderLayout.NORTH );
    container.add( panel, BorderLayout.SOUTH );
    return container;
  }

  public Vector readAccounts() {
    String s = System.getProperty( "file.separator" );
    String userDir = System.getProperty( "user.dir" );
    String dataDir = userDir + s + "bingo" + s + "data";    
    String sAccountsFile = dataDir + s + "accounts.dat";
    Vector result = new Vector();
    
    try {
      BufferedReader bf = new BufferedReader( new FileReader( sAccountsFile ) );
      String line = bf.readLine();
      while ( line != null ) {
        Account ac = new Account();
        StringTokenizer st = new StringTokenizer( line," ;," );
        ac.user = st.hasMoreTokens() ? st.nextToken() : "";
        ac.pass = st.hasMoreTokens() ? st.nextToken() : "";
        ac.host = st.hasMoreTokens() ? st.nextToken() : "-";
        ac.nsid = st.hasMoreTokens() ? st.nextToken() : "-";
        result.add( ac );
        line = bf.readLine();
      }
      bf.close();
    } catch ( Exception e ) {
      //e.printStackTrace();
    }
      
      Account ac = new Account();
      ac.user = "";
      ac.pass = "";
      ac.host = "";
      ac.nsid = "";
      result.add( ac );
      
    
    return result;
  }
}
