package bingo.util;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import bingo.base.*;
import bingo.svm.*;
import bingo.db.*;
import bingo.crawler.*;


/**
 *  Dialog zur Ausgabe einer Fehlermeldung whrend des Klassifizierens bzw.
 *  Modellierens.
 *
 *@author     T
 *@created    15. Juni 2001
 */
public class NoModelDialog extends JDialog implements ActionListener {

  private SessionBuffer session = SessionBuffer.getInstance();
  private Object[] domain_cb1 = {"Abbruch", "Lese IDF/Feature-Tabelle", "Lese SVM-Modell aus Datei", "Lese SVM-Modell aus DB"};
  private JComboBox cb1 = new JComboBox( domain_cb1 );
  private JButton ok = new JButton( "Ok" );
  private JTextField label = new JTextField();
  private JFrame parentFrame;
  private String errorMsg;
  private URL_Crawler crawler = null;


  /**
   *  Konstruktor fr das NoModelDialog Objekt
   *
   *@param  parentFrame  Der Start-Frame.
   *@param  title        Der Titel.
   *@param  errorMsg     Die Fehlermeldung.
   *@param  crawler      Description of the Parameter
   */
  public NoModelDialog( JFrame parentFrame, String title, String errorMsg, URL_Crawler crawler ) {
    super( parentFrame, title, true );
    this.parentFrame = parentFrame;
    this.errorMsg = errorMsg;
    this.crawler = crawler;
    this.setSize( 400, 115 );
    Container content = this.getContentPane();
    content.setLayout( new BorderLayout() );

    cb1.setBackground( Color.white );

    label.setText( errorMsg );
    label.setFont( new Font( "Serif", Font.BOLD, 15 ) );
    label.setBackground( Color.white );
    label.setForeground( Color.red );
    label.setEditable( false );

    JPanel panel1 = new JPanel();
    JPanel panel2 = new JPanel();

    panel1.setLayout( new FlowLayout() );
    //panel1.add( pBar );

    panel2.setLayout( new FlowLayout() );
    panel2.add( cb1 );
    panel2.add( ok );

    content.add( label, BorderLayout.CENTER );
    content.add( panel2, BorderLayout.SOUTH );

    if ( errorMsg.indexOf( "SVM" ) > -1 ) {
      cb1.setSelectedIndex( 2 );
    } else if ( errorMsg.indexOf( "IDF" ) > -1 ) {
      cb1.setSelectedIndex( 1 );
    } else {
      cb1.setSelectedIndex( 0 );
    }

    ok.addActionListener( this );

    setLocationRelativeTo( parentFrame );
    
    if ( crawler != null ) {
      crawler.stop();
    }
    
    session.bShowParserDialog = false;
  }


  /**
   *  berwacht den Button.
   *
   *@param  e  Das Event.
   */
  public void actionPerformed( ActionEvent e ) {
    String cmd = e.getActionCommand();
    if ( cmd.equals( "Ok" ) ) {
      if ( cb1.getSelectedIndex() == 0 ) {
        super.dispose();
      } else if ( cb1.getSelectedIndex() == 1 ) {
        new IDFReader( parentFrame, true ).setVisible( true );
        super.dispose();
      } else if ( cb1.getSelectedIndex() == 2 ) {
        try {
          FileDialog fileDialog = new FileDialog( parentFrame, "Modelldatei lesen", FileDialog.LOAD );
          fileDialog.setVisible( true );
          if ( fileDialog.getFile() == null ) {
            return;
          }
          String sFileName = fileDialog.getDirectory() + fileDialog.getFile();
          Vector nodeModelVector = (Vector)FileInterface.readZippedObject( sFileName );
          NodeModel nm = (NodeModel)nodeModelVector.firstElement();
          if ( nm != null ) {
            TreeNode rootNode = ( (DefaultMutableTreeNode)nm.getNode() ).getRoot();
            session.setModellingType( nm.getModellingType() );
            session.setNodeModel( nodeModelVector );
            session.setRootNode( (DefaultMutableTreeNode)rootNode );
            SVMTreeClassifier.setModel( rootNode, nodeModelVector );
          } else {
            throw new NullPointerException( "Keine gltige Modelldatei!" );
          }
        } catch ( Exception any ) {
          this.errorMsg = any.toString().trim();
          label.setText( errorMsg );
          return;
        }
        super.dispose();
      } else if ( cb1.getSelectedIndex() == 3 ) {
        new ReadModelDialog( parentFrame ).setVisible( true );
        super.dispose();
      }
    }
  }

}

