package bingo.util;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import bingo.db.*;
import bingo.base.*;
import bingo.crawler.*;
import bingo.crawler.handler.*;
import bingo.crawler.handler.parser.TextItem;
import bingo.crawler.handler.parser.Source;
import bingo.crawler.ControlDialog;


/**
 *  Die Klasse dient dem Parsen von HTML-Dokumenten unter Verwendung des
 *  HTMLEditorKit.ParserCallback. Optional werden die Stammformen gebildet bzw.
 *  Stoppwrter gestrichen. Diese Klasse hat eine optionale graphische Ausgabe.
 *
 *@author     T
 *@created    15. Juni 2001
 */
public class ParserFrame extends JInternalFrame implements ActionListener, Runnable {
  
  private static SessionBuffer session = SessionBuffer.getInstance();
  
  private String sURL;
  private JScrollPane editorPane;
  private JScrollPane parserPane;
  private JButton ok = new JButton( "OK" );
  private JTextArea parserTextArea = new JTextArea();
  private JLabel label1 = new JLabel( " HTML Ansicht " );
  private JLabel label2 = new JLabel( " Parser Ansicht " );
  private JEditorPane htmlPane;
  
  private HTMLEditorKit kit =
  new HTMLEditorKit() {
    protected HTMLEditorKit.Parser getParser() {
      HTMLEditorKit.Parser parser =
      new HTMLEditorKit.Parser() {
        public void parse( Reader r, HTMLEditorKit.ParserCallback cb, boolean ignoreCharSet ) {
          try {
            new ParserDelegator().parse( r, cb, true );
          } catch ( Exception any ) {
            //any.printStackTrace();
          }
        }
      };
      return parser;
    }
  };
  
  
  /**
   *  Konstruiert einen neuen DefaultHTMLParser zu einer URL mit einer GUI in
   *  Form eines eigenen Frames.
   *
   *@param  sURL  Die URL
   */
  public ParserFrame( String sURL ) {
    super( "Quellansicht",
    true,
    //resizable
    true,
    //closable
    true,
    //maximizable
    true );
    //iconifiable
    
    this.sURL = sURL;
    
    BINGODesktop.getInstance().add( this );
    setTitle( "Quellansicht: " + sURL );
    setSize( 700, 600 );
    setVisible( true );
    
    Container content = getContentPane();
    content.setLayout( new BorderLayout() );
    
    htmlPane = new JEditorPane();
    htmlPane.setEditable( false );
    parserTextArea.setEditable( false );
    parserTextArea.setLineWrap( true );
    editorPane = new JScrollPane( htmlPane );
    parserPane = new JScrollPane( parserTextArea );
    
    JSplitPane splitPane = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
    splitPane.setDividerSize( 2 );
    splitPane.setContinuousLayout( true );
    splitPane.setTopComponent( editorPane );
    splitPane.setBottomComponent( parserPane );
    splitPane.setDividerLocation( 420 );
    
    label1.setFont( new Font( "Serif", Font.ITALIC, 18 ) );
    label2.setFont( new Font( "Serif", Font.ITALIC, 18 ) );
    
    content.add( splitPane, BorderLayout.CENTER );
    content.add( ok, BorderLayout.SOUTH );
    
    setContentPane( content );
    ok.addActionListener( this );
    
    htmlPane.setEditorKit( kit );
    new Thread( this ).start();
  }
  
  public void run() {
    displayURL( sURL );
  }
  
  private void displayURL( String sURL ) {
    String content = null;
    try {
      
      Link link = new Link( sURL );
      Page page = new Page( link );
 
      content = DummyHttpURLConnection.readURL( sURL );
      /*
      if ( session.dbConnect ) {
        content = session.dbinterface.pullContent( new Link( sURL ).getURL() );
      } else {
        content = readURL( sURL );
      }
      */
      //System.out.println( content );
      DummyHttpURLConnection dhc = new DummyHttpURLConnection( link.url2r.origin, content );
      CrawlHandler ch = new HandlerManager().getHandler( link.url2r, dhc );
      ch.handle( null, link.url2r, dhc );
      
 
      try {
        HTMLDocument doc = (HTMLDocument)kit.createDefaultDocument();
        htmlPane.read( new StringReader( content ), doc );
      } catch ( Exception e ) {
        htmlPane.setText( "Content cannot be displayed!" );
      }
      
      for ( int i = 0; i < page.text.size(); i++ ) {
        parserTextArea.append( ((TextItem)page.text.elementAt(i)).getText() + " # " );
      }
      
    } catch ( Exception e ) {
      handleException( e );
    }
  }
  
  
  /**
   *  berwacht den Button.
   *
   *@param  e  Das Event.
   */
  public void actionPerformed( ActionEvent e ) {
    String cmd = e.getActionCommand();
    if ( cmd.equals( "OK" ) ) {
      dispose();
    }
  }
  
  
  /**
   *  Behandelt eine Exception, die whrend des Parsens geworfen wurde.
   *
   *@param  e  Die Exception.
   */
  public void handleException( Exception e ) {
    //e.printStackTrace();
    session.setErrorMsg( e.toString() );
    if ( session.bShowParserDialog ) {
      new ParserDialog( BINGODesktop.getInstance(), "Fehler beim Parsen der URL !", true, sURL, e.toString() ).setVisible( true );
    }
  }
}

