package bingo.util;

import java.io.*;
import java.util.*;
import java.awt.*;
import javax.swing.JFrame;
import javax.swing.tree.*;
import bingo.util.*;
import bingo.base.*;
import bingo.main.*;
import bingo.db.*;
import bingo.svm.*;
import bingo.crawler.*;
import xml.*;
import edu.brandeis.cs.steele.wn.*;

import java.sql.*;
import oracle.jdbc.dbaccess.*;
import oracle.jdbc.driver.*;
import oracle.sql.*;


/**
 *  Die Klasse dient als Zwischenspeicher fr diverse Objekte, auf die das
 *  System zur Laufzeit zugreift. Zudem werden systemweit gltige Konstanten
 *  definiert. Darber hinaus ist das Objekt zustndig fr den Aufbau und
 *  Abschluss der Oracle-Datenbankverbindung inklusive der Benutzer und
 *  Login-Verwaltung. Da zur Laufzeit nur genau eine Instanz dieses Objekts
 *  existieren darf, ist es als Singelton implementiert.
 *
 *@author     T
 *@created    15. Juni 2001
 */
public class SessionBuffer implements Serializable {
  
  public boolean bShowParserDialog = true;
  public boolean bParserBreak = false;
  private Hashtable stopWords;
  private Vector baseURLBuffer = new Vector();
  private Vector registeredObjects = new Vector();
  private TreeNode rootNode = null;
  private Vector nodeModel = null;
  private Hashtable svmFeatures = null;
  private Random random = new Random( new java.util.Date().getTime() );
  public static myDBInterface dbinterface = null;
  public static oracle.jdbc.driver.OracleDriver oracleDriver = null;
  public static BINGOCrawler crawler;
  
  //Systemweite Einstellungen
  
  public static int numDF = 1000;
  public static int numMI = 300;
  public static int numHITS = 12;
  
  public static int maxCrawlerThreads = 12;
  public static int downloadTimeout = 90 * 1000;
  
  public static boolean offlineCrawling = true;
  
  public static int batchSize = 100;
  public static int chunkSize = 1024;
  
  public static int numGoogleLink = 0;
  public static int numGoogleRelated = 0;
  
  public static HashMap allMimes;
  public static HashMap allowedMimes;

  public static String sAllowedMimesFile;
  public static String sAllMimesFile;
  
  public static boolean useSVMClassifier = true;

  public static boolean saveStopwordTerms = false;
  
  /**
   *  Die Stoppwortdatei.
   */
  public static String sStopwordFile;
  /**
   *  Die Standard-Bookmarkdatei zum Parsen der Hierachie.
   */
  public static String sBookmarkFile;
  /**
   *  Die Datei mit den Favoriten.
   */
  public static String sFavoriteFile;
  /**
   *  Die Datei mit den Standard-Trainingsdaten.
   */
  public static String sDefaultsFile;
  /**
   *  Der Name der Tabelle mit allen erfassten Dokumenten.
   */
  public static String sTable = "ARCHIVEDOCUMENTS";
  /**
   *  Eine Fehlermeldung.
   */
  public static String errorMsg = null;
  /**
   *  Timeout-Zeit fr eine TimedSocket in Millis.
   */
  public static int timeoutDelay = 20000;
  /**
   *  Maximal erlaubte Hierarchietiefe (Hhe des Baums).
   */
  public static int maxDepth = 100;
  
  public static int maxCrawlingDepth = 4;
  
  /**
   *  Datenbankverbindung ja/nein.
   */
  public static boolean dbConnect = false;
  
  /**
   *  Sollen die Quelltexte mitgespeichert werden?
   */
  public static boolean storeStreams = false;
  
  /**
   *  Benutzte Stemming.
   */
  public static boolean bSTEM = true;
  /**
   *  Lsche Stoppwrter.
   */
  public static boolean bCUT = true;
  /**
   *  Benutze WordNet Dictionary.
   */
  public static boolean bWN = false;
  /**
   *  Beachte Dokumente pro Knoten oder global.
   */
  private static boolean obeyLevel = true;
  /**
   *  Feature-Selektion auf Basis der Oberkategorie oder direkt im Kindknoten
   *  (restriktiver, da weniger Terme)
   */
  private static boolean useUnionSelect = true;
  /**
   *  Benutze eine Logdatei 'session.log'.
   */
  public static boolean useLog = false;
  /**
   *  Der Standard-Schwellwert fr SVM-Modelle. Der ermittelte Abstand zur
   *  Hyperebene muss fr ein positives Klassifikationsergbenis eines Dokuments
   *  ber diesem Wert liegen.
   */
  public static double dThreshold = 0.1;
  /**
   *  Der Oracle-Host.
   */
  public static String sHost;
  
  /**
   *  Der Service-Name fr die Oracle-Verbindung.
   */
  public static String sServiceName;
  
  /**
   *  Das Startverzeichnis.
   */
  public static String userDir = null;
  /**
   *  Das Datenverzeichnis.
   */
  public static String dataDir = null;
  /**
   *  Benutzername.
   */
  public static String defaultUser;
  
  public static String defaultUserPasswd;
  
  private static int modellingType = 0;
  
  private static BufferedWriter logFile;
  /**
   *  Die Oracle-Verbindung.
   */
  public static OracleConnection oracleConnection = null;
  
  /**
   *  Maximale Grsse der Crawler-Queue in der DB. Normalerweise unbeschrnkt.
   */
  public static int maxQueueSize = 20000;
  
  public static boolean hardFocus = false;
  public static boolean softFocus = true;
  
  public static int maxTunnelDistance = 3;
  // max laenge der Bruecke
  
  private DictionaryDatabase dictionary;
  private Hashtable cachedSynonyms;
  
  // Singelton
  private static SessionBuffer sessionBuffer = null;
  
  public int featuresPerClass = 1000;
  
  public int defaultFrameHeight;
  public int defaultFrameWidth;
  
  protected SessionBuffer() {
    String s = System.getProperty( "file.separator" );
    if ( userDir == null ) {
      this.userDir = System.getProperty( "user.dir" );
    }
    this.dataDir = userDir + s + "bingo" + s + "data";
    
    sStopwordFile = dataDir + s + "stopwords.html";
    sBookmarkFile = dataDir + s + "bookmarks.html";
    sFavoriteFile = dataDir + s + "favorites.dat";
    sDefaultsFile = dataDir + s + "default.dat";
    sAllowedMimesFile = dataDir + s + "allowed_mimes.dat";
    sAllMimesFile = dataDir + s + "all_mimes.dat";
    
    if ( useLog ) {
      initLog();
    }
    
    if ( bWN ) {
      dictionary = new FileBackedDictionary();
      cachedSynonyms = new Hashtable( 10000 );
    }
    
    if ( allMimes == null ) {
      allMimes = readAllMimes();
    }
    
    if ( allowedMimes == null ) {
      allowedMimes = readAllowedMimes();
    }
  
    try {
      GraphicsDevice device = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
      defaultFrameHeight = device.getDisplayMode().getHeight() * 4/7;
      defaultFrameWidth = device.getDisplayMode().getWidth() * 5/7;
    } catch ( Error any ) {
      defaultFrameHeight = 768;
      defaultFrameWidth = 1024;
    }
  }
  
  
  /*
   *  Returns the max allowed size for the given datatype (HTML, PDF...)
   */
  public static int getMaxsizeFor( String mime ) {
    try {
      if ( allowedMimes == null ) {
        return 0;
      }
      return ((MimeType)allowedMimes.get( mime )).getMaxSize();
    } catch ( NullPointerException e ) {
      System.err.println( "ContentType <" + mime + "> not found in allowed mimes!" );
      return 0;
    }
  }
  
  
  /**
   *  Initialisiert die Log-Datei 'session.log'.
   */
  public static void initLog() {
    try {
      logFile = new BufferedWriter( new FileWriter( userDir + System.getProperty( "file.separator" ) + "session.log", false ) );
      logFile.newLine();
      logFile.write( "LOG-FILE-STARTUP: " + new java.util.Date().toString() );
      logFile.newLine();
      logFile.newLine();
      logFile.flush();
      useLog = true;
    } catch ( Exception any ) {
      any.printStackTrace();
    }
  }
  
  
  /**
   *  Setzt das bSTEM Attribut, das angibt, ob der Parser zustzlich Stemming
   *  von Wrtern verwenden soll.
   *
   *@param  b  Der Wert.
   */
  public void setSTEM( boolean b ) {
    this.bSTEM = b;
  }
  
  
  /**
   *  Setzt das bCUT Attribut, das angibt, ob der Parser zustzlich Stoppwrter
   *  lschen soll.
   *
   *@param  b  Der Wert.
   */
  public void setCUT( boolean b ) {
    this.bCUT = b;
  }
  
  
  /**
   *  Legt fest, ob beim Modellieren nur URLs des jeweiligen Vaterknotens als
   *  Diskrimatoren benutzt werden sollen, oder ob fr jeden Knoten immer alle
   *  URLs der Hierarchie gegenbergestellt werden.
   *
   *@param  b  Der Wert.
   */
  public void setObeyLevel( boolean b ) {
    this.obeyLevel = b;
  }
  
  
  /**
   *  Legt fest, ob beim Modellieren Klassifizieren eines Dokuments
   *  Feature-Selektion auf der Vereiningung der Feature-Rume der
   *  Unterkategorien oder direkt im Kindknoten (restriktiver, da weniger Terme)
   *  gewhlt werden soll.
   *
   *@param  b  Der Wert.
   */
  public void setUseUnionSelect( boolean b ) {
    this.useUnionSelect = b;
  }
  
  
  /**
   *  Liefert, ob beim Modellieren Klassifizieren eines Dokuments
   *  Feature-Selektion auf der Vereiningung der Feature-Rume der
   *  Unterkategorien oder direkt im Kindknoten (restriktiver, da weniger Terme)
   *  gewhlt werden soll.
   *
   *@return    useUnionSelect Der Wert.
   */
  public boolean getUseUnionSelect() {
    return useUnionSelect;
  }
  
  
  /**
   *  Setzt das bShowParserDialog Attribut.
   *
   *@param  b  The new ShowParserDialog value
   */
  public void setShowParserDialog( boolean b ) {
    this.bShowParserDialog = b;
  }
  
  
  /**
   *  Setzt das bParserBreak Attribut.
   *
   *@param  b  Der Wert.
   */
  public void setParserBreak( boolean b ) {
    this.bParserBreak = b;
  }
  
  
  /**
   *  Parst und ldt Stoppwrter mit Hilfe des Objekts StopwordParser.
   */
  public void setStopwords() {
    StopwordParser swp = new StopwordParser( sStopwordFile );
    Vector v = swp.parseStopwords();
    stopWords = new Hashtable( v.size() );
    for ( Enumeration enum = v.elements(); enum.hasMoreElements();  ) {
      String next = (String)enum.nextElement();
      stopWords.put( next, this );
    }
  }
  
  
  /**
   *  Definiert die Termgewichtung von Features beim Modellieren und
   *  Klassifizieren. Entweder TF, TF* IDF oder TF/maxTF * log( IDF ).
   *
   *@param  i  Der Wert.
   */
  public void setModellingType( int i ) {
    this.modellingType = i;
  }
  
  
  /**
   *  Definiert den Wurzelknoten des Hierachiebaums und damit referenziert den
   *  gesamten Baum.
   *
   *@param  tn  Der Wurzelknoten.
   */
  public void setRootNode( TreeNode tn ) {
    this.rootNode = tn;
  }
  
  
  /**
   *  Definiert den Vektor mit SVM-Modellen zu den einzelnen Knoten des
   *  Hierarchiebaums.
   *
   *@param  nm  Der Vektor mit SVM-Modellen.
   */
  public void setNodeModel( Vector nm ) {
    this.nodeModel = nm;
  }
  
  
  
  public synchronized void setAllFeatureSpaces( Hashtable h ) {
    System.out.println("____________________________________" );
    for ( Enumeration enum = h.elements(); enum.hasMoreElements(); ) {
      FeatureSpace fSpace = (FeatureSpace)enum.nextElement();      
      System.out.println( "SET: " + fSpace.getClassName() + " (" +  fSpace.size() + ")" );
    }
    System.out.println("____________________________________" );
    this.svmFeatures = h;
  }
  
  public synchronized void setAllFeatureSpaces( Vector s ) {
    Hashtable h = new Hashtable( s.size() );
    System.out.println("____________________________________" );
    for ( Enumeration enum = s.elements(); enum.hasMoreElements(); ) {
      FeatureSpace fSpace = (FeatureSpace)enum.nextElement();
      System.out.println( "SET: " + fSpace.getClassName() + " (" +  fSpace.size() + ")" );
      h.put( fSpace.getClassName(), fSpace );
    }
    System.out.println("____________________________________" );
    this.svmFeatures = h;
  }
  
  public synchronized void setFeatureSpace( FeatureSpace fSpace ) {
    if ( svmFeatures == null ) {
      svmFeatures = new Hashtable();
    }
    if ( svmFeatures.containsKey( fSpace.getClassName() ) ) {
      svmFeatures.remove( fSpace );
    }
    svmFeatures.put( fSpace.getClassName(), fSpace );
  }
  
  
  /**
   *  Liefert den Vektor mit allen zwischengespeicherten BaseURL-Objekten.
   *
   *@return    Der Vektor.
   */
  public Vector getURLBuffer() {
    return baseURLBuffer;
  }
  
  
  /**
   *  Prft einen String auf Stoppwort.
   *
   *@param  s  Description of the Parameter
   *@return    The stopword value
   */
  public boolean isStopword( String s ) {
    return ( stopWords.get( s ) != null );
  }
  
  
  public Enumeration getStopwords() {
    return stopWords.elements();
  }
  
  
  public String lookupDictionary( String s ) {
    String cSyn = (String)cachedSynonyms.get( s );
    try {
      if ( cSyn != null ) {
        return cSyn;
      } else {
        for ( int i = 0; i < POS.CATS.length; i++ ) {
          IndexWord word = dictionary.lookupIndexWord( POS.CATS[i], s );
          if ( word != null ) {
            Synset[] senses = word.getSenses();
            Synset sense = senses[0];
            String syn = sense.getWord( 0 ).getDescription().toLowerCase();
            System.out.println( "WN:     " + s + " --> " + syn );
            cachedSynonyms.put( s, syn );
            return syn;
          }
        }
      }
    } catch ( Exception any ) {}
    return s;
  }
  
  
  public void setUseWordNet( boolean b ) {
    if ( b ) {
      dictionary = new FileBackedDictionary();
      if ( cachedSynonyms == null ) {
        cachedSynonyms = new Hashtable( 10000 );
      }
    } else {
      dictionary = null;
    }
    bWN = b;
  }
  
  
  /**
   *  Liefert die Modellierungsmethode.
   *
   *@return    Der Wert.
   */
  public int getModellingType() {
    return modellingType;
  }
  
  
  /**
   *  Liefert den Wurzelknoten des Hierachiebaums und damit referenziert den
   *  gesamten Baum.
   *
   *@return    Der Wurzelknoten.
   */
  public TreeNode getRootNode() {
    return rootNode;
  }
  
  
  /**
   *  Liefert den Vektor mit SVM-Modellen zu den einzelnen Knoten des
   *  Hierarchiebaums.
   *
   *@return    Der Vektor.
   */
  public Vector getNodeModel() {
    return nodeModel;
  }
  
  
  /**
   *  Liefert die Tabelle mit den IDF-Werten aller Features einer Klasse.
   *
   *@param  sClass  Die Klasse als String.
   *@return         Die zugehrige IDF-Tabelle.
   */
  public IDFTable getIDFTable( String sClass ) {
    if ( svmFeatures == null ) {
      return null;
    }
    for ( Enumeration enum = svmFeatures.elements(); enum.hasMoreElements();  ) {
      IDFTable t = (IDFTable)enum.nextElement();
      if ( t.getClassName().equals( sClass ) ) {
        return t;
      }
    }
    return null;
  }
  
  public FeatureSpace getFeatureSpace( String sClass ) {
    if ( svmFeatures == null ) {
      return null;
    }
    return (FeatureSpace)svmFeatures.get( sClass );
  }
  
  public Hashtable getAllFeatureSpaces() {
    return svmFeatures;
  }
  
  
  /**
   *  Lscht die Zwischenablage.
   */
  public void clearURLBuffer() {
    this.baseURLBuffer = new Vector();
  }
  
  
  /**
   *  Setzt ein Objekt in die Zwischenablage.
   *
   *@param  baseURL  Die BaseURL.
   */
  public void addURLtoBuffer( Object baseURL ) {
    baseURLBuffer.addElement( baseURL );
  }
  
  
  /**
   *  Setzt einen Vektor von Objekten als Zwischenablage.
   *
   *@param  vURLs  The new uRLBuffer value
   */
  public void setURLBuffer( Vector vURLs ) {
    baseURLBuffer = vURLs;
  }
  
  
  /**
   *  Registriert ein Objekt vom Typ TreeUpdateListener zur Benachrichtigung bei
   *  Hierachienderungen.
   *
   *@param  l  Der Listener.
   */
  public void registerForTreeUpdating( TreeUpdateListener l ) {
    if ( !registeredObjects.contains( l ) ) {
      registeredObjects.addElement( l );
    }
  }
  
  
  /**
   *  Benachrichtigt alle registrierten Objekte vom Typ TreeUpdateListener, dass
   *  eine nderung der Trainingsdaten bzw. des Hierachiebaums vorliegt.
   *
   *@param  vTrain  Die neuen Trainingsdaten.
   */
  public void updateTree( Vector vTrain ) {
    for ( Enumeration enum = registeredObjects.elements(); enum.hasMoreElements();  ) {
      Object o = enum.nextElement();
      try {
        ( (TreeUpdateListener)o ).updateTree( vTrain );
      } catch ( Exception any ) {
        any.printStackTrace();
      }
    }
  }
  
  
  /**
   *  Schliesst die OracleConnection.
   */
  public void closeDB() {
    if ( dbConnect ) {
      try {
        dbinterface.close();
        DBFrame.close();
        oracleConnection.close();
        System.out.println( "DB-Verbindung: " + sServiceName + ":" + sHost + " geschlossen." );
      } catch ( Exception e ) {
        e.printStackTrace();
      }
    }
  }
  
  
  /**
   *  Speichert eine Fehlermeldung.
   *
   *@param  s  Die neue Fehlermeldung.
   */
  public static void setErrorMsg( String s ) {
    errorMsg = s;
  }
  
  
  /**
   *  Erstellt eine OracleConnection ber die JDBC-Schnittstelle
   *  (jdbc.driver.OracleDriver).
   *
   *@return    Die OracleConnection.
   */
  public static synchronized OracleConnection getOracleConnection() {
    OracleConnection oc = null;
    try {
      if ( oracleDriver == null ) {
        oracleDriver = new oracle.jdbc.driver.OracleDriver();
        DriverManager.registerDriver( oracleDriver );
      }
      
      // DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      //Connection conn = DriverManager.getConnection("jdbc:oracle:oci8:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=134.96.246.200)(PORT=1521)))(CONNECT_DATA=(service_name=sydney.world)))","bingo", "00bingo00");
      oc = (OracleConnection)DriverManager.getConnection(
      "jdbc:oracle:thin:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=" + sHost + ")(PORT=1521)))(CONNECT_DATA=(SERVICE_NAME=" + sServiceName + ")(server = dedicated)))",
      defaultUser,
      defaultUserPasswd
      );
      oc.setAutoCommit( false );
      SessionBuffer.log( "Neue DB-Verbindung: " + sServiceName + ":" + sHost + " aufgebaut." );
      dbConnect = true;
      
    } catch ( SQLException q ) {
      //q.printStackTrace();
      System.err.println( q );
      dbConnect = false;
    }
    return oc;
  }
  
  
  /**
   *  Entscheidet ob beim Modellieren nur URLs des jeweiligen Vaterknotens als
   *  Diskrimatoren benutzt werden sollen, oder ob fr jeden Knoten immer alle
   *  URLs der Hierarchie gegenbergestellt werden.
   *
   *@return    Der Wert.
   */
  public static boolean getObeyLevel() {
    return obeyLevel;
  }
  
  
  /**
   *  Liefert eine zwischengespeicherte Fehlermeldung.
   *
   *@return    Die Fehlermeldung.
   */
  public static String getErrorMsg() {
    return errorMsg;
  }
  
  
  /**
   *  Erzeugt eine Zufallszahl (Integer), die in einem gebenenen Intervall liegt
   *  und die nicht bereits in einem Vector von Zahlen vorhanden ist.
   *
   *@param  excluded  Der Vector mit auszuschliessenden Zahlen
   *@param  lower     Die untere Schranke des Intervalls.
   *@param  upper     Die obere Schranke des Intervalls.
   *@return           Die Zufallszahl.
   */
  public int getRandom( Vector excluded, int lower, int upper ) {
    Integer i = new Integer( 0 );
    boolean bFound = false;
    
    do {
      bFound = false;
      i = new Integer( Math.abs( ( random.nextInt() + lower ) % upper ) );
      for ( Enumeration enum = excluded.elements(); enum.hasMoreElements();  ) {
        if ( i.intValue() == ( (Integer)enum.nextElement() ).intValue() ) {
          bFound = true;
          break;
        }
      }
    } while ( bFound );
    
    return i.intValue();
  }
  
  
  /**
   *  Liefert ein neues Statement fr eine DB-Verbindung
   *
   *@return    Das Statement.
   */
  public static OracleStatement getStatement() {
    try {
      return (OracleStatement)oracleConnection.createStatement();
    } catch ( Exception any ) {
      return null;
    }
  }
  
  
  /**
   *  Methode zum Aufruf der (einzigen) Instanz der Klasse. Es wird keine
   *  Datenbankverbindung aufgebaut.
   *
   *@return    Die Instanz.
   */
  public static synchronized SessionBuffer getInstance() {
    if ( sessionBuffer == null ) {
      sessionBuffer = new SessionBuffer();
      sessionBuffer.setStopwords();
    }
    return sessionBuffer;
  }
  
  
  /**
   *  Methode zum Aufruf der (einzigen) Instanz der Klasse. Es wird eine
   *  Datenbankverbindung zu dem spezififizierten Oracle-Host aufgebaut. Dieser
   *  Verbindungsaufbau ist nur einmal zur Laufzeit erlaubt.
   *
   *@param  host         Der Host.
   *@param  serviceName  Der ServiceName.
   *@return              Die Instanz.
   */
  public static SessionBuffer getInstance( String user, String pass, String host, String serviceName ) {
    defaultUser = user;
    defaultUserPasswd = pass;
    sHost = host;
    sServiceName = serviceName;
    
    if ( sessionBuffer == null ) {
      sessionBuffer = new SessionBuffer();
      sessionBuffer.setStopwords();
    }
    if ( !dbConnect ) {
      oracleConnection = getOracleConnection();
      if ( oracleConnection == null ) {
        System.exit( 0 );
      }
      dbinterface = new myDBInterface();
    }
    return sessionBuffer;
  }
  
  
  /**
   *  Methode zum Aufruf der (einzigen) Instanz der Klasse. Es wird angegeben,
   *  ob eine Datenbankverbindung zu einem vorher spezififizierten Oracle-Host
   *  aufgebaut werden soll. Dieser Verbindungsaufbau ist nur einmal zur
   *  Laufzeit erlaubt.
   *
   *@param  connect  Benutze DB-Anbindung.
   *@return          Die Instanz.
   */
  public static SessionBuffer getInstance( boolean connect ) {
    if ( !dbConnect && connect ) {
      LoginDialog lDialog = new LoginDialog();
      if ( lDialog.isOK() ) {
        defaultUser = lDialog.getUser();
        defaultUserPasswd = lDialog.getPassword();
        sHost = lDialog.getHost();
        sServiceName = lDialog.getSID();
        oracleConnection = getOracleConnection();
        dbinterface = new myDBInterface();
      } else {
        defaultUser = null;
        defaultUserPasswd = null;
        sHost = null;
        sServiceName = null;
      }
      if ( sHost == null || !connect ) {
        SessionBuffer.log( "DB-Verbindung: <keine>" );
        dbConnect = false;
      }
    }
    
    if ( !dbConnect && !BINGODesktop.isNull() ) {
      BINGODesktop.getInstance().disableDBMenues();
    }
    
    if ( sessionBuffer == null ) {
      sessionBuffer = new SessionBuffer();
      sessionBuffer.setStopwords();
    }
    
    return sessionBuffer;
  }
  
  
  /**
   *  Erstellt einen Log-Eintrag in 'session.log'.
   *
   *@param  sEvent  Der Eintrag.
   */
  public static void log( String sEvent ) {
    System.out.println( sEvent );
    if ( useLog ) {
      try {
        logFile.write( sEvent );
        logFile.newLine();
        logFile.flush();
      } catch ( Exception any ) {
        any.printStackTrace();
        useLog = false;
      }
    }
  }
  
  
  public static int getBatchSize() {
    return batchSize;
  }
  
  
  public static int getChunkSize() {
    return chunkSize;
  }
  
  
  public void finalize() {
    closeDB();
  }
  
  
  public static BINGOCrawler getCrawler() {
    if ( crawler == null ) {
      crawler = new BINGOCrawler();
    }
    return crawler;
  }
  
  
  private static void asciis() {
    try {
      Statement stmt = getStatement();
      ResultSet rset = null;
      for ( int i = 20; i < 200; i++ ) {
        rset = stmt.executeQuery( "SELECT CHR(" + String.valueOf( i ) + ") FROM DUAL" );
        rset.next();
        System.out.println( i + ": " + rset.getString( 1 ) );
      }
      rset.close();
      stmt.close();
    } catch ( Exception e ) {
      e.printStackTrace();
    }
  }
  
  
  public int getNumGoogleLink() {
    return numGoogleLink;
  }
  
  
  public int getNumGoogleRelated() {
    return numGoogleRelated;
  }
  
  
  public void setNumGoogleLink( int num ) {
    numGoogleLink = num;
  }
  
  
  public void getNumGoogleRelated( int num ) {
    numGoogleRelated = num;
  }
  
  
  public int getFeaturesPerClass() {
    return featuresPerClass;
  }
  
  
  public void setFeaturesPerClass( int num ) {
    featuresPerClass = num;
  }
  
  
  public HashMap readAllMimes() {
    HashMap result = new HashMap();
    try {
      BufferedReader bf = new BufferedReader( new FileReader( sAllMimesFile ) );
      String line = bf.readLine();
      while ( line != null ) {
        int pos = line.indexOf( " " );
        String key = ( line.substring( 0, pos ) ).trim().toLowerCase();
        String value = ( line.substring( pos + 1, line.length() ) ).trim().toLowerCase();
        if ( !result.containsKey( key ) ) {
          result.put( key, new MimeType( key, value, false, 400000 ) );
        }
        line = bf.readLine();
      }
      bf.close();
    } catch ( Exception e ) {
      e.printStackTrace();
    }
    
    return result;
  }
  
  
  public HashMap readAllowedMimes() {
    HashMap result = new HashMap();
    
    try {
      BufferedReader bf = new BufferedReader( new FileReader( sAllowedMimesFile ) );
      String line = bf.readLine();
      while ( line != null ) {
        int pos = line.indexOf( " " );
        int size = Integer.parseInt( line.substring( pos + 1, line.length() ) );
        String extension = "";
        String description = line.substring( 0, pos );
        result.put( description, new MimeType( extension, description, true, size ) );
        line = bf.readLine();
      }
      bf.close();
    } catch ( Exception e ) {
      e.printStackTrace();
    }
    
    return result;
  }
  
  public void setParameters( bingo.adapter.BingoParameters p ) {
    numDF = p.numDF;
    numMI = p.numMI;
    numHITS = p.numHITS;
    maxCrawlerThreads = p.maxCrawlerThreads;
    downloadTimeout = p.downloadTimeout;
    offlineCrawling = p.offlineCrawling;
    batchSize = p.batchSize;
    chunkSize = p.chunkSize;
    numGoogleLink = p.numGoogleLink;
    numGoogleRelated = p.numGoogleRelated;
    useSVMClassifier = p.useSVMClassifier;
    saveStopwordTerms = p.saveStopwordTerms;
    maxCrawlingDepth = p.maxCrawlingDepth;
    storeStreams = p.storeStreams;
    dThreshold = p.dThreshold;
    modellingType = p.modellingType;
    maxQueueSize = p.maxQueueSize;
    
    hardFocus = p.hardFocus;
    softFocus = p.softFocus;
    
    maxTunnelDistance = p.maxTunnelDistance;
  }
  
  public bingo.adapter.BingoParameters getParameters() {
    bingo.adapter.BingoParameters p = new bingo.adapter.BingoParameters();
    p.numDF = numDF;
    p.numMI = numMI;
    p.numHITS = numHITS;
    p.maxCrawlerThreads = maxCrawlerThreads;
    p.downloadTimeout = downloadTimeout;
    p.offlineCrawling = offlineCrawling;
    p.batchSize = batchSize;
    p.chunkSize = chunkSize;
    p.numGoogleLink = numGoogleLink;
    p.numGoogleRelated = numGoogleRelated;
    p.useSVMClassifier = useSVMClassifier;
    p.saveStopwordTerms = saveStopwordTerms;
    p.maxCrawlingDepth = maxCrawlingDepth;
    p.storeStreams = storeStreams;
    p.dThreshold = dThreshold;
    p.modellingType = modellingType;
    p.maxQueueSize = maxQueueSize;
    
    p.hardFocus = hardFocus;
    p.softFocus = softFocus;
    
    p.maxTunnelDistance = maxTunnelDistance;
    return p;
  }
}

