package bingo;

import java.sql.*;
import oracle.jdbc.dbaccess.*;
import oracle.sql.*;

/**
 * Bean fuer "bingo_feed_admin.jsp":
 * Zeigt Statistiken des vorhandenen Datenbestandes und die Parameter des
 * Systems (welche Dokumente sollen angezeigt werden, Handling der
 * OTHERS-Klassen, etc.).
 * Die gewaehlten Parameter werden von "bingo_feed_admin.jsp" an
 * "bingo_feed_settings.jsp" zur Weiterverarbeitung uebermittelt.
 */
public class bingo_feed_admin
{

  /**
   * Fuehrt die DB-Abfrage durch:
   * Oeffnet die DB-Verbindung, fragt die Inlays sowie die aktuellen
   * DB-Einstellungen ab und speichert die erhaltenen Werte in den
   * entsprechenden Attributen, schliesst die Verbindung.
   * @param dbconn Angabe der Datenbankverbindung
   * @param user Angabe der Benutzernamen fuer DB-Verbindung
   * @param passwort Angabe des Passworts fuer DB-Verbindung
   */
  public void runDBQuery(String dbconn, String user, String passwort)
  {
    this.hadSQLException = false;
    this.errorMessage = "";
    try
    {
      // DB-Verbindung oeffnen
      DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      Connection conn = DriverManager.getConnection(dbconn, user, passwort);
      Statement stmt = conn.createStatement();
      ResultSet rset;      

      // Abfrage der aktuellen DB-Einstellungen
      rset = stmt.executeQuery ("select URL_SELECT, OTHERS_SELECT, SCORE_SELECT, TOP_SELECT from feedback_options");
      if (rset.next ())
      {
        urlSelect = rset.getString(1);
        if (rset.getString("OTHERS_SELECT").equalsIgnoreCase("TRUE"))
            othersSelect = true;
        else 
            othersSelect = false;
        scoreSelect = rset.getString("SCORE_SELECT");
        topSelect = rset.getString("TOP_SELECT");
      }      
      
      // Abfrage, ob ein Feedback in der DB existiert
      rset = stmt.executeQuery ("select count(*) from feedback");
      rset.next();
      if (rset.getInt(1) > 0)
        this.feedbackVorhanden = true;
      else
        this.feedbackVorhanden = false;

      // Schliessen der DB-Verbindung
      stmt.close();
      conn.close();
    }
    catch (SQLException e)
    {
      this.hadSQLException = true;
      this.errorMessage = e.getMessage ();
    }
  }

  /**
   * Gibt an, ob bei DB-Abfrage Fehler aufgetreten sind.
   * @return false, wenn Abfrage fehlerfrei
   */
  public boolean getHadSQLException ()
  {
    return this.hadSQLException;
  }

  /**
   * Gibt (falls vorhanden) Fehlermeldung der DB-Abfrage aus.
   * @return Fehlermeldung der DB-Abfrage
   */
  public String getErrorMessage ()
  {
    if (errorMessage.length() != 0)
      return errorMessage;
    else
      return "No ErrorMessage available";
  }


  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return aktuelle Einstellung fuer URL_SELECT
   */
  public String getUrlSelect ()
  {
    return this.urlSelect;
  }

  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return aktuelle Einstellung fuer OTHERS_SELECT
   */
  public boolean getOthersSelect ()
  {
    return this.othersSelect;
  }
  
  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return aktuelle Einstellung fuer SCORE_SELECT
   */
  public String getScoreSelect ()
  {
    return this.scoreSelect;
  }
  
  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return aktuelle Einstellung fuer TOP_SELECT
   */
  public String getTopSelect ()
  {
    return this.topSelect;
  }
  
  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return true, falls Feedback in der DB existiert
   */
  public boolean getFeedbackVorhanden()
  {
    return this.feedbackVorhanden;    
  }

  
  private boolean hadSQLException = false;
  private String errorMessage = "";
  private String urlSelect = "MBC";
  private boolean othersSelect = true;
  private String topSelect = "20";
  private String scoreSelect ="SvmScore";
  private boolean feedbackVorhanden = true;
 

}