package bingo;

import java.lang.Integer;
import java.util.Vector;
import java.util.Hashtable;

import java.sql.*;
 
/**
 * Bean fuer "bingo_feed_check.jsp":
 * Generiert die Evaluations-Seite fuer die vorgegebene Iteration
 * unter Beruecksichtigung der Einstellungen aus der Tabelle
 * feedback_options.
 */
public class bingo_feed_check
{
  /**
   * oeffnet die Verbindung zur Datenbank, fuehrt alle notwendigen
   * Abfragen aus und speichert die Ergebnisse in entsprechenden
   * Strukturen, schliesst die DB-Verbindung
   * @param dbconn Angabe der Datenbankverbindung
   * @param user Angabe der Benutzernamen fuer DB-Verbindung
   * @param passwort Angabe des Passworts fuer DB-Verbindung
   */
  public void executeDBQuery (String dbconn, String user, String passwort)
  {
    this.hadSQLException = false;
    this.errorMessage = "";
    String query = null;
    try
    {
      // oeffnen der DB-Verbindung
      DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      Connection conn = DriverManager.getConnection(dbconn, user, passwort);
      final Statement stmt = conn.createStatement();
      final Statement stmt1 = conn.createStatement();
      ResultSet rset;
      ResultSet rset1;

      // Abfrage der Feedback_Options
      boolean othersSelect = true;
      String urlSelect = "MBC";
      String scoreSelect = "SvmScore";
      String top = "20";
      rset = stmt.executeQuery ("select * from feedback_options");      
      if (rset.next())
      {
        urlSelect = rset.getString("URL_SELECT");
        if (rset.getString("OTHERS_SELECT").equalsIgnoreCase("TRUE"))
            othersSelect = true;
        else 
            othersSelect = false;
        scoreSelect = rset.getString("SCORE_SELECT");
        top = rset.getString("TOP_SELECT");
      }
      char[] urlarten = urlSelect.toCharArray();
      
      
      // Abfrage, ob es bereits ein Feedback gibt
      rset = stmt.executeQuery ("select count(*) from feedback");
      rset.next();
      if (rset.getInt(1) > 0)
        this.feedbackVorhanden = true;
      else
        this.feedbackVorhanden = false;
      
      
      // Abfrage der verschiedenen Klassen
      query = "select distinct Class_ID, Name from Classes "; 
      if (!othersSelect)
        query += "where Name not like '%others%' and Name not like '%OTHERS%' ";
      query += "order by Name";
      rset = stmt.executeQuery (query); 
      
      
      // Abfrage der detaillierten Daten fuer jede Klasse
      klassen.clear();
      klassenId.clear();
      int index = 0;
      while (rset.next())
      {
        int classId = rset.getInt("Class_ID");
        
        query = "SELECT AVG(" + scoreSelect + ") FROM BingoDocuments WHERE Class_ID = " + classId + " AND Status IN ('B','M')";
        rset1 = stmt1.executeQuery (query);
        rset1.next();
        double aver = rset1.getDouble(1);
        if (aver == 0)
           aver = (double)0.00001;
        rset1.close();
        
        String className = rset.getString ("Name");
        Vector inhalt = new Vector();        
        
        query = "select * from ( ";
        query += "select BD.ID, BD.URL, ArchiveFiles.Title, ";
        query += "ArchiveFiles.Preview, BD."+scoreSelect+", BD.class_id, Feedback.class_new ";               
        query += "from BingoDocuments BD LEFT OUTER JOIN Feedback On (BD.ID=Feedback.ID), ";
        query += "ArchiveFiles ";
        query += "where BD.ID=ArchiveFiles.ID and ";
        query += "BD.class_id='"+ classId +"'"; 
        if (urlSelect.length()>0)
        {
            for (int i=0; i < urlarten.length; i++)
            { 
                if (i==0) 
                    query+= "and ( ";
                else 
                    query += "or ";
                query += "BD.status='"+urlarten[i]+"' ";
            }
            query += " ) ";
        } 
        query +="order by BD."+ scoreSelect+ " desc ";
        query +=") where rownum <= '"+top+"'";        
        rset1 = stmt1.executeQuery (query);
        
        // Ergenisse fuer jede Klasse speichern
        while (rset1.next())
        {
          Hashtable details = new Hashtable();
          details.put("ID", rset1.getString(1));
          details.put("URL", rset1.getString(2));
          details.put("Title", rset1.getString(3));
          details.put("Preview", rset1.getString(4));
          details.put("Rating", new Integer ((int) Math.ceil(100 * rset1.getDouble(5) / aver)).toString() + " % ") ;
          details.put("Score", rset1.getString(5));
          details.put("KlassenId", rset1.getString(6));          
          if (rset1.getString(7) != null)
            details.put("KlasseNeu", rset1.getString(7));
          else
            details.put("KlasseNeu", (new Integer(classId)).toString());
          inhalt.addElement(details);
        }
        klassen.put(className, inhalt);
        klassenId.put(new Integer(index++), className);
        klassenId.put(className, new Integer (classId));
      }
      
      //Schliessen der DB-Verbindung
      stmt.close();
      stmt1.close();
      conn.close();
    }
    catch (SQLException e)
    {
      this.hadSQLException = true;
      this.errorMessage = e.getMessage() + query;
    }
    query = null;
  }



  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return Klassen gemappt mit einem Vektor, der die Informationen
   * der zu dieser Klasse gehoerenden URLs als Hashtables speichert
   */
  public Hashtable getKlassen ()
  {
    return this.klassen;
  }
  
  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return Indexe gemappt mit Klassennamen sowie Klassennamen 
   * gemappt mit den Klassen-IDs
   */
  public Hashtable getKlassenId ()
  {
    return this.klassenId;
  }
  
  /**
   * Abfrage der aus der DB ausgelesenen Daten
   * @return true, wenn ein Feedback in der Datenbank existiert
   */
  public boolean getFeedbackVorhanden ()
  {
    return this.feedbackVorhanden;
  }

  /**
   * Gibt an, ob bei DB-Abfrage Fehler aufgetreten sind.
   * @return false, wenn Abfrage fehlerfrei
   */
  public boolean getHadSQLException ()
  {
    return this.hadSQLException;
  }

  /**
   * Gibt (falls vorhanden) Fehlermeldung der DB-Abfrage aus.
   * @return Fehlermeldung der DB-Abfrage
   */
  public String getErrorMessage ()
  {
    if (errorMessage.length() != 0)
      return errorMessage;
    else
      return "No ErrorMessage available";
  }


  /**
   * speichert die verschiedenen Klassen mit saemtlichen Details:
   * mappt den Klassennamen mit Vector Inhalt, dieser Vector
   * enthaelt die Details zu jeder Seite als Hashtable
   */
  private Hashtable klassen = new Hashtable();
  
  /**
   * ordnet einem Index einen Klassennamen zu (fuer die alphabetische 
   * Reihenfolge bei der Ausgabe) und speichert auerdem zu jedem 
   * Klassennamen die entsprechende ID
   */
  private Hashtable klassenId = new Hashtable();
  
  /**
   * gibt an, ob bereits ein Feedback in der Datenbank existiert
   */
  private boolean feedbackVorhanden = false;

  private boolean hadSQLException = false;

  private String errorMessage = "";

}