package bingo;

import java.util.Vector;
import java.util.HashMap;
import java.util.Hashtable;
import java.lang.Integer;

import java.sql.*;
import oracle.jdbc.dbaccess.*;
import oracle.sql.*;

/**
 * Bean fuer "bingo_feed_summary.jsp":
 * Anzeige der resultierenden Statistiken auf der Basis des
 * eingegebenen User-Feedbacks.
 */
public class bingo_feed_summary
{
  
  public Exception error = null;
   
  /**
   * Fuehrt die DB-Abfrage aus:
   * Oeffnet die Verbindung; berechnet fuer jede Klasse Accuracy,
   * Precision und Recall; berechnet die jeweiligen Micro- und Macro-
   * bewertungen; schliesst die DB-Verbindung wieder.
   * @param dbconn Angabe der Datenbankverbindung
   * @param user Angabe der Benutzernamen fuer DB-Verbindung
   * @param passwort Angabe des Passworts fuer DB-Verbindung
   */
  public void runDBQuery(String dbconn, String user, String passwort)
  {
    this.hadSQLException = false;
    this.errorMessage = "";
    try
    {
      // DB-Verbindung oeffnen
      DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      Connection conn = DriverManager.getConnection(dbconn, user, passwort);
      Statement stmt = conn.createStatement();
      ResultSet rset;

      // Feedback-Option abfragen
      boolean othersSelect = true;
      String urlSelect = "MBC";
      rset = stmt.executeQuery ("select * from feedback_options");      
      if (rset.next())
      {
        urlSelect = rset.getString("URL_SELECT");
        if (rset.getString("OTHERS_SELECT").equalsIgnoreCase("TRUE"))
            othersSelect = true;
        else 
            othersSelect = false;
      }
      char[] urlarten = urlSelect.toCharArray();
      
      // Prfen, ob Feedback vorhanden ist
      rset = stmt.executeQuery ("select count(*) from feedback");
      rset.next();
      if (rset.getInt(1) > 0)
        this.feedbackVorhanden = true;
      else
        this.feedbackVorhanden = false;
      
      // Wenn Feedback da ist, Berechnungen durchfuehren
      try
      {
          if (this.feedbackVorhanden)
          {
              // Liste aller Klassen vorbereiten
              String query= "select distinct Class_ID, Name from Classes ";
              if (!othersSelect)
                query += "where Name not like '%others%' and Name not like '%OTHERS%'";
              query += "order by Name asc";
              rset = stmt.executeQuery (query); 
              Vector klassen = new Vector();
              HashMap klassennamen = new HashMap();
              int index = 0;
              while (rset.next())
              {
                klassen.addElement(new Long(rset.getLong("Class_ID")));
                klassennamen.put(new Long(rset.getLong("Class_ID")), rset.getString("Name"));
              }

              // Pro Klasse Accuracy, Precision und Recall berechnen; dabei Werte fuer
              // Micro- und Macrobewertung sammeln
              bewertung.clear();
              ordnung.clear();
              micromacro.clear();
              int accuracyMicroOben = 0;
              int accuracyMicroUnten = 0;
              int accuracyMacroOben = 0;
              int accuracyMacroUnten = 0;
              int precisionMicroOben = 0;
              int precisionMicroUnten = 0;
              int precisionMacroOben = 0;
              int precisionMacroUnten = 0;
              int recallMicroOben = 0;
              int recallMicroUnten = 0;
              int recallMacroOben = 0;
              int recallMacroUnten = 0;
              for (int i=0; i < klassen.size(); i++)
              {
                long aktKlassenId = ((Long) klassen.elementAt(i)).longValue();       

                //Dokumente die zu C klassifiziert wurden und zu C gehren
                query = "select count (*) from Feedback, BingoDocuments where class_old ='" + aktKlassenId + "' and result='1' ";
                query += "and Feedback.ID = BingoDocuments.ID ";
                if (urlSelect.length()>0)
                {
                    for (int j=0; j < urlarten.length; j++)
                    {
                        if (j==0) 
                            query+= "and ( ";
                        else 
                            query += "or ";
                        query += "BingoDocuments.status='"+urlarten[j]+"' ";
                    }
                    query += " ) ";
                }
                
                System.out.println (query);
                
                rset = stmt.executeQuery(query);
                rset.next();
                float a = (float) rset.getInt(1);

                //Dokumente die zu C klassifiziert wurden und nicht zu C gehren
                query = "select count (*) from Feedback, BingoDocuments where class_old ='" + aktKlassenId + "' and result!='1' ";
                query += "and Feedback.ID = BingoDocuments.ID ";
                if (urlSelect.length()>0)
                {
                    for (int j=0; j < urlarten.length; j++)
                    {
                        if (j==0) 
                            query+= "and ( ";
                        else 
                            query += "or ";
                        query += "BingoDocuments.status='"+urlarten[j]+"' ";
                    }
                    query += " ) ";
                }
                rset = stmt.executeQuery(query);
                rset.next();
                float b = rset.getInt(1);

                //Dokumente die nicht zu C klassifiziert wurden und zu C gehren
                query = "select count (*) from Feedback, BingoDocuments where class_new ='" + aktKlassenId + "' and result!='1' ";
                query += "and Feedback.ID = BingoDocuments.ID ";
                if (urlSelect.length()>0)
                {
                    for (int j=0; j < urlarten.length; j++)
                    {
                        if (j==0) 
                            query+= "and ( ";
                        else 
                            query += "or ";
                        query += "BingoDocuments.status='"+urlarten[j]+"' ";
                    }
                    query += " ) ";
                }
                rset = stmt.executeQuery(query);
                rset.next();
                float c = (float) rset.getInt(1);

                //Dokumente die nicht zu C klassifiziert wurden und nicht zu C gehren
                query = "select count (*) from Feedback, BingoDocuments where class_old !='" + aktKlassenId + "' ";
                query += "and class_new !='" + aktKlassenId + "' ";
                query += "and Feedback.ID = BingoDocuments.ID ";
                
                if (urlSelect.length()>0)
                {
                    for (int j=0; j < urlarten.length; j++)
                    {
                        if (j==0) 
                            query+= "and ( ";
                        else 
                            query += "or ";
                        query += "BingoDocuments.status='"+urlarten[j]+"' ";
                    }
                    query += " ) ";
                }
                rset = stmt.executeQuery(query);
                rset.next();
                float d = rset.getInt(1);

                // Pro Klasse Accuracy, Precision und Recall berechnen
                Hashtable details = new Hashtable();           
                details.put("accuracy", Float.toString((float)(Math.round(1000 * (a+d) / (a+b+c+d))) / 1000));
                int breite1 = Math.round( 500 * (a+d)/(a+b+c+d) );
                details.put("accuracyBreite1", new Integer (breite1));
                details.put("accuracyBreite2", new Integer (500-breite1));

                details.put("precision", Float.toString((float)(Math.round(1000 * a /(a+b))) / 1000));
                breite1 = Math.round( 500 * a/(a+b) );
                details.put("precisionBreite1", new Integer (breite1));
                details.put("precisionBreite2", new Integer (500-breite1));

                details.put("recall", Float.toString((float)(Math.round(1000 * a / (a+c))) / 1000));
                breite1 = Math.round( 500 * a /(a + c) );
                details.put("recallBreite1", new Integer (breite1));
                details.put("recallBreite2", new Integer (500-breite1));            

                ordnung.put (new Integer(i), klassennamen.get(new Long(aktKlassenId)));
                bewertung.put (klassennamen.get(new Long(aktKlassenId)), details);

                // Wert fuer Micro- und Macrobewertungen aktualisieren            
                accuracyMicroOben += a + d;
                accuracyMicroUnten += a + b + c + d;
                accuracyMacroOben += (a+d) / (a+b+c+d);
                accuracyMacroUnten += 1.0;

                precisionMicroOben += a;
                precisionMicroUnten += a + b;
                precisionMacroOben += (a / (a + b));
                precisionMacroUnten += 1.0;

                recallMicroOben += a;
                recallMicroUnten += a + c;
                recallMacroOben += (a / (a + c));
                recallMacroUnten += 1.0;

              }
              
              if (accuracyMicroOben <= 0)
                 accuracyMicroOben = 1;
              if (accuracyMicroUnten <= 0)
                 accuracyMicroUnten = 1;
              if (accuracyMacroOben <= 0)
                 accuracyMacroOben = 1;
              if (accuracyMacroUnten <= 0)
                 accuracyMacroUnten = 1;
              
              if (precisionMicroOben <= 0)
                 precisionMicroOben = 1;
              if (precisionMicroUnten <= 0)
                 precisionMicroUnten = 1;
              if (precisionMacroOben <= 0)
                 precisionMacroOben = 1;
              if (precisionMacroUnten <= 0)
                 precisionMacroUnten = 1;
              
              if (recallMicroOben <= 0)
                 recallMicroOben = 1;
              if (recallMicroUnten <= 0)
                 recallMicroUnten = 1;
              if (recallMacroOben <= 0)
                 recallMacroOben = 1;
              if (recallMacroUnten <= 0)
                 recallMacroUnten = 1;
              
              

              // Ergebnisse der Micro- und Macrobewertungen speichern              
              micromacro.put ("accuracyMicro",Float.toString((float)(Math.round(1000 * accuracyMicroOben/accuracyMicroUnten)) / 1000));
              int breite1 = Math.round( 500 * accuracyMicroOben/(accuracyMicroUnten+1) );
              micromacro.put ("accuracyMicroBreite1", new Integer (breite1));              
              micromacro.put ("accuracyMicroBreite2", new Integer (500 - breite1));

              micromacro.put ("accuracyMacro",Float.toString((float)(Math.round(1000 * accuracyMacroOben/accuracyMacroUnten)) / 1000));
              breite1 = Math.round( 500 * accuracyMacroOben/accuracyMacroUnten );
              micromacro.put ("accuracyMacroBreite1", new Integer (breite1));
              micromacro.put ("accuracyMacroBreite2", new Integer (500 - breite1));

              micromacro.put ("precisionMicro",Float.toString((float)(Math.round(1000 * precisionMicroOben/precisionMicroUnten)) / 1000));
              breite1 = Math.round( 500 * precisionMicroOben/precisionMicroUnten );
              micromacro.put ("precisionMicroBreite1", new Integer (breite1));
              micromacro.put ("precisionMicroBreite2", new Integer (500 - breite1));

              micromacro.put ("precisionMacro",Float.toString((float)(Math.round(1000 * precisionMacroOben/precisionMacroUnten)) / 1000));
              breite1 = Math.round( 500 * precisionMacroOben/precisionMacroUnten );
              micromacro.put ("precisionMacroBreite1", new Integer (breite1));
              micromacro.put ("precisionMacroBreite2", new Integer (500 - breite1));     

              micromacro.put ("recallMicro",Float.toString((float)(Math.round(1000 * recallMicroOben/recallMicroUnten)) / 1000));
              breite1 = Math.round( 500 * recallMicroOben/recallMicroUnten );
              micromacro.put ("recallMicroBreite1", new Integer (breite1));
              micromacro.put ("recallMicroBreite2", new Integer (500 - breite1));

              micromacro.put ("recallMacro",Float.toString((float)(Math.round(1000 * recallMacroOben/recallMacroUnten)) / 1000));
              breite1 = Math.round( 500 * recallMacroOben/recallMacroUnten );
              micromacro.put ("recallMacroBreite1", new Integer (breite1));
              micromacro.put ("recallMacroBreite2", new Integer (500 - breite1));
          }
      }
      catch (ArithmeticException e)
      {
        this.error = e; 
        this.feedbackVorhanden = false;
      }
          
      // DB-Verbindung schliessen
      stmt.close();
      conn.close();
    }
    catch (SQLException e)
    {
      this.error = e;
      
      this.hadSQLException = true;
      this.errorMessage = e.getMessage ();
    }
  }

  

  /**
   * Gibt an, ob bei DB-Abfrage Fehler aufgetreten sind.
   * @return false, wenn Abfrage fehlerfrei
   */
  public boolean getHadSQLException ()
  {
    return this.hadSQLException;
  }

  /**
   * Gibt (falls vorhanden) Fehlermeldung der DB-Abfrage aus.
   * @return Fehlermeldung der DB-Abfrage
   */
  public String getErrorMessage ()
  {
    if (errorMessage.length() != 0)
      return errorMessage;
    else
      return "No ErrorMessage available";
  }

  
  /**
   * Ausgabe der DB-Ergebnisse
   * @return Hashtable, die die Klassennamen mit einer Hashtable mappt, welche
   * die Werte fuer Accuracy, Precision und Recall dieser Klasse enthaelt
   */
  public Hashtable getBewertung()
  {
    return this.bewertung;
  }
  
  /**
   * Ausgabe der DB-Ergebnisse
   * @return Map, welche die Ergebnisse der Micro- und Macrobewertungen
   * enthaelt
   */
  public HashMap getMicroMacro()
  {
    return this.micromacro;
  }
  
  /**
   * Abfrage der DB-Ergebnisse
   * @ return HashMap, welche Index fuer alphabetische Reihenfolge mit 
   * Klassennamen mappt
   */
  public HashMap getOrdnung()
  {
    return this.ordnung;
  }
  
  /**
   * Ausgabe der DB-Ergebnisse
   * @return true, falls Feedback fuer Berechnungen vorhanden
   */
  public boolean getFeedbackVorhanden()
  {
    return this.feedbackVorhanden;
  } 
  
  /**
   * HashMap, welche Index fuer alphabetische Reihenfolge mit Klassennamen mappt
   */
  private HashMap ordnung = new HashMap();
  
  /**
   * Hashtable, die die Klassen mit einer Hashtable mappt, welcher
   * wiederum die Genauigkeitswerte fuer jede einzelne Iteration dieser
   * Klasse als Hashtables enthaelt
   */
  private Hashtable bewertung = new Hashtable();

  /**
   * Map, welche die Ergebnisse der Micro- und Macrobewertungen
   * enthaelt
   */
  private HashMap micromacro = new HashMap();
  
  private boolean feedbackVorhanden = false;
  
  private boolean hadSQLException = false;
  private String errorMessage = "";

}