package schema;

import java.util.*;
import java.sql.*;
import oracle.jdbc.driver.*;

/**
 *  Die Klasse dient als Zwischenspeicher fr diverse Objekte, auf die das
 *  System zur Laufzeit zugreift. Zudem werden systemweit gltige Konstanten
 *  definiert. Darber hinaus ist das Objekt zustndig fr den Aufbau und
 *  Abschluss der Oracle-Datenbankverbindung inklusive der Benutzer und
 *  Login-Verwaltung. Da zur Laufzeit nur genau eine Instanz dieses Objekts
 *  existieren darf, ist es als Singelton implementiert.
 */

public class SessionBuffer
{
  public static Hashtable stopWords;
  public static oracle.jdbc.driver.OracleDriver oracleDriver = null;
  
  //Systemweite Einstellungen
  
  public static int maxProcessingThreads = 20;
  public static int maxStorageThreads = 10;
  
  public static int batchSize = 100;
  public static int chunkSize = 512*1024;
  
  public static boolean dbConnect = false;
  
  public static String sHost;
  public static String sServiceName;
  public static String userDir = null;
  public static String dataDir = null;
  public static String defaultUser;
  public static String defaultUserPasswd;
  
  public static OracleConnection oracleConnection = null;
  
  // Singelton
  private static SessionBuffer sessionBuffer = null;
    
  protected SessionBuffer()
  {
    String s = System.getProperty( "file.separator" );
    if ( userDir == null )
      this.userDir = System.getProperty( "user.dir" );
    this.dataDir = userDir + s + "sources";
  }
  
  public boolean isStopword( String s )
  {
    return ( stopWords.containsKey(s) );
  }
  
  public Enumeration getStopwords()
  {
    return stopWords.elements();
  }
  
  // =====================================================================
  
  public static synchronized OracleConnection getOracleConnection()
  {
    OracleConnection oc = null;
    try
    {
      if ( oracleDriver == null )
      {
        oracleDriver = new oracle.jdbc.driver.OracleDriver();
        DriverManager.registerDriver( oracleDriver );
      }
      
      // DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      //Connection conn = DriverManager.getConnection("jdbc:oracle:oci8:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=134.96.246.200)(PORT=1521)))(CONNECT_DATA=(service_name=sydney.world)))","bingo", "00bingo00");
      try
      {
        oc = (OracleConnection)DriverManager.getConnection(
        "jdbc:oracle:thin:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=" + sHost + ")(PORT=1521)))(CONNECT_DATA=(SERVICE_NAME=" + sServiceName + ")(server = dedicated)))",
        defaultUser,
        defaultUserPasswd
        );
      }
      catch (Exception e)
      {
        
      }
      
      if (oc == null)  // thin versuchen
      {
        System.out.println("Native OCI-Connection failed, use Java Thin instead");
        oc = (OracleConnection)DriverManager.getConnection(
        "jdbc:oracle:thin:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=" + sHost + ")(PORT=1521)))(CONNECT_DATA=(SERVICE_NAME=" + sServiceName + ")(server = dedicated)))",
        defaultUser,
        defaultUserPasswd
        );
      }
      
      oc.setAutoCommit( false );
      //      SessionBuffer.log( "Neue DB-Verbindung: " + sServiceName + ":" + sHost + " aufgebaut." );
      dbConnect = true;
      
    } catch ( Exception q )
    {
      q.printStackTrace();
      dbConnect = false;
    }
    return oc;
  }
  
/*  
  public static synchronized OracleConnection getAdminOracleConnection()
  {
    OracleConnection oc = null;
    try
    {
      if ( oracleDriver == null )
      {
        oracleDriver = new oracle.jdbc.driver.OracleDriver();
        DriverManager.registerDriver( oracleDriver );
      }
      
      Properties prop = new Properties();
      prop.put("internal_logon","sysdba");
      
      String url = "jdbc:oracle:thin:";
      String dbString = "(DESCRIPTION = (ADDRESS_LIST = (ADDRESS = "
      + "(PROTOCOL = TCP)(HOST = " + sHost + ")(PORT = 1521)))"
      + "(CONNECT_DATA = (SERVICE_NAME = " + sServiceName + " )(server = dedicated))))";
      
      oc = (OracleConnection)DriverManager.getConnection(url + "@" + dbString, prop);
      oc.setAutoCommit(true);
      
      if (oc != null)
        oc.setAutoCommit( false );
      
    } catch ( SQLException q )
    {
      q.printStackTrace();
      dbConnect = false;
    }
    return oc;
  }
*/  
  
  /**
   *  Methode zum Aufruf der (einzigen) Instanz der Klasse. Es wird keine
   *  Datenbankverbindung aufgebaut.
   *
   *@return    Die Instanz.
   */
  public static synchronized SessionBuffer getInstance()
  {
    if ( sessionBuffer == null )
    {
      sessionBuffer = new SessionBuffer();
    }
    return sessionBuffer;
  }
  
  
  /**
   *  Methode zum Aufruf der (einzigen) Instanz der Klasse. Es wird eine
   *  Datenbankverbindung zu dem spezififizierten Oracle-Host aufgebaut. Dieser
   *  Verbindungsaufbau ist nur einmal zur Laufzeit erlaubt.
   *
   *@param  host         Der Host.
   *@param  serviceName  Der ServiceName.
   *@return              Die Instanz.
   */
  public static SessionBuffer getInstance( String user, String pass, String host, String serviceName )
  {
    defaultUser = user;
    defaultUserPasswd = pass;
    sHost = host;
    sServiceName = serviceName;
    
    if ( sessionBuffer == null )
    {
      sessionBuffer = new SessionBuffer();
    }
    //    if ( !dbConnect ) {
    //      oracleConnection = getOracleConnection();
    //      if ( oracleConnection == null ) {
    //        System.exit( 0 );
    //      }
    //      dbinterface = new myDBInterface();
    //    }
    return sessionBuffer;
  }
  
  
  /**
   *  Methode zum Aufruf der (einzigen) Instanz der Klasse. Es wird angegeben,
   *  ob eine Datenbankverbindung zu einem vorher spezififizierten Oracle-Host
   *  aufgebaut werden soll. Dieser Verbindungsaufbau ist nur einmal zur
   *  Laufzeit erlaubt.
   */
  public static SessionBuffer getInstance( boolean connect )
  {
    if ( !dbConnect && connect )
    { 
      LoginDialog lDialog = new LoginDialog();
      if ( lDialog.isOK() )
      {
        defaultUser = lDialog.getUser();
        defaultUserPasswd = lDialog.getPassword();        
        sHost = lDialog.getHost();
        sServiceName = lDialog.getSID();
        
        lDialog = null;
      }
      else
      {
        System.exit(0);
      }
    }
    
    if ( sessionBuffer == null )
    {
      sessionBuffer = new SessionBuffer();
    }
    return sessionBuffer;
  }
  
  public static int getBatchSize()
  {
    return batchSize;
  }
  
  public static int getChunkSize()
  {
    return chunkSize;
  }
}