drop table seev_feature;

drop table seev_origin;

drop table seev_documents CASCADE CONSTRAINTS;

drop table seev_temp_results;

drop table seev_history_temp_user;

drop table seev_temp_user;

commit;

create table SEEV_DOCUMENTS
(
  STATUS    CHAR(1),
  HUBSCORE  FLOAT default 0.0001,
  AUTHSCORE FLOAT default 0.0001,
  ID        NUMBER not null,
  URL       VARCHAR2(3000) not null,
  HOSTNAME  VARCHAR2(255),
  LASTDATE  DATE default sysdate,
  TITLE     VARCHAR2(255),
  CLASSNAME VARCHAR2(500) default 'UNKNOWN/',
  TEXT      VARCHAR2(4000) default 'not available',
  DOCSIZE   FLOAT default 0,
  PRIMARY KEY (ID)
);

CREATE TABLE SEEV_FEATURE(
    Term               varchar2(200)  NOT NULL,
    TF                 number         DEFAULT 0 ,
    TF_IDF             number         DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_ORIGIN(
    Term               varchar2(200)  NOT NULL,
    TF                 number         DEFAULT 0 ,
    TF_IDF             number         DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

create table SEEV_history_temp_user
(
 host varchar2(500) not null,
 query varchar2(500) not null,
 time number not null
);

create table SEEV_temp_user
(
 host varchar2(500) not null,
 sessionID integer not null,
 query varchar2(500) not null,
 time number not null,
 PRIMARY KEY (sessionID) 
);

create table SEEV_temp_results
(
 sessionid integer not null,
 ID integer NOT NULL,
 FOREIGN KEY (SESSIONID) REFERENCES SEEV_temp_user
  ON DELETE CASCADE,
 FOREIGN KEY (ID) REFERENCES SEEV_Documents
  ON DELETE CASCADE
);

commit;

insert into SEEV_documents
select status, hubscore+0.000001 as hubscore, authscore+0.000001 as authscore, id, url, hostname, lastdate, title, lower(classname), text, docsize from
  (select T.*, name as classname from
   (select status, hubscore, authscore, AD.id as ID, url, hostname, lastmod as lastdate, class_id, AF.TITLE as title, preview as text, dbms_lob.getlength(source) as docsize from
     archivedocuments AD, 
     crawldocuments CD,
     archivefiles AF
    where AD.ID=CD.ID and AD.ID=AF.ID) T 
    left join 
    classes
    on T.class_id = classes.class_id
  );

commit;

---FEATURE

--- computing of tfidf scores

insert into SEEV_FEATURE
(
select T.* from 
(
 select T1.term as term, tf, tf*idf as tf_idf, id from
 (
   select term, log(2, (select count(distinct(ID)) as N from terms)/count(distinct(id))) as IDF from 
          DOCUMENTFEATURES
   group by term
 ) T1
 ,
 ( 
   select term, tf/maxtf as TF, T1.id as ID from 
         DOCUMENTFEATURES T1,
         (select id, max(tf) as maxtf from DOCUMENTFEATURES group by id) T2
   where T1.id = T2.id
 ) T2
 where t1.term=t2.term
) T,
SEEV_DOCUMENTS D
where D.id=T.id
);

commit;


---ORIGIN

drop table SEEV_TEMP;

commit;

create table SEEV_TEMP
(
id   number not null,
term varchar2(200) not null,
TF    number not null
); 

commit;

insert into SEEV_TEMP
(
 select id, term, count(*) as TF from terms group by id, term
);

commit;

create index SEEV_TEMP_ID on SEEV_TEMP(id);

commit;

--- computing of tfidf scores

insert into SEEV_ORIGIN
(
select T.* from 
(
 select T1.term as term, tf, tf*idf as tf_idf, id from
 (
   select term, log(2, (select count(distinct(ID)) as N from terms)/count(distinct(id))) as IDF from 
          terms
   group by term
 ) T1
 ,
 ( 
   select term, tf/maxtf as TF, T1.id as ID from 
         SEEV_TEMP T1,
         (select id, max(tf) as maxtf from SEEV_TEMP group by id) T2
   where T1.id = T2.id
 ) T2
 where t1.term=t2.term
) T,
SEEV_DOCUMENTS D
where D.id=T.id
);

commit;

drop table SEEV_TEMP;

commit;

create index SEEV_FEATURE_TFIDF on SEEV_FEATURE (ID, TERM);

create index SEEV_ORIGIN_TFIDF on SEEV_ORIGIN (ID, TERM);

create index SEEV_FEATURE_KEY on SEEV_FEATURE (term);

create index SEEV_ORIGIN_KEY on SEEV_ORIGIN (term);

create index SEEV_DOC_HOSTNAME on SEEV_DOCUMENTS (HOSTNAME);

commit;


