drop table seev_documents CASCADE CONSTRAINTS;

drop table SEEV_FEEDBACK CASCADE CONSTRAINTS;

drop table SEEV_PROFILE CASCADE CONSTRAINTS;

drop table seev_temp_user CASCADE CONSTRAINTS;

drop table seev_history_temp_user;

drop table seev_classes;

drop table SEEV_FEEDBACK_DOCS;

drop table SEEV_DISTINCT_PROFILE;

drop table SEEV_GLOBAL_PROFILE;

drop table SEEV_USED_PROFILE;

drop table SEEV_FEATURE_CLASS_0;

drop table SEEV_FEATURE_CLASS_0;

drop table SEEV_FEATURE_CLASS_1;

drop table SEEV_FEATURE_CLASS_2;

drop table SEEV_FEATURE_CLASS_3;

drop table SEEV_FEATURE_CLASS_4;

drop table SEEV_FEATURE_CLASS_temp 

commit;

create table SEEV_DOCUMENTS
(
  STATUS    CHAR(1),
  HUBSCORE  FLOAT default 0.0001,
  AUTHSCORE FLOAT default 0.0001,
  DOCID     NUMBER not null,
  URL       VARCHAR2(3000) not null,
  HOSTNAME  VARCHAR2(255),
  LASTDATE  DATE default sysdate,
  TITLE     VARCHAR2(255),
  CLASSNAME VARCHAR2(500) default 'UNKNOWN/',
  TEXT      VARCHAR2(4000) default 'not available',
  DOCSIZE   FLOAT default 0,
  PRIMARY KEY (DOCID)
);

create table SEEV_CLASSES
(
  CLASS_ID NUMBER not null,
  NAME     VARCHAR2(500) not null
);

create table SEEV_history_temp_user
(
 userId      NUMBER not null,
 hostname    varchar2(500) not null,
 query       BLOB not null,
 time        number not null
);

create table SEEV_temp_user
(
 SESSIONID   NUMBER not null,
 userId      NUMBER not null,
 hostname    varchar2(500) not null,
 query       BLOB not null,
 time        number not null,
 PRIMARY KEY (SESSIONID) 
);

create table SEEV_FEEDBACK
(
  FEEDBACKID        NUMBER not null,
  USERID            NUMBER default -1,
  QUERY             BLOB not null,    
  LASTDATE          DATE default sysdate,
  primary key (FEEDBACKID)
);

create table SEEV_PROFILE
(
  PROFILEID NUMBER not null,
  w0 FLOAT default 1.0,
  w1 FLOAT default 1.0,
  w2 FLOAT default 1.0,
  w3 FLOAT default 1.0,
  LASTDATE Date default sysdate,
  PRIMARY KEY (PROFILEID)
);


create table SEEV_FEEDBACK_DOCS
(
 FEEDBACKID   NUMBER NOT NULL,
 DOCID        NUMBER NOT NULL,
 FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK
   ON DELETE CASCADE,
 FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
   ON DELETE CASCADE,
 STATUS  NUMBER not null,
 primary key (FEEDBACKID, DOCID)
);

create table SEEV_DISTINCT_PROFILE
(
  PROFILEID  NUMBER not null,
  FEEDBACKID NUMBER not null,
  FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK 
   ON DELETE CASCADE,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_FEEDBACK         
   ON DELETE CASCADE,
  primary key(PROFILEID, FEEDBACKID)
);

create table SEEV_GLOBAL_PROFILE
(
  PROFILEID  NUMBER not null,
  FEEDBACKID NUMBER not null unique,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_FEEDBACK         
   ON DELETE CASCADE
);

create table SEEV_USED_PROFILE
(
  PROFILEID  NUMBER not null,
  FEEDBACKID NUMBER not null unique,
  FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK 
   ON DELETE CASCADE,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_FEEDBACK         
   ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_0(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0.0 ,
    DOCID             NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_1(
    KEY               varchar2(200)  default ' ',
    VALUE             number         DEFAULT 0.0 ,
    DOCID              NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_2(
    KEY               varchar2(200)  DEFAULT ' ',
    VALUE             number         DEFAULT 0.0 ,
    DOCID              NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_3(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_4(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

commit;

insert into SEEV_documents
select status, hubscore+0.000001 as hubscore, authscore+0.000001 as authscore, id, url, hostname, lastdate, title, lower(classname), text, docsize from
  (select T.*, name as classname from
   (select status, hubscore, authscore, AD.id as ID, url, hostname, lastmod as lastdate, class_id, AF.TITLE as title, preview as text, dbms_lob.getlength(source) as docsize from
     archivedocuments AD, 
     crawldocuments CD,
     archivefiles AF
    where AD.ID=CD.ID and AD.ID=AF.ID) T 
    left join 
    classes
    on T.class_id = classes.class_id
  );

commit;

insert into SEEV_CLASSES
select * from CLASSES;

---FEATURE_CLASSES

--- computing of tfidf scores

CREATE TABLE SEEV_FEATURE_CLASS_temp
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID             integer        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

insert into SEEV_FEATURE_CLASS_temp
(
select T.* from 
(
 select T1.term as KEY, tf*idf as VALUE, id as DOCID from
 (
   select term, log(2, (select count(distinct(ID)) as N from terms)/count(distinct(ID))) as IDF from 
          DOCUMENTFEATURES
   group by term
 ) T1
 ,
 ( 
   select term, tf/maxtf as TF, T1.id as ID from 
         DOCUMENTFEATURES T1,
         (select id, max(tf) as maxtf from DOCUMENTFEATURES group by id) T2
   where T1.id = T2.id
 ) T2
 where t1.term=t2.term
) T,
SEEV_DOCUMENTS D
where D.DOCID=T.DOCID
);

commit;

insert into SEEV_FEATURE_CLASS_0
select key, value/max_value, docid from SEEV_FEATURE_CLASS_temp, (select max(value) as max_value from SEEV_FEATURE_CLASS_temp) 

commit;


--- authority score

insert into SEEV_FEATURE_CLASS_1
select ' ', AUTHSCORE/max_score as value, DOCID from SEEV_DOCUMENTS, (select max(authscore) as max_score from Seev_documents)

--- classes with SVM score

insert into SEEV_FEATURE_CLASS_2
select classname, c.svmscore/max_value as value, docid from seev_documents s, crawldocuments c, (select max(svmscore) max_value from crawldocuments)
where s.docid = c.id and c.svmscore > 0.0001

commit;

insert into SEEV_FEATURE_CLASS_2
select classname, 0.0001, docid from seev_documents s, crawldocuments c, (select max(svmscore) max_value from crawldocuments)
where s.docid = c.id and c.svmscore <= 0.0001

commit;

--- date score
insert into SEEV_FEATURE_CLASS_3
select TO_CHAR(lastdate) as key, MONTHS_BETWEEN(lastdate, MN)/MONTHS_BETWEEN(Mx,Mn) as value, docid from 
  seev_documents, 
  (select max(lastdate) as MX, min(lastdate) as MN from seev_documents)

commit;

create index SEEV_FEATURE_CLASS_0_key on SEEV_FEATURE_CLASS_0 (key);

create index SEEV_FEATURE_CLASS_1_key on SEEV_FEATURE_CLASS_1 (key);

create index SEEV_FEATURE_CLASS_2_key on SEEV_FEATURE_CLASS_2 (key);

create index SEEV_FEATURE_CLASS_3_key on SEEV_FEATURE_CLASS_3 (key);


commit;


