drop table SEEV_DOCUMENTS CASCADE CONSTRAINTS;

drop table SEEV_FEEDBACK CASCADE CONSTRAINTS;

drop table SEEV_PROFILE CASCADE CONSTRAINTS;

drop table seev_temp_user CASCADE CONSTRAINTS;

drop table seev_users;

drop table seev_history_temp_user;

drop table seev_classes;

drop table SEEV_FEEDBACK_DOCS;

drop table SEEV_DISTINCT_PROFILE;

drop table SEEV_GLOBAL_PROFILE;

drop table SEEV_EVALUATION;

drop table SEEV_USED_PROFILE;

drop table SEEV_FEATURE_CLASS_0;

drop table SEEV_FEATURE_CLASS_1;

drop table SEEV_FEATURE_CLASS_2;

drop table SEEV_FEATURE_CLASS_3;

drop table SEEV_FEATURE_CLASS_4;

drop table SEEV_FEATURE_CLASS_5;

drop table SEEV_FEATURE_CLASS_temp;

drop table SEEV_FEEDBACK_SESSION; 

commit;

create table SEEV_DOCUMENTS
(
  STATUS    CHAR(1),
  HUBSCORE  FLOAT default 0.0001,
  AUTHSCORE FLOAT default 0.0001,
  DOCID     NUMBER not null,
  URL       VARCHAR2(3000) not null,
  HOSTNAME  VARCHAR2(255),
  LASTDATE  DATE default sysdate,
  TITLE     VARCHAR2(255),
  CLASSNAME VARCHAR2(500) default 'UNKNOWN/',
  TEXT      VARCHAR2(4000) default 'not available',
  DOCSIZE   FLOAT default 0,
  PRIMARY KEY (DOCID)
);

create table SEEV_CLASSES
(
  CLASS_ID NUMBER not null,
  NAME     VARCHAR2(500) not null
);

create table SEEV_USERS
(
 userId      NUMBER not null,
 name        varchar2(255),
 email       varchar2(255) not null,
 login       varchar2(50) not null unique,
 pass        number not null,
 regdate     date,     
 primary key (userId)
);

create table SEEV_HISTORY_TEMP_USER
(
 userId      NUMBER not null,
 hostname    varchar2(500) not null,
 query       BLOB not null,
 time        number not null
);

create table SEEV_TEMP_USER
(
 SESSIONID   NUMBER not null,
 userId      NUMBER not null,
 hostname    varchar2(500) not null,
 query       BLOB not null,
 time        number not null,
 PRIMARY KEY (SESSIONID) 
);

create table SEEV_FEEDBACK
(
  FEEDBACKID        NUMBER not null,
  USERID            NUMBER default -1,
  QUERY             BLOB not null,    
  LASTDATE          DATE default sysdate,
  primary key (FEEDBACKID)
);

create table SEEV_PROFILE
(
  PROFILEID NUMBER not null,
  w0 FLOAT default 1.0,
  w1 FLOAT default 1.0,
  w2 FLOAT default 1.0,
  w3 FLOAT default 1.0,
  w4 FLOAT default 1.0,
  w5 FLOAT default 1.0,
  LASTDATE Date default sysdate,
  PRIMARY KEY (PROFILEID)
);


create table SEEV_FEEDBACK_DOCS
(
 FEEDBACKID   NUMBER NOT NULL,
 DOCID        NUMBER NOT NULL,
 FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK
   ON DELETE CASCADE,
 FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
   ON DELETE CASCADE,
 STATUS  NUMBER not null,
 primary key (FEEDBACKID, DOCID)
);

drop table SEEV_FEEDBACK_SESSION;

create table SEEV_FEEDBACK_SESSION
(
 FEEDBACKID   NUMBER NOT NULL,
 SESSIONID    NUMBER NOT NULL,
 FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK
   ON DELETE CASCADE,
 FOREIGN KEY (SESSIONID) REFERENCES SEEV_TEMP_USER
   ON DELETE CASCADE,
 primary key (FEEDBACKID, SESSIONID)
);

create table SEEV_DISTINCT_PROFILE
(
  PROFILEID  NUMBER not null,
  FEEDBACKID NUMBER not null,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE,
  FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK 
   ON DELETE CASCADE,
  primary key(PROFILEID, FEEDBACKID)
);

create table SEEV_GLOBAL_PROFILE
(
  PROFILEID    NUMBER not null,
  DISPROFILEID NUMBER not null,
  score        number,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE
);

create table SEEV_USED_PROFILE
(
  PROFILEID  NUMBER not null,
  SESSIONID NUMBER not null unique,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE,
  FOREIGN KEY (SESSIONID) REFERENCES SEEV_TEMP_USER 
   ON DELETE CASCADE
);

create table SEEV_EVALUATION
(
 feedbackid NUMBER not null,
 FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK 
   ON DELETE CASCADE 
);

CREATE TABLE SEEV_FEATURE_CLASS_0(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0.0 ,
    DOCID             NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_1(
    KEY               varchar2(200)  default ' ',
    VALUE             number         DEFAULT 0.0 ,
    DOCID              NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_2(
    KEY               varchar2(200)  DEFAULT ' ',
    VALUE             number         DEFAULT 0.0 ,
    DOCID              NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_3(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_4(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_5(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_DOCUMENTS
      ON DELETE CASCADE
);

drop SEEV_FEEDBACK_BLOB;

create table SEEV_FEEDBACK_BLOB
(
 sessionID      NUMBER not null,
 feedback       BLOB not null
);

drop SEEV_GLOBAL_PROFILE_BLOB;

create table SEEV_GLOBAL_PROFILE_BLOB
(
 profileID      NUMBER not null,
 profile       BLOB not null,
 FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE
);


commit;

---date filtering

delete from archivedocuments
where lastmod > sysdate or to_char(lastmod, 'YYYY') < '1986'; 

---create document table

insert into SEEV_DOCUMENTS
select status, hubscore, authscore, id, url, hostname, lastdate, title, lower(name) as classname, text, docsize from
 (select status, hubscore+0.00000001 as hubscore, pagerank+0.00000001 as authscore, AD.id as ID, url, hostname, lastmod as lastdate, class_id, AF.TITLE as title, preview as text, dbms_lob.getlength(source) as docsize from
     archivedocuments AD, 
     crawldocuments CD,
     archivefiles AF
    where AD.ID=CD.ID and AD.ID=AF.ID and CD.ID = AF.ID
 ) T 
left join 
 classes
on T.class_id = classes.class_id
;

commit;

insert into SEEV_CLASSES
select * from CLASSES;

---FEATURE_CLASSES

--- computing of tfidf scores

CREATE TABLE SEEV_FEATURE_CLASS_temp
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0,
    DOCID             number        NOT NULL
);

drop table SEEV_FEATURE_CLASS_TEMP_MAXTF;

drop table SEEV_FEATURE_CLASS_TEMP_IDF;

drop table SEEV_FEATURE_CLASS_TEMP_TFIDF;

CREATE TABLE SEEV_FEATURE_CLASS_TEMP_TFIDF
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID             number        NOT NULL
);

CREATE TABLE SEEV_FEATURE_CLASS_TEMP_IDF
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0
);

CREATE TABLE SEEV_FEATURE_CLASS_TEMP_MAXTF
(
    VALUE             number         DEFAULT 0,
    DOCID             number        NOT NULL
);

---insert into SEEV_FEATURE_CLASS_TEMP_TFIDF
---select term as key, count(term) as value, id as docid from terms
---group by id, term;

insert into SEEV_FEATURE_CLASS_TEMP_TFIDF
select term as key, tf as value, id as docid from DOCUMENTFEATURES;

create index terms_key on SEEV_FEATURE_CLASS_TEMP_TFIDF (key);

create index terms_docid on SEEV_FEATURE_CLASS_TEMP_TFIDF (docid);

insert into SEEV_FEATURE_CLASS_TEMP_IDF (key, value)
select key, count(distinct(docid)) as value from  SEEV_FEATURE_CLASS_TEMP_TFIDF
group by key;

insert into SEEV_FEATURE_CLASS_TEMP_MAXTF (value, docid)
select sum(value) as value, docid from SEEV_FEATURE_CLASS_TEMP_TFIDF 
group by docid;

commit;

create index seev__temp_idf_term on SEEV_FEATURE_CLASS_TEMP_IDF (key);

create index seev_temp_idf_maxtf on SEEV_FEATURE_CLASS_TEMP_MAXTF (docid);

commit;

drop table SEEV_FEATURE_CLASS_TEMP_3;

CREATE TABLE SEEV_FEATURE_CLASS_TEMP_3
(
  N number DEFAULT 1
);

insert into SEEV_FEATURE_CLASS_TEMP_3 
select count(docid) from SEEV_DOCUMENTS

insert into seev_feature_class_temp (key, value, docid)
select /*+ ALL_ROWS */ T1.key, log(2, N/T2.value)*T1.value/T3.value, T1.docid from 
SEEV_DOCUMENTS                 T0, 
SEEV_FEATURE_CLASS_TEMP_TFIDF  T1, 
SEEV_FEATURE_CLASS_TEMP_IDF    T2, 
SEEV_FEATURE_CLASS_TEMP_MAXTF  T3, 
SEEV_FEATURE_CLASS_TEMP_3
where T0.docid = T1.docid and 
      T1.docid = T3.docid and 
      T0.docid = T3.docid and 
      T1.key = T2.key;

commit;

drop table SEEV_FEATURE_CLASS_TEMP_TFIDF;

drop table SEEV_FEATURE_CLASS_TEMP_IDF;

drop table SEEV_FEATURE_CLASS_TEMP_MAXTF;

drop table SEEV_FEATURE_CLASS_TEMP_3;

drop table SSEEV_FEATURE_CLASS_TEMP_2;

CREATE TABLE SEEV_FEATURE_CLASS_TEMP_2
(
    MAX_VALUE number DEFAULT 1
);

insert into SEEV_FEATURE_CLASS_TEMP_2
select max(value) as MAX_VALUE from SEEV_FEATURE_CLASS_temp;

commit;

insert into SEEV_FEATURE_CLASS_0
select key, value/max_value, docid from SEEV_FEATURE_CLASS_temp, SEEV_FEATURE_CLASS_TEMP_2   

commit;

drop table SEEV_FEATURE_CLASS_temp;

drop table SEEV_FEATURE_CLASS_temp_2;

--- authscore score

insert into SEEV_FEATURE_CLASS_1
select ' ', authscore/max_score as value, DOCID from SEEV_DOCUMENTS, (select max(authscore) as max_score from Seev_documents)

--- date score
insert into SEEV_FEATURE_CLASS_2
select to_char(lastdate, 'YYYY-MM-DDFMFX') as key, (MONTHS_BETWEEN(lastdate, MN)+0.0001) / (MONTHS_BETWEEN(MX,MN)+0.0001) as value, docid 
from 
 seev_documents,
 (select max(lastdate) as MX, min(lastdate) as MN from seev_documents)
;

commit;

--- old date score

insert into SEEV_FEATURE_CLASS_3
select to_char(lastdate, 'YYYY-MM-DDFMFX') as key, 1 - (MONTHS_BETWEEN(lastdate, MN)+0.0001) / (MONTHS_BETWEEN(MX,MN)+0.0001) as value, docid 
from 
 seev_documents,
 (select max(lastdate) as MX, min(lastdate) as MN from seev_documents)
;

--- classes with SVM score

--- class "Science, Education"
insert into SEEV_FEATURE_CLASS_4
select classname, c.svmscore/max_value as value, docid from seev_documents s, crawldocuments c, (select max(svmscore) max_value from crawldocuments)
where c.class_id = -330651214 and s.docid = c.id and c.svmscore > 0.0001 

commit;

insert into SEEV_FEATURE_CLASS_4
select classname, 0.0001, docid from seev_documents s, crawldocuments c
where s.docid = c.id and (c.class_id != -330651214 or c.svmscore < 0.0001)

commit;

--- class "Entertainment"
insert into SEEV_FEATURE_CLASS_5
select classname, c.svmscore/max_value as value, docid from seev_documents s, crawldocuments c, (select max(svmscore) max_value from crawldocuments)
where c.class_id = 1364338324 and s.docid = c.id and c.svmscore > 0.0001 

commit;

insert into SEEV_FEATURE_CLASS_5
select classname, 0.0001, docid from seev_documents s, crawldocuments c
where s.docid = c.id and (c.class_id != 1364338324 or c.svmscore < 0.0001)

commit;

-- index creation

create index SEEV_FEATURE_CLASS_0_key on SEEV_FEATURE_CLASS_0 (key);

create index SEEV_FEATURE_CLASS_0_docid on SEEV_FEATURE_CLASS_0 (docid);

create index SEEV_FEATURE_CLASS_0_key_id on SEEV_FEATURE_CLASS_0 (key, docid);


create index SEEV_FEATURE_CLASS_1_key on SEEV_FEATURE_CLASS_1 (key);

create index SEEV_FEATURE_CLASS_1_docid on SEEV_FEATURE_CLASS_1 (docid);

create index SEEV_FEATURE_CLASS_1_key_id on SEEV_FEATURE_CLASS_1 (key, docid);


create index SEEV_FEATURE_CLASS_2_key on SEEV_FEATURE_CLASS_2 (key);

create index SEEV_FEATURE_CLASS_2_docid on SEEV_FEATURE_CLASS_2 (docid);

create index SEEV_FEATURE_CLASS_2_key_id on SEEV_FEATURE_CLASS_2 (key, docid);


create index SEEV_FEATURE_CLASS_3_key on SEEV_FEATURE_CLASS_3 (key);

create index SEEV_FEATURE_CLASS_3_docid on SEEV_FEATURE_CLASS_3 (docid);

create index SEEV_FEATURE_CLASS_3_key_id on SEEV_FEATURE_CLASS_3 (key, docid);


create index SEEV_FEATURE_CLASS_4_key on SEEV_FEATURE_CLASS_4 (key);

create index SEEV_FEATURE_CLASS_4_docid on SEEV_FEATURE_CLASS_4 (docid);

create index SEEV_FEATURE_CLASS_4_key_id on SEEV_FEATURE_CLASS_4 (key, docid);


create index SEEV_FEATURE_CLASS_5_key on SEEV_FEATURE_CLASS_5 (key);

create index SEEV_FEATURE_CLASS_5_docid on SEEV_FEATURE_CLASS_5 (docid);

create index SEEV_FEATURE_CLASS_5_key_id on SEEV_FEATURE_CLASS_5 (key, docid);


commit;


