drop table seev_documents CASCADE CONSTRAINTS;

drop table SEEV_FEEDBACK CASCADE CONSTRAINTS;

drop table SEEV_PROFILE CASCADE CONSTRAINTS;

drop table seev_temp_user CASCADE CONSTRAINTS;

drop table seev_users;

drop table seev_history_temp_user;

drop table seev_classes;

drop table SEEV_FEEDBACK_DOCS;

drop table SEEV_DISTINCT_PROFILE;

drop table SEEV_GLOBAL_PROFILE;

drop table SEEV_USED_PROFILE;

drop table SEEV_FEATURE_CLASS_0;

drop table SEEV_FEATURE_CLASS_1;

drop table SEEV_FEATURE_CLASS_2;

drop table SEEV_FEATURE_CLASS_3;

drop table SEEV_FEATURE_CLASS_temp 

commit;

create table SEEV_DOCUMENTS
(
  STATUS    CHAR(1),
  HUBSCORE  FLOAT default 0.0001,
  AUTHSCORE FLOAT default 0.0001,
  DOCID     NUMBER not null,
  URL       VARCHAR2(3000) not null,
  HOSTNAME  VARCHAR2(255),
  LASTDATE  DATE default sysdate,
  TITLE     VARCHAR2(255),
  CLASSNAME VARCHAR2(500) default 'UNKNOWN/',
  TEXT      VARCHAR2(4000) default 'not available',
  DOCSIZE   FLOAT default 0,
  PRIMARY KEY (DOCID)
);

create table SEEV_CLASSES
(
  CLASS_ID NUMBER not null,
  NAME     VARCHAR2(500) not null
);

create table SEEV_users
(
 userId      NUMBER not null,
 name        varchar2(255),
 email       varchar2(255) not null unique,
 login       varchar2(50) not null unique,
 pass        number not null,
 regdate     date,     
 primary key (userId)
);

create table SEEV_history_temp_user
(
 userId      NUMBER not null,
 hostname    varchar2(500) not null,
 query       BLOB not null,
 time        number not null
);

create table SEEV_temp_user
(
 SESSIONID   NUMBER not null,
 userId      NUMBER not null,
 hostname    varchar2(500) not null,
 query       BLOB not null,
 time        number not null,
 PRIMARY KEY (SESSIONID) 
);

create table SEEV_FEEDBACK
(
  FEEDBACKID        NUMBER not null,
  USERID            NUMBER default -1,
  QUERY             BLOB not null,    
  LASTDATE          DATE default sysdate,
  primary key (FEEDBACKID)
);

create table SEEV_PROFILE
(
  PROFILEID NUMBER not null,
  w0 FLOAT default 1.0,
  w1 FLOAT default 1.0,
  w2 FLOAT default 1.0,
  w3 FLOAT default 1.0,
  w4 FLOAT default 1.0,
  LASTDATE Date default sysdate,
  PRIMARY KEY (PROFILEID)
);


create table SEEV_FEEDBACK_DOCS
(
 FEEDBACKID   NUMBER NOT NULL,
 DOCID        NUMBER NOT NULL,
 FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK
   ON DELETE CASCADE,
 FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
   ON DELETE CASCADE,
 STATUS  NUMBER not null,
 primary key (FEEDBACKID, DOCID)
);

create table SEEV_DISTINCT_PROFILE
(
  PROFILEID  NUMBER not null,
  FEEDBACKID NUMBER not null,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE,
  FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK 
   ON DELETE CASCADE,
  primary key(PROFILEID, FEEDBACKID)
);

create table SEEV_GLOBAL_PROFILE
(
  PROFILEID    NUMBER not null,
  DISPROFILEID NUMBER not null,
  score        number,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE
);

create table SEEV_USED_PROFILE
(
  PROFILEID  NUMBER not null,
  FEEDBACKID NUMBER not null unique,
  FOREIGN KEY (PROFILEID) REFERENCES SEEV_PROFILE         
   ON DELETE CASCADE,
  FOREIGN KEY (FEEDBACKID) REFERENCES SEEV_FEEDBACK 
   ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_0(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0.0 ,
    DOCID             NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_1(
    KEY               varchar2(200)  default ' ',
    VALUE             number         DEFAULT 0.0 ,
    DOCID              NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_2(
    KEY               varchar2(200)  DEFAULT ' ',
    VALUE             number         DEFAULT 0.0 ,
    DOCID              NUMBER        NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

CREATE TABLE SEEV_FEATURE_CLASS_3(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

commit;

insert into SEEV_documents
select status, hubscore, authscore, id, url, hostname, lastdate, title, lower(name) as classname, text, docsize from
 (select status, hubscore+0.000001 as hubscore, authscore+0.000001 as authscore, AD.id as ID, url, hostname, lastmod as lastdate, class_id, AF.TITLE as title, preview as text, dbms_lob.getlength(source) as docsize from
     archivedocuments AD, 
     crawldocuments CD,
     archivefiles AF
    where AD.ID=CD.ID and AD.ID=AF.ID and CD.ID = AF.ID
 ) T 
left join 
 classes
on T.class_id = classes.class_id
;

commit;

create index seev_documents_docid on seev_documents(docid);

commit;

insert into SEEV_CLASSES
select * from CLASSES;

---FEATURE_CLASSES

--- computing of tfidf scores

CREATE TABLE SEEV_FEATURE_CLASS_temp
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID             integer        NOT NULL,
);

drop table seev_feature_class_temp_tfidf;

drop table seev_feature_class_temp_idf;

drop table seev_feature_class_temp_maxtf;

CREATE TABLE SEEV_FEATURE_CLASS_temp_tfidf
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID             integer        NOT NULL
);

CREATE TABLE SEEV_FEATURE_CLASS_temp_idf
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0
);

CREATE TABLE SEEV_FEATURE_CLASS_temp_maxtf
(
    VALUE             number         DEFAULT 0 ,
    DOCID             integer        NOT NULL
);

insert into seev_feature_class_temp_tfidf
select term as key, count(term) as value, id as docid from terms
group by id, term;

create index terms_key on seev_feature_class_temp_tfidf (key);

create index terms_docid on seev_feature_class_temp_tfidf (docid);

insert into seev_feature_class_temp_idf (key, value)
select key, count(distinct(docid)) as value from  SEEV_FEATURE_CLASS_tfidf
group by key;

insert into seev_feature_class_temp_maxtf (value, docid)
select sum(value) as value, docid from SEEV_FEATURE_CLASS_tfidf 
group by docid;

commit;

create index seev__temp_idf_term on seev_feature_class_temp_idf (key);

create index seev_temp_idf_maxtf on seev_feature_class_temp_maxtf (docid);

commit;

drop table seev_feature_class_temp_3;

CREATE TABLE SEEV_FEATURE_CLASS_temp_3
(
  N number DEFAULT 1
);

insert into SEEV_FEATURE_CLASS_temp_3 
select count(docid) from seev_documents

insert into seev_feature_class_temp (key, value, docid)
select /*+ ALL_ROWS */ T1.key, log(2, N/T2.value)*T1.value/T3.value, T1.docid from SEEV_DOCUMENTS t0, eev_feature_class_temp_tfidf T1, SEEV_FEATURE_CLASS_temp_idf T2, SEEV_FEATURE_CLASS_temp_maxtf T3, SEEV_FEATURE_CLASS_temp_3
where t0.docid = t1.docid and 
      T1.docid = T3.docid and 
      t0.docid = t3.docid and 
      T1.key = T2.key;

commit;

drop table seev_feature_class_temp_tfidf;

drop table seev_feature_class_temp_idf;

drop table seev_feature_class_temp_maxtf;

drop table SEEV_FEATURE_CLASS_temp_2;

CREATE TABLE SEEV_FEATURE_CLASS_temp_2
(
    MAX_VALUE number DEFAULT 1
);

insert into SEEV_FEATURE_CLASS_temp_2
select max(value) as MAX_VALUE from SEEV_FEATURE_CLASS_temp;

commit;

insert into SEEV_FEATURE_CLASS_0
select key, value/max_value, docid from SEEV_FEATURE_CLASS_temp, SEEV_FEATURE_CLASS_temp_2   

commit;

drop table SEEV_FEATURE_CLASS_temp;

drop table SEEV_FEATURE_CLASS_temp_2;

--- authority score

insert into SEEV_FEATURE_CLASS_1
select ' ', AUTHSCORE/max_score as value, DOCID from SEEV_DOCUMENTS, (select max(authscore) as max_score from Seev_documents)

--- classes with SVM score

insert into SEEV_FEATURE_CLASS_2
select classname, c.svmscore/max_value as value, docid from seev_documents s, crawldocuments c, (select max(svmscore) max_value from crawldocuments)
where s.docid = c.id and c.svmscore > 0.0001

commit;

insert into SEEV_FEATURE_CLASS_2
select classname, 0.0001, docid from seev_documents s, crawldocuments c, (select max(svmscore) max_value from crawldocuments)
where s.docid = c.id and c.svmscore <= 0.0001

commit;

--- date score
insert into SEEV_FEATURE_CLASS_3
select to_char(lastdate, 'YYYY-MM-DDFMFX') as key, (MONTHS_BETWEEN(lastdate, MN)+0.0001) / (MONTHS_BETWEEN(MX,MN)+0.0001) as value, docid 
from 
 seev_documents,
 (select max(lastdate) as MX, min(lastdate) as MN from seev_documents)
;

commit;

--- old date score

drop table SEEV_FEATURE_CLASS_4;

CREATE TABLE SEEV_FEATURE_CLASS_4
(
    KEY               varchar2(200)  NOT NULL,
    VALUE             number         DEFAULT 0 ,
    DOCID              NUMBER       NOT NULL,
    FOREIGN KEY (DOCID) REFERENCES SEEV_Documents
      ON DELETE CASCADE
);

insert into SEEV_FEATURE_CLASS_4
select to_char(lastdate, 'YYYY-MM-DDFMFX') as key, 1 - (MONTHS_BETWEEN(lastdate, MN)+0.0001) / (MONTHS_BETWEEN(MX,MN)+0.0001) as value, docid 
from 
 seev_documents,
 (select max(lastdate) as MX, min(lastdate) as MN from seev_documents)
;

-- index creation

create index SEEV_FEATURE_CLASS_0_key on SEEV_FEATURE_CLASS_0 (key);

create index SEEV_FEATURE_CLASS_1_key on SEEV_FEATURE_CLASS_1 (key);

create index SEEV_FEATURE_CLASS_2_key on SEEV_FEATURE_CLASS_2 (key);

create index SEEV_FEATURE_CLASS_3_key on SEEV_FEATURE_CLASS_3 (key);

create index SEEV_FEATURE_CLASS_4_key on SEEV_FEATURE_CLASS_4 (key);

create index SEEV_FEATURE_CLASS_0_docid on SEEV_FEATURE_CLASS_0 (docid);

create index SEEV_FEATURE_CLASS_1_docid on SEEV_FEATURE_CLASS_1 (docid);

create index SEEV_FEATURE_CLASS_2_docid on SEEV_FEATURE_CLASS_2 (docid);

create index SEEV_FEATURE_CLASS_3_docid on SEEV_FEATURE_CLASS_3 (docid);

create index SEEV_FEATURE_CLASS_4_docid on SEEV_FEATURE_CLASS_4 (docid);

create index SEEV_FEATURE_CLASS_0_key_id on SEEV_FEATURE_CLASS_0 (key, docid);

create index SEEV_FEATURE_CLASS_1_key_id on SEEV_FEATURE_CLASS_1 (key, docid);

create index SEEV_FEATURE_CLASS_2_key_id on SEEV_FEATURE_CLASS_2 (key, docid);

create index SEEV_FEATURE_CLASS_3_key_id on SEEV_FEATURE_CLASS_3 (key, docid);

create index SEEV_FEATURE_CLASS_4_key_id on SEEV_FEATURE_CLASS_4 (key, docid);


commit;


