package search;

import java.util.*;
import java.lang.*;
import java.sql.*;
import oracle.jdbc.dbaccess.*; 
import oracle.jdbc.driver.*;
import oracle.sql.*;
import java.io.*;


public class DBconnect extends KernelDB
{

public DBconnect()
{
}
    
    
public boolean createSession(long sessionID, long userID, String HostName, Query query_object)
 {   
   if (get_error_status()) return false;  
   try
   {
    String SQLquery = "insert into "+TEMP_USER_TABLE+"(sessionID, userID, HostName, Query, Time) values(?, ?, ?, empty_blob(), ?)";
    PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
    SqlForm.setLong(1, sessionID);
    SqlForm.setLong(2, userID);
    SqlForm.setString(3, HostName);
    SqlForm.setLong(4, new java.util.Date().getTime());
    SqlForm.execute();
    SqlForm.close();
    oracle_conn.commit();
    
    SQLquery = "select query from "+TEMP_USER_TABLE+" where sessionID = ? for UPDATE";
    oracle_conn.setAutoCommit(false);
    SqlForm = oracle_conn.prepareStatement(SQLquery);
    SqlForm.setLong(1,sessionID);
    ResultSet rset = SqlForm.executeQuery();
    rset.next();
    oracle.sql.BLOB blob = ((OracleResultSet)rset).getBLOB(1);
    OutputStream blobOut = ((BLOB)blob).getBinaryOutputStream();
    ObjectOutputStream outstream = new ObjectOutputStream( blobOut );
    outstream.writeObject(query_object);
    outstream.flush();
    blobOut.flush();
    blobOut.close();
    rset.close();
    SqlForm.close();
    oracle_conn.commit();
    oracle_conn.setAutoCommit(true);
    return true;       
  }
  catch (Exception e)
  {
   set_error_message("dbconnect: SESSION ERROR \n"+e.getMessage());
   e.printStackTrace();
  }
  return false;
 }

 public boolean checkSession(long sessionID)
 {
    if (get_error_status()) return false; 
    try
    {
     String SQLquery = "select sessionID from "+TEMP_USER_TABLE+" where sessionID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();  
        
        SQLquery ="update "+TEMP_USER_TABLE+" set time ="+new java.util.Date().getTime()+" where sessionID = ?";
        SqlForm = oracle_conn.prepareStatement(SQLquery);
        SqlForm.setLong(1, sessionID);
        SqlForm.execute();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("checkSession error \n"+e.toString());
     return false;
    }
    return false;
 }

 public boolean checkProfile(long profileID)
 {
    if (get_error_status()) return false; 
    try
    {
     String SQLquery = "select profileID from "+this.PROFILE_TABLE+" where profileID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profileID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();          
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("checkProfile error \n"+e.toString());
     return false;
    }
    return false;
 } 
 
 public long getUserID(long sessionID)
 {
    if (get_error_status()) return BAD_ID; 
    try
    {
     String SQLquery = "select userID from "+TEMP_USER_TABLE+" where sessionID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next()) 
      {
        long l = rset.getLong("userID");
        rset.close();
        SqlForm.close();
        return l;
      }
     rset.close();
     SqlForm.close();
    }
    catch (Exception e)
    {
     set_error_message("getUserID error \n"+e.toString());
     return BAD_ID;
    }
    return DEFAULT_USER_ID;
 }
 
 public String getHostName(long sessionID)
 {
    if (get_error_status()) return null; 
    try
    {
     String SQLquery = "select HostName from "+TEMP_USER_TABLE+" where sessionID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next()) 
      {
        String s = rset.getString("HostName");
        rset.close();
        SqlForm.close();
        return s;
      }
     rset.close();
     SqlForm.close();
    }
    catch (Exception e)
    {
     set_error_message("getHostName error \n"+e.toString());
     return null;
    }
    return null;
 }

 public Query getQuery(long sessionID)
 {
    if (get_error_status()) return null; 
    try
    {
     String SQLquery = "select query from "+TEMP_USER_TABLE+" where sessionID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next()) 
      {
        rset.getBlob("query");
        oracle.sql.BLOB blob = ((OracleResultSet)rset).getBLOB( "query" );
        ObjectInputStream ois = new ObjectInputStream(blob.getBinaryStream());
        Query _query = (Query)ois.readObject();
        ois.close();
        rset.close();
        SqlForm.close();
        return _query;
      }
     rset.close();
     SqlForm.close();
    }
    catch (Exception e)
    {
     set_error_message("getQuery form blob error \n"+e.toString());
     return null;
    }
    return null;
 }

 public Result_List getSessionList(long sessionID)
 {
   if (!checkSession(sessionID)) return null;
   String SQLquery = "";
   try
   { 
     String countQuery = "select count(*) from " + getIDStr(sessionID);
     int count = 0;
     PreparedStatement SqlForm2 = oracle_conn.prepareStatement(countQuery);
     ResultSet rset2 = SqlForm2.executeQuery();
     if (rset2.next()) count = rset2.getInt(1);
     rset2.close();
     SqlForm2.close();
     if (count == 0) return null;
    
     SQLquery = "select * from " + getIDStr(sessionID);
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     ResultSet rset = SqlForm.executeQuery();
     Result_Document[] rd_list = new Result_Document[count];
     int j = 0;
     while (rset.next())
      {
        long doc_id = BAD_ID;
        double[] features = new double[GLOBAL_NF];
        doc_id = rset.getLong(1);
        
        for (int i=0; i<GLOBAL_NF; i++)
         {
            features[i] = rset.getDouble(i+2);
         }
        Result_Document rd = new Result_Document(doc_id, features);
        rd_list[j] = rd;
        j++;
     }
    rset.close();
    SqlForm.close(); 
    return new Result_List(rd_list);
   }
   catch(Exception e)
   {
     set_error_message("getSessionList Error general\n"+SQLquery+"\n"+e.toString());
   }
   return null;
 }
 
 public String createJoinQuery(Query query, int i)
 {
   if (query == null || i<0 || i>=query.get_NF()) return null;
   
   String ValuePart = " T_0"+".value ";
   String FromPart  = FEATURE_NAME_PREFIX+i+" T_0 ";
   String WherePart = "";
   String SQLqueryJoin = "(select T_0.DOCID, "; 
   String equal = " = ";
   if (i == CLASS) equal = " like";  
   String addition = "%";
   if (i!=CLASS) addition = "";
   
   if (query.isKey(i))
    { 
      int n = query.getDimSize(i);
      if (n>0 && !query.isDefaultKey(i,0)) WherePart = " where T_0.key "+equal+" '"+query.getKey(i,0)+addition+"' ";
      for (int j=1; j<n; j++)
       {
         WherePart += " and T_"+j+".key "+equal+" '"+query.getKey(i,j)+addition+"' and T_0.DOCID = T_"+j+".DOCID";
         ValuePart += "+ T_"+j+".value ";
         FromPart  +=", "+ FEATURE_NAME_PREFIX+i+" T_"+j+" ";
       }   
    }  
   
   SQLqueryJoin += ValuePart+" as score"+i+" from "+FromPart+" "+WherePart+" )";

   if (i<query.get_NF()-1)
    {
      SQLqueryJoin = "(select T"+i+".score"+i+", T.*  from " + SQLqueryJoin + " T"+i+" join " + createJoinQuery(query, i+1) + " on T"+i+".DOCID = T.DOCID) T";
    }
   else
     SQLqueryJoin += " T ";
   
   return SQLqueryJoin.toLowerCase();     
 }
 
 public int get_doc_feedback_status(long doc_id, long sessionID)
 {
   if (this.get_error_status() || doc_id == this.BAD_ID || !checkSession(sessionID)) return this.NEUTRAL;
   try
   {
     String SQLquery = "select distinct(status) from "+this.FEEDBACK_SESSION_TABLE+" T1, "+this.FEEDBACK_DOCS_TABLE+" T2 where sessionid = ? and T1.feedbackID = T2.feedbackID and DOCID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     SqlForm.setLong(2, doc_id);
     ResultSet rset = SqlForm.executeQuery();
     int _status = this.NEUTRAL;
     if (rset.next()) _status = rset.getInt(1);
     if (rset.next()) _status = this.NEUTRAL; //one document has different statuses in the same session
     return _status;
   }
   catch (Exception e)
   {
    this.set_error_message("get_doc_feedback_status error\n"+e.toString());   
   }
   return this.NEUTRAL;
 }
 
 public Collection ObtainDocBodies(Result_List r_l, long sessionID, int start, int howmany)
 {
   if (r_l == null | r_l.get_error_status() || start < 0 || howmany <= 0 || start + howmany > r_l.getLength()) return null;
   try
   {
    Vector V = new Vector();
    String SQLquery = "select * from "+DOCUMENTS_TABLE+" where DOCID = ?";
    for (int i=0; i<howmany; i++)
    {
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     long doc_id = r_l.get_doc(start + i).get_doc_id_longValue();
     SqlForm.setLong(1, doc_id);
     ResultSet rset = SqlForm.executeQuery();       
     if (rset.next())
      {
       String _url =       rset.getString("url");
       String _title =     rset.getString("title");
       String _text =      rset.getString("text");
       double _size =      rset.getDouble("docsize"); 
       String _classname = rset.getString("classname");
       java.sql.Date _LastDate =    rset.getDate("lastdate");
       long _ID =          rset.getLong("DOCID");
       if (_title==null || _title=="") _title = DEFAULT_TITLE;
       Result_Body RB = new Result_Body(_url, _title, _text, _size/1000, _classname, _ID, _LastDate.getTime(), sessionID);       
       RB.setScores(r_l.get_doc(start+i).unit_scores);
       RB.set_feedback_status(this.get_doc_feedback_status(doc_id, sessionID));
       if (RB.getStatus()) V.add(RB);
      }
      rset.close();
      SqlForm.close();
    }
     return V;  
   }
   catch(Exception e)
   {
     e.printStackTrace();
     set_error_message("getDocument Bodies Error \n"+e.toString());    
   }
   return null; 
 }
  
 public Result_List getNewList(Query query, long sessionID)
 {
   if (query == null) return null;
   String SQLquery = "";
   try
   { 
     if (!createSessionTable(sessionID)) return null;
     SQLquery = createJoinQuery(query, 0);  
     String temp = "DOCID";
     for (int i=0; i<query.get_NF(); i++)
       temp += ", score"+i+"/"+query.getDimSize(i); 
     SQLquery = "insert into "+getIDStr(sessionID)+" select "+temp+" from " + SQLquery;
     set_error_message("num keywords "+query.getDimSize(0));
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.execute();
     SqlForm.close(); 
     oracle_conn.commit();
     return getSessionList(sessionID);
   }
   catch(Exception e)
   { 
     set_error_message("getSessionNewList Error \n"+SQLquery+"\n"+e.toString());    
   }
   return null;  
 }
 
 public Result_List getNewList(Query query)
 {
   if (query == null) return null;
   String SQLquery = "";
   try
   { 
     SQLquery = createJoinQuery(query, 0);
     SQLquery = "select * from " + SQLquery;
     set_error_message(SQLquery);
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     ResultSet rset = SqlForm.executeQuery();
     
     String countQuery = "select count(*) from "+SQLquery;
     int count = 0;
     PreparedStatement SqlForm2 = oracle_conn.prepareStatement(countQuery);
     ResultSet rset2 = SqlForm2.executeQuery();
     if (rset2.next()) count = rset2.getInt(1);
     rset2.close();
     SqlForm2.close();
     
     if (count == 0) return null;
     
     Result_Document[] rd_list = new Result_Document[count];
     int j = 0;
     while (rset.next())
      {
        long doc_id = BAD_ID;
        double[] features = new double[GLOBAL_NF];
        doc_id = rset.getLong(1);
        for (int i=0; i<GLOBAL_NF; i++)
         features[i] = rset.getDouble(i+2);   
        Result_Document rd = new Result_Document(doc_id, features);
        rd_list[j] = rd;
        j++;
     }
    rset.close();
    SqlForm.close(); 
    return new Result_List(rd_list);
   }
   catch(Exception e)
   {       
     set_error_message("getSessionList Error new \n"+SQLquery+"\n"+e.toString());    
   }
   return null;
 }

 public boolean createSessionTable(long sessionID)
 {
   if (!checkSession(sessionID)) return false;
   try
   {
    String SQLquery = "create TABLE " + getIDStr(sessionID) + "(DOCID Number default -1 not null";
    for (int i=0; i<GLOBAL_NF; i++)
     SQLquery = SQLquery + ", score"+ i +" FLOAT default 0.0 ";
    SQLquery = SQLquery + " , FOREIGN KEY (DOCID) REFERENCES SEEV_Documents ON DELETE CASCADE)";
    return createTable(SQLquery);   
   }
   catch(Exception e)
   {
     set_error_message("createSession Error \n"+e.toString());  
   }
   return false;  
 }
 
 public boolean storeSessionList(long sessionID, Result_List RL)
 {
    if (RL == null || RL.get_error_status() || !checkSession(sessionID)) return false;
    String SQLquery = "";
    try
    {
     int n = RL.getLength();
     for (int i=0; i<n; i++)
      {
       Result_Document rd = RL.get_doc(i);  
       SQLquery = "insert into " + getIDStr(sessionID) + "(DOCID ";
       for (int j=0; j<rd.getNF(); j++)
        SQLquery = SQLquery + ", " + FEATURE_NAME_PREFIX + i;
       SQLquery = SQLquery + ") Values(? ";
       for (int j=0; j<rd.getNF(); j++)
        SQLquery = SQLquery + ", ?";
       SQLquery = SQLquery + ")";
       PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
       SqlForm.setLong(1, rd.get_doc_id_longValue());
       for (int j=0; j<rd.getNF(); j++)
        SqlForm.setDouble(j+2, rd.unit_scores[j]);        
       if (!SqlForm.execute()) return false;
       SqlForm.close();
       oracle_conn.commit();
      }
    }
    catch(Exception e)
    {
      set_error_message("Error in SessionStrore \n"+SQLquery+"\n"+e.toString());  
    }
    return false;  
 }
   
 public Collection getClasses()
 {     
   if (get_error_status()) return null;   
   
   String SQLquery = "select distinct(name) as name from "+CLASS_TABLE;
   Vector classes = new Vector();
   try
   {
   PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
   ResultSet rset = SqlForm.executeQuery();   
   classes.add(DEFAULT_CLASS);
   while (rset.next())
    {
      String temp = rset.getString(1);
      if (temp != null) classes.add(temp);
    }
   
   rset.close();
   SqlForm.close();
   }
   catch (Exception e)
   {
     set_error_message("getClasses "+e.toString());
   }
    return classes;
 }

 public void storeBlobFeedback(long sessionID, Feedback feedback)
 {
   if (feedback == null || feedback.get_error_status() || !checkSession(sessionID)) return;
   try
   {     
    String SQLquery = "insert into "+this.FEEDBACK_BLOB_TABLE+"(sessionID, feedback) values(?, empty_blob())";
    PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
    SqlForm.setLong(1, sessionID);
    SqlForm.executeUpdate();
    SqlForm.close();
    oracle_conn.commit();
      
    feedback.set_sessionID(sessionID);
    SQLquery = "select feedback from "+this.FEEDBACK_BLOB_TABLE+" where sessionID = ? for UPDATE";
    oracle_conn.setAutoCommit(false);
    SqlForm = oracle_conn.prepareStatement(SQLquery);
    SqlForm.setLong(1,sessionID);
    ResultSet rset = SqlForm.executeQuery();
    rset.next();
    oracle.sql.BLOB blob = ((OracleResultSet)rset).getBLOB(1);
    OutputStream blobOut = ((BLOB)blob).getBinaryOutputStream();
    ObjectOutputStream outstream = new ObjectOutputStream( blobOut );
    outstream.writeObject(feedback);
    outstream.flush();
    blobOut.flush();
    blobOut.close();
    rset.close();
    SqlForm.close();
    oracle_conn.commit();
    oracle_conn.setAutoCommit(true);
   }
   catch(Exception e)
   {
     set_error_message("Error in storeBlobFeedback \n"+e.toString());
   }    
 } 

  public Feedback getBlobFeedback(long sessionID)
  {
    if (get_error_status()) return null; 
    try
    {
     String SQLquery = "select feedback from "+this.FEEDBACK_BLOB_TABLE+" where sessionID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next()) 
      {
        oracle.sql.BLOB blob = ((OracleResultSet)rset).getBLOB(1);
        ObjectInputStream ois = new ObjectInputStream(blob.getBinaryStream());
        Feedback _feedback = (Feedback)ois.readObject();
        ois.close();
        rset.close();
        SqlForm.close();
        return _feedback;
      }
     rset.close();
     SqlForm.close();
    }
    catch (Exception e)
    {
     set_error_message("getFeedback form blob error \n"+e.toString());
     return null;
    }
    return null;
 }

 
 public boolean storeFeedback(long sessionID, Feedback feedback)
 {
   if (feedback == null || feedback.get_error_status() || !checkSession(sessionID)) return false;
   try
   {
    long feedbackID = feedback.getID();
    if (!checkFeedback(feedbackID))   
    {
     String SQLquery = "select userid, query from "+ TEMP_USER_TABLE+" where sessionid = ?";
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (!rset.next()) 
     {
      rset.close();
      SqlForm.close();
      return false;
     }
    
     long            userid  = rset.getLong(1);
     oracle.sql.BLOB blob    = (oracle.sql.BLOB)rset.getBlob(2);
     java.sql.Date   date    = new java.sql.Date(new java.util.Date().getTime());
    
     rset.close();
     SqlForm.close();
    
     SQLquery = "insert into "+FEEDBACK_TABLE+" values(?, ?, ?, ?)";   
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, feedbackID);
     SqlForm.setLong(2, userid);
     SqlForm.setBlob(3, blob);
     SqlForm.setDate(4, date);
    
     SqlForm.execute();
     SqlForm.close();
     oracle_conn.commit();     
    }
    
    //store feedback as blob
    this.storeBlobFeedback(sessionID, feedback);
    
    if (!checkFeedback(feedbackID)) return false;
    
    String SQLquery = "insert into "+FEEDBACK_DOCS_TABLE+" values(?, ?, ?)";    
    for (int i=0; i<feedback.getLength(); i++)
     {
        PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
        SqlForm.setLong(1, feedbackID);
        SqlForm.setLong(2, feedback.getDOCID(i));
        SqlForm.setInt(3, feedback.getStatus(i));
        SqlForm.execute();
        SqlForm.close();
     }    
    
    //Feedback_session
    SQLquery = "insert into "+this.FEEDBACK_SESSION_TABLE+" (feedbackID, sessionID) values(?, ?)";
    PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
    SqlForm.setLong(1, feedback.getID());
    SqlForm.setLong(2, sessionID);
    SqlForm.execute();
    SqlForm.close();    
    
    return true;
   }
   catch(Exception e)
   {
     set_error_message("Error in storeFeedback \n"+e.toString());  
   }

   return false;  
 }
 
 
 
 public boolean checkFeedback(long feedbackID)
 {
    if (get_error_status()) return false; 
    try
    {
     String SQLquery = "select feedbackID from "+FEEDBACK_TABLE+" where feedbackID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, feedbackID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("checkFeedback error \n"+e.toString());
     return false;
    }
    return false;
 } 
 
 public boolean deleteSession()
 {
  if (get_error_status()) return false;     
  try
  {   
   java.util.Date D = new java.util.Date();      
   long current_time = D.getTime();  
   String query = "select sessionid from "+TEMP_USER_TABLE+" where sessionid not in (select sessionid from "+FEEDBACK_BLOB_TABLE+") and ?-time>"+SESSION_TIMEOUT;   
   set_error_message(query);
   PreparedStatement SqlForm = oracle_conn.prepareStatement(query);
   SqlForm.setLong(1, current_time);
   ResultSet rset = SqlForm.executeQuery();  
   long _ID=0;

   PreparedStatement temp = oracle_conn.prepareStatement(query);
   while (rset.next())
    {
      try
      {
      _ID = rset.getLong("sessionid");   
      String subQuery = "drop table "+getIDStr(_ID);  
      temp = oracle_conn.prepareStatement(subQuery);
      boolean t = temp.execute(); 
      temp.close();

      subQuery = "insert into "+HISTORY_TABLE+" select userID, hostname, query, time from "+TEMP_USER_TABLE+" where sessionid=?";  
      temp = oracle_conn.prepareStatement(subQuery);
      temp.setLong(1, _ID);
      t = temp.execute(); 
      temp.close();

      subQuery = "delete from "+TEMP_USER_TABLE+" where sessionid=?";  
      temp = oracle_conn.prepareStatement(subQuery);
      temp.setLong(1, _ID);
      t = temp.execute(); 
      temp.close();
      
      }
      catch (Exception e1) 
      {
        temp.close();  
        set_error_message("local delete error\n"+e1.toString());
      }
    }
    rset.close();
    SqlForm.close();
   
   String query1="insert into "+HISTORY_TABLE+" select userID, hostname, query, time from "+TEMP_USER_TABLE+" where ?-time>"+SESSION_TIMEOUT;
   SqlForm = oracle_conn.prepareStatement(query1);
   SqlForm.setLong(1, current_time);
   SqlForm.execute();
   SqlForm.close();
   oracle_conn.commit();
   
   
   query = "delete from "+TEMP_USER_TABLE+" where sessionid not in (select sessionid from "+FEEDBACK_BLOB_TABLE+") and ?-time>"+SESSION_TIMEOUT;   
   SqlForm = oracle_conn.prepareStatement(query);
   SqlForm.setLong(1, current_time);
   SqlForm.execute();
   SqlForm.close();
   oracle_conn.commit();

   return true;
  }
  catch (Exception e)
  {
   set_error_message("dbconnect Error dQuery \n"+e.toString());
   return false;
  }
 }

 public boolean storeDistinctProfile(long feedbackID, Profile profile)
 {
   if (get_error_status() && profile==null && profile.get_error_status() && !checkFeedback(feedbackID)) return false;
   try
    {
     if (!storeProfile(profile)) return false;
     String SQLquery = "insert into "+this.DISTINCT_PROFILE_TABLE+" values(?, ?)";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profile.getID());
     SqlForm.setLong(2, feedbackID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("storeDistinctProfile error \n"+e.toString());
    }
    return false;             
 }
 
 public boolean storeEvaluation(long feedbackID)
 {
   if (get_error_status() && !checkFeedback(feedbackID)) return false;
   try
    {
     String SQLquery = "insert into "+this.EVALUATION_TABLE+" values(?)";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, feedbackID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("storeEvaluation error \n"+e.toString());
    }
    return false;             
 } 
 
 public boolean deleteNegativ(long[] docs, long sessionID)
 {
   if (docs == null || !checkSession(sessionID)) return false;
   try
   {
    String SQLquery = "delete from "+ getIDStr(sessionID)+" where docid = ?";  
    for (int i=0; i<docs.length; i++)
     {
      PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
      SqlForm.setLong(1, docs[i]);
      SqlForm.execute();
     }   
   }
   catch (Exception e)
   {
     set_error_message("delete negativs error \n"+e.toString());
     return false;
   }
   return true;
 }
 
 public boolean storeProfile(Profile profile)
 {
    if (get_error_status() && profile==null && profile.get_error_status()) return false; 
    try
    {
        
     String SQLquery = "insert into "+PROFILE_TABLE+" values(?";
     for (int i=0; i<GLOBAL_NF; i++)  SQLquery += ", ?";
     SQLquery += ", ?)";
     
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profile.getID());
     for (int i=0; i<GLOBAL_NF; i++)
       SqlForm.setDouble(i+2, profile.getWeight(i));
     java.sql.Date   date    = new java.sql.Date(new java.util.Date().getTime());
     SqlForm.setDate(GLOBAL_NF+2, date);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("storeProfile error \n"+e.toString());
    }
    return false;          
 }

 public Profile get_used_profile(long sessionID)
 {
    if (get_error_status() && !this.checkSession(sessionID)) return null; 
    try
    {
     String SQLquery = "SELECT profileID FROM "+this.USED_PROFILE_TABLE+" WHERE sessionID = ?"; 
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, sessionID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        long profileID = rset.getLong(1);
        rset.close();
        SqlForm.close();
        return this.getProfile(profileID);
      }
    }
    catch (Exception e)
    {
      set_error_message("get_used_profile error\n"+e.toString());  
    }
    return null;
 }
 
 public boolean store_used_profile(long profileID, long sessionID)
 {
    if (get_error_status() && profileID == 0 && sessionID == 0 && !this.checkSession(sessionID) && !this.checkProfile(profileID)) return false; 
    try
    {        
     String SQLquery = "UPDATE "+this.USED_PROFILE_TABLE+" SET profileID = ? WHERE sessionID = ?"; 
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profileID);
     SqlForm.setLong(2, sessionID);
     if (SqlForm.executeUpdate() == 0)
      {
       SqlForm.close();
       SQLquery = "INSERT INTO "+this.USED_PROFILE_TABLE+" (profileID, sessionID) values(?, ?)";     
       SqlForm = oracle_conn.prepareStatement(SQLquery);
       SqlForm.setLong(1, profileID);
       SqlForm.setLong(2, sessionID);
       SqlForm.execute();       
       SqlForm.close();
      }
     return true;
    }
    catch (Exception e1)
    {
     try
     {
      String SQLquery = "INSERT INTO "+this.USED_PROFILE_TABLE+" (profileID, sessionID) values(?, ?)";     
      PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
      SqlForm.setLong(1, profileID);
      SqlForm.setLong(2, sessionID);
      SqlForm.execute();
      SqlForm.close();
      return true;
     }
     catch (Exception e2)
     {
      set_error_message("storeProfile error \n"+e1.toString()+"\n"+e2.toString());
     }
    }
    return false;          
 } 
 
 public Profile getDistinctProfile(long feedbackID)
 {
    if (get_error_status() || !checkFeedback(feedbackID)) return null; 
    try
    {
     String SQLquery = "select profileID from "+DISTINCT_PROFILE_TABLE+" where feedbackID = ?";  
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, feedbackID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        long profileID = rset.getLong(1);
        Profile profile = getProfile(profileID);
        rset.close();
        SqlForm.close();
        oracle_conn.commit();
        return profile;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("getDistinctProfile error \n"+e.toString());
     return null;
    }
    return null;     
 }
 
 public Profile getProfile(long profileID)
 {
    if (get_error_status()) return null; 
    try
    {
        
     String SQLquery = "SELECT w0";
     for (int i=1; i<GLOBAL_NF; i++)
         SQLquery += ", w"+i;
     SQLquery += " FROM "+PROFILE_TABLE+" WHERE profileID = ?";  
     
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profileID);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        double[] p_vector = new double[GLOBAL_NF];  
        for (int i=0; i<GLOBAL_NF; i++)
         p_vector[i] = rset.getDouble(i+1);
        
        Profile profile = new Profile(profileID, p_vector);
        
        rset.close();
        SqlForm.close();
        return profile;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("get_profile_1 error \n"+e.toString());
    }
    return null;     
 }
}