 package search;

import java.util.*;
import java.lang.*;
import java.io.*;

public class Document extends Error implements Serializable
{
  
    private HashMap[] features;
    private Object[][] keys;
    private Object[][] values;
    private boolean[] isValue;
    private boolean[] isKey;    
    private int NF = GLOBAL_NF;
    private boolean KeysValues = false;
    
    public Document()
    {
      set_error_status();  
    }
    
    public Document(int _NF)
    {
     set_NF(_NF);   
    }
    
    public void set_NF(int _NF) 
    {
     if (_NF<=0) set_error_status();
      else
      {
       NF = _NF;
       
       features = new HashMap[NF];
       for (int i=0; i<NF; i++)
         features[i] = new HashMap();
       
       isValue =  new boolean[NF];
       isKey =  new boolean[NF];
       
       keys = new Object[NF][];
       values = new Object[NF][];
              
       for (int i=0; i<NF; i++) isValue[i] = true;
       for (int i=0; i<NF; i++) isKey[i] = false;       
       unset_error_status();
      }
    }
    
   public void createKeyList()
   {
     KeysValues = false;  
     for (int i=0; i<NF; i++)
      {
        keys[i] = features[i].keySet().toArray();
        values[i] = features[i].values().toArray();
      }
     KeysValues = true;
   }
  
   public boolean setKey(int i)
   {
     if (i<0 || i>=NF) return false;
     isKey[i] = !isKey[i]; 
     return true;
   }

   public boolean isDefaultKey(int i, int j)
   {
     if (getKey(i,j) == null) return true;
     if (getKey(i,j).equals(DEFAULT_OBJECT)) return true;
     return false;  
   }
   
   public boolean isKey(int i)
   {
     if (i<0 || i>=NF) return true; //the value is exist and if not it will be replaced by some defualt value
     return isKey[i];
   }

   public int get_NF()
   {
    return NF;
   }
   
   //i-th feature class, (key, value), value =0|1 basically
   public boolean put(int i, Object key, Object value)
   {
    if (i<0 || i>=NF || key == null || value == null) return false;
    features[i].put(key, value);
    return true;
   }
   
   //i-th feature class, value for objects without key, like authority score
   public boolean put(int i, Object value)
   {
    if (i<0 || i>=NF || value == null) return false;
    features[i].put(DEFAULT_OBJECT, value);
    return true;
   }   
   
   //returns the # of pairs in i-throws feature class for current document
   public int getDimSize(int i)
   {       
    if (i<0 || i>=NF) return 0;
    return features[i].size();
   }
   
   //return sa (Key, Value) pair
   public Object get(int i, Object key)
   {
    if (i<0 || i>=NF || key == null) return null;
    return features[i].get(key);
   }

   //returns a key of j-th element in i-th dimension
   public Object getKey(int i, int j)
   {
    if (i<0 || j<0 || i>=NF || j>=getDimSize(i)) return null;
    if (!KeysValues) createKeyList();
    return keys[i][j];
   }

   //returns a value of j-th element in i-th dimension   
   public Object getValue(int i, int j)
   {
    if (i<0 || j<0 || i>=NF || j>=getDimSize(i)) return null;
    if (!KeysValues) createKeyList();
    return values[i][j];
   }
   
}
