package search;

import java.lang.*;
import java.util.*;
import java.io.*;

public class Feedback extends Error implements Serializable
{
  private long sessionID = 0;
  private Feedback_Document[] list;
  private HashMap hash_list;
  private long[] pos_set;
  private long[] neg_set;
  private long feedbackID = 0;  
  private int n_pos = 0;
  private int n_neg = 0;
  
  public Feedback(long _feedbackID, Feedback_Document[] _list) 
  {
    if (_list!=null && _list.length > 0 && _feedbackID != 0) 
     {
      list = _list;
      hash_list = new HashMap();
      for (int i=0; i<list.length; i++)
        hash_list.put(new Long(list[i].getDOCID()), new Integer(list[i].getStatus()));
      feedbackID = _feedbackID;
      reset();
     }
     else set_error_status();
  }  
  
  public void reset()
  {
    computeNP();
    pos_set = new long[n_pos];
    neg_set = new long[n_neg];
    int k_p = 0;
    int k_n = 0;
    for (int i=0; i<list.length; i++)
    if (this.getStatus(i) == POSITIV) {pos_set[k_p] = this.getDOCID(i); k_p++;}
     else if (this.getStatus(i) == NEGATIV) {neg_set[k_n] = this.getDOCID(i); k_n++;} 
  }
  
  public long get_pos_doc(int i)
  {
    if (i<0 || i>n_pos || get_error_status()) return 0;
    if (pos_set == null) reset();  
    return pos_set[i];
  }
  
  public long get_neg_doc(int i)
  {
    if (i<0 || i>n_neg || get_error_status()) return 0;
    if (neg_set == null) reset();  
    return neg_set[i];
  }
  
  public Feedback(Feedback_Document[] _list) 
  {
    if (_list!=null && _list.length > 0) 
     {
      list = _list;
      hash_list = new HashMap();
      for (int i=0; i<list.length; i++)
        hash_list.put(new Long(list[i].getDOCID()), new Integer(list[i].getStatus()));
      feedbackID = this.getNewID();
      computeNP();
     }
     else set_error_status();
  }
  
  public void computeNP()
  {
    n_pos = 0;
    n_neg = 0;
    for (int i=0; i<getLength(); i++)
     if (list[i].getStatus() == POSITIV) n_pos++;
      else if (list[i].getStatus() == NEGATIV) n_neg++;
  }
  
  public int getNumPos()
  {
    return n_pos;  
  }
  
  public void set_sessionID(long _sessionID)
  {
   sessionID = _sessionID;   
  }
  
  public long get_sessionID()
  {
    return sessionID;  
  }
  
  public int getNumNeg()
  {
    return n_neg;  
  }
  
  public int getLength()
  {
   if (get_error_status()) return 0;
   return list.length;
  }
  
  public long getDOCID(int i)
  {
    if (!get_error_status() && i>=0 && i<list.length) return list[i].getDOCID();   
    return this.BAD_ID;
  }
  
  public int getStatus(int i)
  {
    if (!get_error_status() && i>=0 && i<list.length) return list[i].getStatus();   
    return this.NEUTRAL;
  }
  
  public int getStatus(long _DOCID)
  {
    Integer I = (Integer)hash_list.get(new Long(_DOCID));
    if (I == null) return NEUTRAL;
    return I.intValue();
  }
   
  public long getID()
  {
    return feedbackID;  
  }
  
  public int length()
  {
    return list.length;  
  }
  
  public double getDocScore(int i)
  {
    if (!get_error_status() && i>=0 && i<list.length) return list[i].getDocScore();
    return 0.0;
  }
  
  public double F(double[] profile, Result_List _r_l)
  {
   if (profile == null || _r_l == null || profile.length != GLOBAL_NF || _r_l.get_error_status()) return -1.0;
   Result_List r_l = _r_l;
   r_l.set_profile(profile);
   if (r_l.get_error_status()) return -1.0;
   double sum = 0.0;
   for (int i=0; i<list.length; i++)
    {
      double x = getDocScore(i) - r_l.get_doc(getDOCID(i)).Sum();  
      sum += x*x;
    }
   return sum;
  }  
}
