package search;

import weka.core.*;
import weka.clusterers.*;
import weka.filters.unsupervised.attribute.*;

public class Global_Profiles_Constructor extends Error
{
    
    private profileDB DB = new profileDB();;
    private ProfileCluster[] clusters;
    private EM clusterer = new EM();
    private FastVector attInfo;
    private int NF;
    private Instances allProfiles = null;
    private double eps = 0.1;

    
    public Global_Profiles_Constructor() 
    {
      setNF(GLOBAL_NF);
      setEps(0.1);
      DB.CloseConnect();
    }
    
    public void setEps(double _eps)
    {
      if (_eps > 0 && _eps < 1) eps = _eps;  
      else eps = 0.1;
      clusterer.setMinStdDev(eps);
    }
    
    public void setNF(int _NF)
    {
      if (_NF > 0) NF = _NF;
      else NF = GLOBAL_NF;
      
      attInfo = new FastVector(NF+1);
      Attribute id = new Attribute("profileID");      
      Attribute[] w = new Attribute[NF];
      for (int i=0; i<NF; i++)
       {
        w[i] = new Attribute("w"+i); 
        attInfo.addElement(w[i]);
       }
      allProfiles = new Instances("profiles", attInfo, 100);
    }
    
    public void generate_new_global_profiles()
    {
        DB.OpenConnect();
        Cluster();      
        int n = clusters.length;
        if (n>0)
        {          
         DB.removeAllGlobalProfiles();       
         this.GLOBALPROFILES = new GlobalProfile[n];   
         for (int i=0; i<n; i++)
         {
            GlobalProfile profile = new GlobalProfile(clusters[i].means, clusters[i]);
            if (profile!= null && !profile.get_error_status())  DB.storeGlobalProfile(profile);
            FeedbackCluster feedbackCluster = new FeedbackCluster(profile.getID());
            feedbackCluster.buildCluster();
            profile.setFeedbackCluster(feedbackCluster);
            this.GLOBALPROFILES[i] = profile;
         }
         DB.store_all_global_profiles_as_blob();
        }
        DB.CloseConnect();
    }
    
    public void process()
    {
      DB.OpenConnect();        
      GLOBALPROFILES = DB.get_all_global_profiles();
      if (this.GLOBALPROFILES == null) generate_new_global_profiles();      
      DB.CloseConnect();
    }
    
    public void Cluster()
    { 
     Profile[] profiles = DB.getAllDistinctProfiles();
     if (profiles != null)
      {
        for (int i=0; i<profiles.length; i++)
         if (profiles[i]!= null && !profiles[i].get_error_status())
          {
           Instance I = new Instance(NF);
           //I.setValue(0, profiles[i].get_Long_ID().toString());
           for (int j=0; j<NF; j++)
            I.setValue(j, profiles[i].getWeight(j));
           allProfiles.add(I);
          }
      try
      {
       clusterer.buildClusterer(allProfiles);
       int n = clusterer.numberOfClusters();
       if (n>0) clusters = new ProfileCluster[n];
       double[][][] x = clusterer.getClusterModelsNumericAtts();
       set_error_message(clusterer.toString());
       for (int i=0; i<n; i++)
        { 
          double[] mean = new double[NF];
          double[] dev  = new double[NF];
          for (int j=0; j<NF; j++)
           {
            mean[j]=x[i][j][0];
            dev[j] =x[i][j][1];
           }
          clusters[i] = new ProfileCluster(NF, mean, dev);
        } 
       
       for (int i=0; i<allProfiles.numInstances(); i++)
        {
         Instance I = allProfiles.instance(i);
         int k = clusterer.clusterInstance(I);
         double[] s = clusterer.distributionForInstance(I);
         clusters[k].add(profiles[i], s[k]);
        }              
      }
      catch (Exception e)
      {
        e.printStackTrace();  
        set_error_message(e.toString());
      }            
     }     
    }  
    
}
