package search;

import java.util.*;
import java.lang.*;

import cern.colt.matrix.linalg.*;
import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;

public class Natural_Score_Approach extends Error
{
    private static Result_List result_list;
    private static Feedback feedback;
    private int NF = GLOBAL_NF;
    private double[][] A = new double[NF][NF];
    private double[] B = new double[NF];
    private double[] E = new double[NF];
    private double step = 0.1; 
    private TreeMap results = new TreeMap();
    private int N = 10;
    private int iter = 2;
    private double max;
    private boolean go = true;
    
    public Natural_Score_Approach(Result_List _result_list, Feedback _feedback) 
    {
      if (_result_list != null && _feedback != null && !_feedback.get_error_status() && !_result_list.get_error_status())
       {
         result_list = _result_list;
         feedback = _feedback;
         for (int i=0; i<NF; i++)
          {B[i] = -1; E[i] = 1;}
       }
      else set_error_status();
    }
    
    public void set_NF(int _NF)
    {
      if (_NF>0)
       {
         NF = _NF;          
         A = new double[NF][NF];
         B = new double[NF];
         E = new double[NF];
         for (int i=0; i<NF; i++) {B[i]=-1; E[i]=1;}
       }
    }
    
    
    public void wave(int k, double[] _p_vector)
    {
       double[] p_vector = (double[])_p_vector.clone();         
       double b = B[k];
       double e = E[k];
       while (b<=e && go)
        {   
         p_vector[k] = b;
         b += step;
         if (k < NF - 1) wave(k+1, p_vector);
          else
          {
           result_list.set_profile(p_vector);
           double d = result_list.estimateFeedback(feedback);
           if (d>0)
           {
            double[] vector = (double[])p_vector.clone();   
            if (results.size() < N) results.put(new Double(d), vector);
             else if (d > ((Double)results.firstKey()).doubleValue())
             {
              results.remove(results.firstKey());
              results.put(new Double(d), vector);
             }
            if (d >= max) go = false; 
           }
          }           
        }
    }
    
    public double[] computeProfile()
    {
      double[] p_vector = new double[NF];
      max = result_list.maxBorder(feedback);
      go = true;
      for (int i=0; i<NF; i++) p_vector[i]=0;
      wave(0, p_vector);
      for (int i=0; i<iter && go; i++)
       {
         TreeMap _results = (TreeMap)results.clone();
         Iterator It = _results.values().iterator();
         double step_old = step/2;
         step = step/N;
         while (It.hasNext())
          {
            Object O = It.next();
            if (O!= null)
            {   
             double[] _p_vector = (double[])O;
             if (_p_vector!=null)
             for (int j=0; j<NF; j++)
              {
                B[j]=_p_vector[j]-step_old;
                E[j]=_p_vector[j]+step_old;
                if (B[j]<-1) B[j]=-1;
                if (E[j]>1)  E[j]=1;
              }
              wave(0, p_vector);
            }
          }
         Double d1 = (Double)_results.lastKey();
         Double d2 = (Double)results.lastKey();
         if (d1.doubleValue() >= d2.doubleValue())
          {
            results = _results;
            go = false;
          }
       }   
      Object key = results.lastKey();
      Object O = results.get(key);
      if (O!=null)
       p_vector = (double[])O;
      return p_vector;  
    }        

}
