package search;

import java.lang.*;
import java.util.*;

public class ProfileCluster extends Error
{    
 private HashMap    IDscore = new HashMap();
 private HashMap    IDbody  = new HashMap();
 private Double[]   scores;
 private Profile[]  profiles; 
 public  double[]   means;
 public  double[]   devs;
 private boolean    isScores = false;
 private boolean    isProfiles = false;
 private int NF = GLOBAL_NF;
    
 public ProfileCluster() 
 {
   set_error_status();  
 }

 public ProfileCluster(double[] mean, double[] dev) 
 {
   set_error_status();
   setParams(GLOBAL_NF, mean, dev);
 }
 
 public ProfileCluster(int _NF, double[] mean, double[] dev) 
 {
   set_error_status();
   setParams(_NF, mean, dev);
 }
 
 public void add(Profile profile, double score)
 {
   if (profile != null && !profile.get_error_status()) 
   {
    IDscore.put(profile.get_Long_ID(), new Double(score));
    IDbody.put(profile.get_Long_ID(), profile);
    isScores = false;
    isProfiles = false;        
   }
 }
  
 public int getSize()
 {
   return IDbody.size(); 
 }
 
 public Profile getProfile(long profileID)
 {
   if (get_error_status()) return null;
   return (Profile)IDbody.get(new Long(profileID));
 }

 public Profile getProfile(Long profileID)
 {
   if (get_error_status() || profileID == null) return null;
   return (Profile)IDbody.get(profileID);
 }

 public Profile getProfile(int i)
 {
   if (get_error_status() || i<0 || i>=getSize()) return null;
   if (!isProfiles)
    {
     Object[] os = IDbody.values().toArray();
     int n = os.length;
     if (n>0)
      {
       profiles = new Profile[n];
       for (int j=0; j<n; j++) profiles[j] =  (Profile)os[j];
      }
     isProfiles = true;
    }
   return profiles[i];
 } 
 
 public double getScore(long profileID)
 {
   if (get_error_status()) return 0;
   return ((Double)IDscore.get(new Long(profileID))).doubleValue();
 }

 public double getScore(Long profileID)
 {
   if (get_error_status()) return 0;
   return ((Double)IDscore.get(profileID)).doubleValue();
 }
  
 public double getScore(int i)
 {
   if (get_error_status() || i<0 || i>=getSize()) return 0;
   if (!isScores)
    {
     Object[] os = IDscore.values().toArray();
     int n = os.length;
     if (n>0)
      {
       scores = new Double[n];
       for (int j=0; j<n; j++) scores[j] =  (Double)os[j];
      }
     isScores = true;
    }
   return scores[i].doubleValue();
 }
 
 
 public Collection getProfiles()
 {
   if (get_error_status()) return null;
   return IDbody.values();
 }
 
 public void setNF(int _NF)
 {
   if (_NF <= 0) NF = GLOBAL_NF;
   else NF = _NF;
 }
 
 public void setParams(int _NF, double[] mean, double[] dev)
 {
   setNF(_NF);
   if (mean == null || dev == null || mean.length != NF || dev.length != NF) set_error_status();
   else
   {
    means = mean; 
    devs = dev; 
    isScores = false;
    isProfiles = false;    
    this.unset_error_status();
   }   
 }
    
 public double getMean(int i)
 {
   if (get_error_status() || i < 0 || i>= NF) return 0;
   return means[i];
 }
 
 public double getDev(int i)
 {
   if (get_error_status() || i < 0 || i>= NF) return 0;
   return devs[i];
 }
 
}
