package search;

import java.util.*;
import java.lang.*;

public class deltaXY extends Error
{
    private Result_List result_list;
    private Feedback feedback;
    private int NF = GLOBAL_NF;
    private double[] posCentroid = new double[NF];
    private double[] negCentroid = new double[NF];
    
    public deltaXY(Result_List _result_list, Feedback _feedback) 
    {
      if (_result_list != null && _feedback != null && !_feedback.get_error_status() && !_result_list.get_error_status())
       {
         result_list = _result_list;
         feedback = _feedback;
         posCentroid = result_list.getPosCentroid(feedback);
         negCentroid = result_list.getNegCentroid(feedback);
       }
      else
        set_error_status();
    }
    
    public void set_NF(int _NF)
    {
      if (_NF>0)
       {
         NF = _NF;          
         posCentroid = new double[NF];
         negCentroid = new double[NF];
       }
    }
        
    public double[] unit()
    {
      double[] d = new double[NF];
      Arrays.fill(d,1.0);
      return d;
    }
    
    public double[] computeProfile()
    {
      if (get_error_status()) return unit();  
      for (int i=0; i<NF; i++) set_error_message("centroids: p>"+posCentroid[i]+" n>"+negCentroid[i]);          
      if (get_error_status()) return null;  
      double[] solution = new double[NF];
      double[] dXY = new double[NF];
      double[] kXY = new double[NF];
      int[] order  = new int[NF];
      
      //compute dxy
      double mindxy = 0;
      for (int i=0; i<NF; i++)
       {
         double dxy = posCentroid[i]-negCentroid[i];
         dXY[i] = dxy;  
         if (Math.abs(dxy) > 0 && (Math.abs(dxy) < mindxy || mindxy == 0)) mindxy = Math.abs(dxy);
       }
      if (mindxy <= 0) mindxy = 1;
       else mindxy = mindxy * 0.9999999;
      
      
      double[] avg = result_list.getAverage();
      
      //compute coefficient kXY_i = Ratio_i*Dxy_i/Average_i
      for (int i=0; i<NF; i++)
       {
         Integer I = new Integer(i);
         double dxy = dXY[i];
         double kxy = 1;         
         if (dxy > 0)
           if (negCentroid[i] > 0) kxy =  1.0*posCentroid[i]/negCentroid[i];
            else kxy = 1000000;
         else if (dxy < 0) 
          if (posCentroid[i] > 0)  kxy = -1.0*negCentroid[i]/posCentroid[i];
           else kxy = -1000000;
         if (dxy != 0) kxy = kxy * dxy / avg[i];//Math.log(Math.abs(dxy)/mindxy);
         kXY[i] = kxy;
         order[i] = i;
       }
      
      
      //sort by coefficient kXY
      for (int i=0; i<NF-1; i++)
      {
        int imax = i;   
        for (int j = i+1; j<NF; j++)
          if (kXY[order[imax]] < kXY[order[j]]) imax = j;
        int x = order[i];
        order[i] = order[imax];
        order[imax] = x;
      }
       

      double sum_pos = 0;
      double sum_neg = 0;
      double max = kXY[order[0]];
      double min = kXY[order[NF-1]];
      max = Math.max(Math.abs(max), Math.abs(min));
      if (max == 0) max = 1;
      
      //distribute the weights among search criteria
      for (int i=0; i<NF; i++)
       {
         double kxy = kXY[order[i]];
         int n = order[i];

         double dxy = dXY[n];
         double x1 = -1*sum_pos/posCentroid[n];
         double x2 = -1*sum_neg/negCentroid[n];
         double x3 = (sum_neg - sum_pos)/dxy;
         double xmin = -1;
         double xmax = 1;
         if (dxy > 0)
          {
            xmax = kxy/max;  
            double xt = Math.max(x1, x2);
            xt = Math.max(xt, x3);
            xmin = Math.max(xt, xmin);
            solution[n] = Math.max(xmin, xmax);
          }
         else
         if (dxy < 0)    
          {
            xmin = kxy/max; 
            double xt = Math.max(x1, x2);
            xmax = Math.min(x3, xmax);
            xmin = Math.max(xt, xmin);
            solution[n] = Math.min(xmin, xmax);
          }
         else 
            solution[n] = 0;  
         
         sum_pos += solution[n]*posCentroid[n];
         sum_neg += solution[n]*negCentroid[n];
        }
      
      return solution;
    }        
    
    
    
}
