package search;

import java.util.*;
import java.lang.*;
import java.sql.*;
import oracle.jdbc.dbaccess.*; 
import oracle.jdbc.driver.*;
import oracle.sql.*;
import java.io.*;


public class profileDB extends KernelDB
{
    
 
 public boolean checkFeedback(long feedbackID)
 {
    if (get_error_status()) return false; 
    try
    {
     String SQLquery = "select feedbackID from "+FEEDBACK_TABLE+" where feedbackID = ?";  
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, feedbackID);
     rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("checkFeedback error \n"+e.toString());
     return false;
    }
    return false;
 } 
 
 public boolean storeGlobalProfile(GlobalProfile profile)
 {
   if (get_error_status() && profile==null && profile.get_error_status()) return false;
   try
    {
     if (!storeProfile(profile)) return false;
     long profileID = profile.getID(); 
     for (int i=0; i<profile.cluster.getSize(); i++)
      {
       Profile _profile = profile.cluster.getProfile(i);
       if (_profile!=null && !_profile.get_error_status())
        {
         String SQLquery = "insert into "+this.GLOBAL_PROFILE_TABLE+" values(?, ?, ?)";  
         SqlForm = oracle_conn.prepareStatement(SQLquery);
         SqlForm.setLong(1, profileID);
         SqlForm.setLong(2, _profile.getID());
         SqlForm.setDouble(3, profile.cluster.getScore(i));
         rset = SqlForm.executeQuery();
         rset.close();
         SqlForm.close();
      }}
      return true;
    }
    catch (Exception e)
    {
     set_error_message("storeGlobalProfile error \n"+e.toString());
    }
    return false;             
 }
 
 public void store_all_global_profiles_as_blob()
 {
   if (this.get_error_status() || this.GLOBAL_PROFILE_TABLE == null) return ;
   try
   {
    for (int i=0; i<this.GLOBALPROFILES.length; i++)
    {
     String SQLquery = "insert into "+this.GLOBAL_PROFILE_BLOB_TABLE+"(profileID, feedback) values(?, empty_blob())";
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, GLOBALPROFILES[i].getID());
     SqlForm.executeUpdate();
     SqlForm.close();
     oracle_conn.commit();
      
     SQLquery = "select profile from "+this.GLOBAL_PROFILE_TABLE+" where profileID = ? for UPDATE";
     oracle_conn.setAutoCommit(false);
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, GLOBALPROFILES[i].getID());
     ResultSet rset = SqlForm.executeQuery();
     rset.next();
     oracle.sql.BLOB blob = ((OracleResultSet)rset).getBLOB(1);
     OutputStream blobOut = ((BLOB)blob).getBinaryOutputStream();
     ObjectOutputStream outstream = new ObjectOutputStream( blobOut );
     outstream.writeObject(GLOBALPROFILES[i]);
     outstream.flush();
     blobOut.flush();
     blobOut.close();
     rset.close();
     SqlForm.close();
     oracle_conn.commit();
     oracle_conn.setAutoCommit(true);   
    }
   }
   catch (Exception e)
    {
     set_error_message("store_all_global_profiles_as_blob error \n"+e.toString());
    }
 }
 
 public GlobalProfile[] get_all_global_profiles()
 {
    if (get_error_status()) return null;
    try
    {     
     String SQLquery = "select count(*) from "+this.GLOBAL_PROFILE_BLOB_TABLE;
     PreparedStatement SqlForm = oracle_conn.prepareStatement(SQLquery);
     ResultSet rset = SqlForm.executeQuery();
     int n = 0;
     if (rset.next()) n = rset.getInt(1);
       else return null;
     GlobalProfile[] temp = new GlobalProfile[n];
     rset.close();
     SqlForm.close();
     
     SQLquery = "Select * from "+this.GLOBAL_PROFILE_BLOB_TABLE;
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     rset = SqlForm.executeQuery();
     int i=0;
     while (rset.next() && i<n)
     {
      oracle.sql.BLOB blob = ((OracleResultSet)rset).getBLOB(1);
      ObjectInputStream ois = new ObjectInputStream(blob.getBinaryStream());
      temp[i] = (GlobalProfile)ois.readObject();
      ois.close();
      rset.close();
      SqlForm.close();
      i++;
     }
     return temp;
    }
    catch (Exception e)
    {
     set_error_message("get_all_global_profiles error \n"+e.toString());
    }
    return null;
 }
 
 public boolean storeProfile(GlobalProfile profile)
 {
    if (get_error_status() && profile==null && profile.get_error_status()) return false; 
    try
    {
     
     String SQLquery = "insert into "+PROFILE_TABLE+" values(?";
     for (int i=0; i<GLOBAL_NF; i++)  SQLquery += ", ?";
     SQLquery += ", ?)";
     set_error_message(SQLquery);
     
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profile.getID());
     for (int i=0; i<GLOBAL_NF; i++)
       SqlForm.setDouble(i+2, profile.getWeight(i));
     java.sql.Date   date    = new java.sql.Date(new java.util.Date().getTime());
     SqlForm.setDate(GLOBAL_NF+2, date);
     ResultSet rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("storeProfile error \n"+e.toString());
    }
    return false;          
 }
 
 public Profile[] getAllDistinctProfiles()
 {
    if (get_error_status()) return null; 
    try
    {
     String SQLquery = "select count(*) from "+PROFILE_TABLE+" t1, "+this.DISTINCT_PROFILE_TABLE+" t2 where t1.profileid = t2.profileid";  
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     rset = SqlForm.executeQuery();
     int n = 0;
     if (rset.next()) n = rset.getInt(1);     
     Profile[] profiles = new Profile[n];
     rset.close();
     SqlForm.close();
     
     SQLquery = "select t1.* from "+PROFILE_TABLE+" t1, "+this.DISTINCT_PROFILE_TABLE+" t2 where t1.profileid = t2.profileid";   
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     rset = SqlForm.executeQuery();
     int k = 0;
     while (rset.next())
      {
        long profileID = rset.getLong(1);
        double[] w = new double[GLOBAL_NF];
        for (int i=0; i<GLOBAL_NF; i++)
          w[i] = rset.getDouble(i+2);        
        Profile profile = new Profile(profileID, w);
        profiles[k] = profile;
        k++;
      }
     rset.close();
     SqlForm.close();
     return profiles;
    }
    catch (Exception e)
    {
     set_error_message("getDistinctProfile error \n"+e.toString());
    }
    return null;
 } 

 public boolean removeAllGlobalProfiles()
 {
  if (get_error_status()) return false; 
    try
    {
     String SQLquery = "delete from "+PROFILE_TABLE+" where profileid in (select distinct(profileid) from "+this.GLOBAL_PROFILE_TABLE+")";  
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     rset = SqlForm.executeQuery();
     rset.close();
     SqlForm.close();
     return true;
    }
    catch (Exception e)
    {
     set_error_message("delete all global profiles error\n"+e.toString());
    }
    return false;
 }
 
 public Profile[] getAllGlobalProfiles()
 {
    if (get_error_status()) return null; 
    try
    {
     String SQLquery = "select count(t1.*) from "+PROFILE_TABLE+" t1, "+this.GLOBAL_PROFILE_TABLE+"t2 where t1.profileid = t2.profileid";  
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     rset = SqlForm.executeQuery();
     int n = 0;
     if (rset.next()) n = rset.getInt(1);     
     Profile[] profiles = new Profile[n];
     rset.close();
     SqlForm.close();
     
     SQLquery = "select t1.* from "+PROFILE_TABLE+" t1, "+this.GLOBAL_PROFILE_TABLE+"t2 where t1.profileid = t2.profileid";   
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     rset = SqlForm.executeQuery();
     int k = 0;
     while (rset.next())
      {
        long profileID = rset.getLong(1);
        double[] w = new double[GLOBAL_NF];
        for (int i=0; i<GLOBAL_NF; i++)
          w[i] = rset.getDouble(i+2);        
        Profile profile = new Profile(profileID, w);
        profiles[k] = profile;
        k++;
      }
     rset.close();
     SqlForm.close();
     return profiles;
    }
    catch (Exception e)
    {
     set_error_message("getDistinctProfile error \n"+e.toString());
    }
    return null;
 }  
 
 public boolean isGlobalProfile(long profileID)
 {
    if (get_error_status()) return false; 
    try
    {
     String SQLquery = "select * from "+PROFILE_TABLE+" t1, "+GLOBAL_PROFILE_TABLE+" t2 where t1.profileID = ? and t1.profileId = t2.profileID";  
     SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profileID);
     rset = SqlForm.executeQuery();
     if (rset.next())
      {
        rset.close();
        SqlForm.close();
        return true;
       }
     rset.close();
     SqlForm.close();
    }
    catch (Exception e)
    {
     set_error_message("isGlobalProfile error \n"+e.toString());
    }
    return false;          
 }
 
 public Profile getProfile(long profileID)
 {
    if (get_error_status()) return null; 
    try
    {
     String SQLquery = "SELECT w0";
     for (int i=1; i<GLOBAL_NF; i++)
         SQLquery += ", w"+i;
     SQLquery += " FROM "+PROFILE_TABLE+" WHERE profileID = ?";  

     SqlForm = oracle_conn.prepareStatement(SQLquery);
     SqlForm.setLong(1, profileID);
     rset = SqlForm.executeQuery();
     if (rset.next())
      {
        double[] p_vector = new double[GLOBAL_NF];  
        for (int i=0; i<GLOBAL_NF; i++)
         p_vector[i] = rset.getDouble(i+1);
        
        Profile profile = new Profile(profileID, p_vector);
        
        rset.close();
        SqlForm.close();
        return profile;
       }
     else
     {   
      rset.close();
      SqlForm.close();
     }
    }
    catch (Exception e)
    {
     set_error_message("get_profile_2 error \n"+e.toString());
    }
    return null;     
 }
 
  public boolean createFeedbackCluster(FeedbackCluster cluster)
 {
  if (!this.get_error_status() && cluster!=null && !cluster.get_error_status())
   {
     try 
     {
       Long profileID = cluster.getProfileID(); 
       if (!isGlobalProfile(profileID.longValue())) return false;
       String SQLquery = "select docid, status from "+this.FEEDBACK_DOCS_TABLE+" fd, "+this.DISTINCT_PROFILE_TABLE+" dp, "+this.GLOBAL_PROFILE_TABLE+" gp where gp.profileID = ? and gp.DISPROFILEID = dp.profileID and dp.feedbackID = fd.feedbackID";  
       SqlForm = oracle_conn.prepareStatement(SQLquery);
       SqlForm.setLong(1, profileID.longValue());
       rset = SqlForm.executeQuery();
       while (rset.next())
       {
         Long docid = new Long(rset.getLong(1));
         int status = rset.getInt(2);
         cluster.add(docid, status);
       }
       return true;
     }
     catch (Exception e)
     {
       set_error_message("feedbcakCluster "+e.toString());  
     }
   }
  return false;   
 }


}