/**
 * SEEV 1.0 an advanced search engine for BINGO!
 * @author  Eske Vladimir
 * 2003
 * @address eske@cs.uni-sb.de
 * All rights reserved
 * Not for commercial use
 */

package search.test;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.net.*;
import search.*;

public class TestThread extends search.Error implements Runnable
{
private Vector terms;
private boolean status = true;
private Date D = new Date();
private testDB DB;
private Object[] keys1;
private Object[] keys2;
private Feedback[] feedbacks;
private static int MAX_QUERY_REQUESTS = 50;
    
    public TestThread(Object[] C1, Object[] C2, Feedback[] _feedbacks) 
    {
     DB = new testDB("account.dat");  
      if (C1 == null || C2 == null || _feedbacks == null) set_error_status();
      else
      {
       keys1 = C1;
       keys2 = C2;
       feedbacks = _feedbacks;
      }
    }
    
    public void run() 
    {  
      set_error_message("Thread started at "+D.getTime());          
      if (!get_error_status())
      {
       int n = keys1.length;
       if (n > this.MAX_QUERY_REQUESTS) n = this.MAX_QUERY_REQUESTS;

       //query_test(keys1, 1, n);
       //query_test(keys1, 2, n);
       //query_test(keys1, 3, n);

       for (int i=0; i<feedbacks.length; i++)
         feedback_test(feedbacks[i], 10);
      }
     set_error_status();
     DB.CloseConnect();
     synchronized (this) {this.notify();}
     set_error_message("Thread stop "+D.getTime()); 
    }
    
    private void feedback_test(Feedback feedback, int num_reuqests)
    {
      if (feedback == null || feedback.get_error_status() || num_reuqests <= 0) return;
      set_error_message("Feedback test started at"+D.getTime());
      int pos = feedback.getNumPos();
      int neg = feedback.getNumNeg();
      Random PR = new Random();
      Random NR = new Random();
      for (int i=0; i<num_reuqests; i++)
       {
         int p = 0; if (pos > 0) PR.nextInt(pos);
         int n = 0; if (neg > 0) NR.nextInt(neg);
         
         if (pos > 0 && p == 0) p = 1;
         if (neg > 0 && n == 0) n = 1;
         
         int[] pos_seeds = new int[p];
         int[] neg_seeds = new int[n];
         
         int k = 0;
         for (int j=0; j<p; j++)
          {
            pos_seeds[j] = k + PR.nextInt(pos - p + j - k + 1);
            k = pos_seeds[j] + 1;
          }  
         
         k = 0;
         for (int j=0; j<n; j++)
          {
            neg_seeds[j] = k + PR.nextInt(neg - n + j - k + 1);
            k = neg_seeds[j] + 1;
          }

         try
         { 
            String url = createFeedbackRequest(pos_seeds, neg_seeds, feedback);   
            synchronized (this) {this.wait(1000);}    
            URL U = new URL(url);
            Date D1 = new Date();
            HttpURLConnection HUC = (HttpURLConnection)U.openConnection();
            if (HUC.getResponseCode()==200)
             {
               InputStream IS = HUC.getInputStream();
               InputStreamReader BF = new InputStreamReader(IS);
               BF.close();
               IS.close();
               Date D2 = new Date();
               this.storeFeedbackResults(url, D1.getTime(), D2.getTime());
             }

           }
           catch (Exception e)
           {
            set_error_status();   
            synchronized (this) {this.notify();}
            set_error_message("testThread feedback\n"+e.toString());   
           }         
       }
    }
    
    private void query_test(Object[] keys, int keys_per_query, int num_requests)
    {
      if (keys == null || keys_per_query <= 0 || num_requests <= 0) return;
      set_error_message("Query test started at"+D.getTime());
      for (int i=0; i<num_requests; i++)          
      {
        Random R = new Random();
        Object[] seeds = new Object[keys_per_query];
        int n = keys.length;
        for (int j=0; j<keys_per_query; j++)
         {
           int l = R.nextInt(n); 
           seeds[j] = keys[l];
         }
         try
         { 
            String url = createRequest(seeds);   
            synchronized (this) {this.wait(200);}    
            URL U = new URL(url);
            Date D1 = new Date();
            HttpURLConnection HUC = (HttpURLConnection)U.openConnection();
            if (HUC.getResponseCode()==200)
             {
               InputStream IS = HUC.getInputStream();
               InputStreamReader BF = new InputStreamReader(IS);
               BF.close();
               IS.close();
               Date D2 = new Date();
               this.storeQueryResults(url, D1.getTime(), D2.getTime());
             }

           }
           catch (Exception e)
           {
            set_error_status();   
            synchronized (this) {this.notify();}
            set_error_message("testThread query\n"+e.toString());   
           }
       }
    }
        
    private String createRequest(Object[] keys)
    {
     if (keys == null) return null;   
     String basePrefix  = "http://139.19.64.154:9090/seev2/jsp/index.jsp?queryStr=";
     String basePostfix = "&submit_query=Submit";
     String temp = basePrefix+keys[0].toString();;
     for (int i=1; i< keys.length; i++)
       temp += "+" + keys[i].toString();
     temp += basePostfix;
     return temp;
    }
    
    private String createFeedbackRequest(int[] pos_seeds, int[] neg_seeds, Feedback feedback)
    {
      if (pos_seeds == null || neg_seeds == null || feedback == null) return null;
      String basePrefix  = "http://139.19.64.154:9090/seev2/jsp/index.jsp?IDF="+feedback.get_sessionID();
      String basePostfix = "&submit_Feedback=Submit";
      String temp = "";
      String tempPrefix = "&docFeedback";
      int k = 0;
      
      for (int i=0; i<pos_seeds.length; i++)
       { 
         temp += tempPrefix + k +"="+ this.POSITIV_FEEDBACK+feedback.get_pos_doc(pos_seeds[i]); 
         k++;
       }

      for (int i=0; i<neg_seeds.length; i++)
       {
         temp += tempPrefix + k +"="+ this.NEGATIV_FEEDBACK+feedback.get_neg_doc(neg_seeds[i]);
         k++;
       }

      temp = basePrefix+temp+basePostfix; 
      return temp;
    }
    
    private void storeQueryResults(String url, long d_1, long d_2)
    {
       String query = "insert into SEEV_QueryTestResults values('"+url+"', "+d_1+", "+d_2+")";
       DB.insert_query(query);
    }
    
    private void storeFeedbackResults(String url, long d_1, long d_2)
    {
       String query = "insert into SEEV_FeedbackTestResults values('"+url+"', "+d_1+", "+d_2+")";
       DB.insert_query(query);
    }
}
