package org.unisb.dbs.util;

import java.util.*;
import java.lang.*;
import java.io.*;

import net.sf.snowball.*;

import java.lang.reflect.Method;

public class StemmerDriver {
  
  private SnowballProgram stemmer;
  private Method stemMethod;
  private Object[] emptyArgs = new Object[0];;
  private int repeat = 1;
  
  private static Hashtable stopwords = null;
  private static Class stemClass = null;
  
  public static String stemmerLanguage = "english";
  
  public StemmerDriver() {
    try {
      if ( stopwords == null ) {
        importStopwords();
      }
      if ( stemClass == null ) {        
        stemClass = Class.forName( "net.sf.snowball.ext." + stemmerLanguage + "Stemmer" );
      }
      stemmer = (SnowballProgram) ( stemClass.newInstance() );
      stemMethod = stemClass.getMethod( "stem", new Class[0] );
    }
    catch ( Exception e ) {
      e.printStackTrace();
    }
    
  }
  
  
  public String prepareString( String from ) {
    String tmp = from;
    
    tmp = tmp.replaceAll( "", "ae" );
    tmp = tmp.replaceAll( "", "ue" );
    tmp = tmp.replaceAll( "", "oe" );
    tmp = tmp.replaceAll( "", "ss" );
    tmp = tmp.replaceAll( "", "ae" );
    tmp = tmp.replaceAll( "", "ue" );
    tmp = tmp.replaceAll( "", "oe" );
    
    tmp = tmp.replaceAll( "\\W", " " );// alle non-word raus
    tmp = tmp.toLowerCase().trim();
    
    return tmp;
  }
  
  
  public String[] stem( char[] data ) {
    return stem( new String( data ) );
  }
  
  
  public String[] stem( String data ) {
    
    String tmp = prepareString( new String( data ) );
    Vector resultWords = new Vector();
    
    StringTokenizer tokenizer = getTokenizer( tmp );
    
    while ( tokenizer.hasMoreTokens() ) {
      String origin = tokenizer.nextToken();
      String word = origin;
      
      
      stemmer.setCurrent( word );
      
      for ( int i = repeat; i != 0; i-- ) {
        try {
          stemMethod.invoke( stemmer, emptyArgs );
        }
        catch ( Exception e ) {
          e.printStackTrace();
        }
      }
      
      word = stemmer.getCurrent();
      
      if ( word != null && word.length() > 0 && !isStopword( origin ) ) {
        resultWords.add( word );
      }
    }
    String[] stems = new String[ resultWords.size() ];
    resultWords.copyInto( stems );
    return stems;
  }
  
  
  public StringTokenizer getTokenizer( String tmp ) {
    return new StringTokenizer( tmp, " \t\n\r\f\"'.,?!:;+*-=<>|/\\_()[]{}&%$~^#" );
  }
  
  public boolean isStopword( String s ) {
    return ( stopwords.get( s ) != null );
  }
  
  private void importStopwords() throws Exception {
    BufferedReader buf = new BufferedReader( new InputStreamReader( this.getClass().getClassLoader().getResource("org/unisb/dbs/util/stopwords.txt").openStream() ) );
    String tmp;
    StringTokenizer st = null;

    stopwords = new Hashtable();
    
    while ( ( tmp = buf.readLine() ) != null ) {
      tmp = prepareString( tmp );
      st = getTokenizer( tmp );
      
      while ( st.hasMoreTokens() ) {
        String token = st.nextToken().toLowerCase().trim();
        // token = stem( token );
        if ( token != null && token.length() > 0 ) {
          stopwords.put( token, token );
        }
      }
    }
    buf.close();
    System.out.println( stopwords.size() + " STOPWORDS." );
  }
 
  public static void main( String[] s ) {
    new StemmerDriver();
  }
  
}
