--- ========================= 
--- DB table: AnchorTerms 
--- C-Schema: AnchorTerms 
--- ========================= 

CREATE TABLE AnchorTerms (
    Term               varchar2(200)  NOT NULL,
    ID                 integer        DEFAULT 0 NOT NULL
)
;

--- indices 

CREATE INDEX AnchorTerm_IndexTerm
    ON AnchorTerms (Term)
;

CREATE INDEX AnchorTerm_IndexID
    ON AnchorTerms (ID)
;




--- ========================= 
--- DB table: ArchiveDocuments 
--- C-Schema: ArchiveDocuments 
--- ========================= 

CREATE TABLE ArchiveDocuments (
    ID                 integer        NOT NULL,
    URL                varchar2(3000) NOT NULL,
    Expiration         number         ,
    Hostname           varchar2(255)  ,
    IP                 varchar2(30)   DEFAULT '127.0.0.1' NOT NULL,
    Hashcode           integer        ,
    Port               integer        DEFAULT 80 ,
    Filename           varchar2(3000) ,
    Lastmod            number         ,
    mime               varchar2(300)  NOT NULL,
    Responsecode       integer        DEFAULT 200 ,
    Contentencoding    varchar2(300)  DEFAULT 'text/html' ,
    Title   varchar2(255),
    PRIMARY KEY (ID) 
)
;

--- indices 

CREATE INDEX Archivedocs_ResponsecodeIndex
    ON ArchiveDocuments (Responsecode)
;


--- ================================================
--- DB Table: hasForms
--- C-Schema: hasForms
--- Tabelle zum Abspeichern von Formularmarkierungen
--- ================================================

CREATE TABLE hasForms (
	ID	integer		not null,	
	PRIMARY KEY (ID),
	FOREIGN KEY (ID) REFERENCES archivedocuments
		ON DELETE CASCADE);


--- ========================= 
--- DB table: ArchiveFiles 
--- C-Schema: ArchiveFiles 
--- ========================= 

CREATE TABLE ArchiveFiles (
    Source             BLOB           DEFAULT empty_blob() ,
    ID                 integer        NOT NULL
);

--- indices 

CREATE UNIQUE INDEX ArchiveFiles_IndexID
    ON ArchiveFiles (ID)
;

--- ========================= 
--- DB table: ClassFeatures 
--- C-Schema: ClassFeatures 
--- ========================= 

CREATE TABLE ClassFeatures (
    term               varchar2(200)  NOT NULL,
    MI                 float          DEFAULT 0 ,
    IG                 float          DEFAULT 0 ,
    IDF                float          DEFAULT 0 ,
    Class_ID           integer        
)
;




--- ========================= 
--- DB table: Classes 
--- C-Schema: Classes 
--- ========================= 

CREATE TABLE Classes (
    Class_ID           integer        NOT NULL,
    Name               varchar2(500)  NOT NULL,
    PRIMARY KEY (Class_ID) 
)
;

--- indices 

CREATE UNIQUE INDEX Classes_Name
    ON Classes (Name)
;




--- ========================= 
--- DB table: Classifier 
--- C-Schema: Classifier 
--- ========================= 

CREATE TABLE Classifier (
    timestamp          date           DEFAULT sysdate ,
    classifier         BLOB           DEFAULT empty_blob() ,
    Step               integer        DEFAULT 0 ,
    Type               integer        DEFAULT -1 
)
;




--- ========================= 
--- DB table: ClusterElements 
--- C-Schema: ClusterElements 
--- ========================= 

CREATE TABLE ClusterElements (
    Cluster_id         integer        NOT NULL,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;




--- ========================= 
--- DB table: Clusters 
--- C-Schema: Clusters 
--- ========================= 

CREATE TABLE Clusters (
    Cluster_id         integer        NOT NULL,
    Annotation         varchar2(300)  ,
    Medoid             integer        NOT NULL,
    PRIMARY KEY (Cluster_id) ,
    FOREIGN KEY (Medoid) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;




--- ========================= 
--- DB table: CrawlDocuments 
--- C-Schema: CrawlDocuments 
--- ========================= 

CREATE TABLE CrawlDocuments (
    status             char           ,
    depth              integer        DEFAULT 0 NOT NULL,
    origin             integer        DEFAULT -1 ,
    HubScore           float          DEFAULT 0 ,
    AuthScore          float          DEFAULT 0 ,
    SvmScore           float          DEFAULT 0 ,
    ID                 integer        NOT NULL,
    class_id           integer        ,
    Clicks             number         DEFAULT 0 ,
    POSevaluation      number         DEFAULT 0 ,
    NEGevaluation      number         DEFAULT 0 ,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE UNIQUE INDEX CrawlDocuments_IndexID
    ON CrawlDocuments (ID)
;

CREATE INDEX CrawlDocuments_IndexStatus
    ON CrawlDocuments (status)
;

CREATE INDEX CrawlDocuments_IndexDepth
    ON CrawlDocuments (depth)
;

CREATE INDEX CrawlDocuments_IndexOrigin
    ON CrawlDocuments (origin)
;

CREATE INDEX CrawlDocuments_IndexClassId
    ON CrawlDocuments (class_id)
;




--- ========================= 
--- DB table: DocumentFeatures 
--- C-Schema: DocumentFeatures 
--- ========================= 

CREATE TABLE DocumentFeatures (
    Term               varchar2(200)  NOT NULL,
    RTF                number         DEFAULT 0 ,
    TF                 number         DEFAULT 0 ,
    TF_IDF             number         DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE INDEX DocumentFeatures_IndexTerm
    ON DocumentFeatures (Term)
;

CREATE INDEX DocumentFeatures_IndexID
    ON DocumentFeatures (ID)
;




--- ========================= 
--- DB table: DocumentUnion 
--- C-Schema: DocumentUnion 
--- ========================= 

CREATE TABLE DocumentUnion (
    term               varchar2(200)  NOT NULL,
    RTF                float          DEFAULT 0 ,
    TF                 float          DEFAULT 0 ,
    TF_IDF             float          DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE UNIQUE INDEX DocumentUnion_IndexID
    ON DocumentUnion (ID)
;

CREATE INDEX DocumentUnion_IndexTerm
    ON DocumentUnion (term)
;




--- ========================= 
--- DB table: Feedback 
--- C-Schema: Feedback 
--- ========================= 

CREATE TABLE Feedback (
    iteration          integer        DEFAULT 0 ,
    result             integer        DEFAULT -1 ,
    class_old          integer        NOT NULL,
    class_new          integer        DEFAULT -1 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE UNIQUE INDEX Feedback_IndexID
    ON Feedback (ID)
;




--- ========================= 
--- DB table: FeedbackArchive 
--- C-Schema: FeedbackArchive 
--- ========================= 

CREATE TABLE FeedbackArchive (
    summary            CLOB           DEFAULT empty_clob() ,
    class              integer        NOT NULL,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE UNIQUE INDEX FeedbackArchive_IndexID
    ON FeedbackArchive (ID)
;




--- ========================= 
--- DB table: GoogleHubs 
--- C-Schema: GoogleHubs 
--- ========================= 

CREATE TABLE GoogleHubs (
    source             CLOB           DEFAULT empty_clob() ,
    lastmod            date           DEFAULT sysdate ,
    query              varchar2(1000) ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE UNIQUE INDEX GoogleHubs_IndexID
    ON GoogleHubs (ID)
;




--- ========================= 
--- DB table: GoogleRelated 
--- C-Schema: GoogleRelated 
--- ========================= 

CREATE TABLE GoogleRelated (
    source             CLOB           DEFAULT empty_clob() ,
    lastmod            date           DEFAULT sysdate ,
    query              varchar2(1000) ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE UNIQUE INDEX GoogleRelated_IndexID
    ON GoogleRelated (ID)
;




--- ========================= 
--- DB table: Links 
--- C-Schema: Links 
--- ========================= 

CREATE TABLE Links (
    Target             varchar2(3000) NOT NULL,
    Link_ID            integer        DEFAULT null ,
    Hostname           varchar2(255)  NOT NULL,
    ID                 integer        ,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE INDEX Links_IndexID
    ON Links (ID)
;

CREATE INDEX Links_IndexLink_ID
    ON Links (Link_ID)
;




--- ========================= 
--- DB table: MimeTypes 
--- C-Schema: MimeTypes 
--- ========================= 

CREATE TABLE MimeTypes (
    mime               varchar2(300)  NOT NULL,
    application        varchar2(200)  ,
    extension          varchar2(10)   ,
    maxsize            float          DEFAULT 1048576 ,
    handler            varchar2(200)  NOT NULL,
    PRIMARY KEY (mime) 
)
;




--- ========================= 
--- DB table: PairFeatures 
--- C-Schema: PairFeatures 
--- ========================= 

CREATE TABLE PairFeatures (
    pair               varchar2(400)  NOT NULL,
    RPF                float          DEFAULT 0 ,
    PF                 float          DEFAULT 0 ,
    PF_IPF             float          DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE INDEX PairFeatures_IndexID
    ON PairFeatures (ID)
;

CREATE INDEX PairFeatures_IndexPair
    ON PairFeatures (pair)
;




--- ========================= 
--- DB table: Preferences 
--- C-Schema: Preferences 
--- ========================= 

CREATE TABLE Preferences (
    lastmod            date           DEFAULT sysdate ,
    Value              varchar2(100)  NOT NULL,
    Name               varchar2(300)  NOT NULL
)
;




--- ========================= 
--- DB table: Stopwords 
--- C-Schema: Stopwords 
--- ========================= 

CREATE TABLE Stopwords (
    Term               varchar2(200)  NOT NULL
)
;




--- ========================= 
--- DB table: TempDocuments 
--- C-Schema: TempDocuments 
--- ========================= 

CREATE TABLE TempDocuments (
    Temp_ID            varchar2(10)   ,
    ID                 varchar2(10)   ,
    Class_ID           varchar2(10)   ,
    Prop_Class_ID      varchar2(10)   ,
    Status             char           
)
;




--- ========================= 
--- DB table: TempQuery 
--- C-Schema: TempQuery 
--- ========================= 

CREATE TABLE TempQuery (
    SessionID          varchar2(255)  NOT NULL,
    ANDquery           varchar2(3000) ,
    ORquery            varchar2(3000) ,
    PRIMARY KEY (SessionID) 
)
;




--- ========================= 
--- DB table: TermPairs 
--- C-Schema: TermPairs 
--- ========================= 

CREATE TABLE TermPairs (
    Term_A             varchar2(200)  NOT NULL,
    Term_B             varchar2(200)  NOT NULL,
    Feature            varchar2(200)  NOT NULL,
    Distance           number         DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE INDEX TermPairs_IndexID
    ON TermPairs (ID)
;

CREATE INDEX TermPairs_IndexFeature
    ON TermPairs (Feature)
;




--- ========================= 
--- DB table: Terms 
--- C-Schema: Terms 
--- ========================= 

CREATE TABLE Terms (
    Term              varchar2(200)  NOT NULL,
    Pos                number         DEFAULT 0 ,
    Origin             varchar2(200),
    isStopword    varchar2(1)  DEFAULT 'F',
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES ArchiveDocuments
      ON DELETE CASCADE
)
;

--- indices 

CREATE INDEX Terms_IndexID
    ON Terms (ID)
;

CREATE INDEX Terms_IndexTerm
    ON Terms (Term)
;

CREATE INDEX Terms_IndexOrigin
    ON Terms (Origin)
;

--- foreign keys 

ALTER TABLE ClusterElements 
    ADD FOREIGN KEY (Cluster_id) REFERENCES Clusters
      ON DELETE CASCADE
;

create table documents
(
  STATUS        CHAR(1),
  HUBSCORE      FLOAT default 0.0001,
  AUTHSCORE     FLOAT default 0.0001,
  ID            NUMBER not null,
  URL           VARCHAR2(3000) not null,
  HOSTNAME      VARCHAR2(255),
  LASTDATE      NUMBER,
  OUTLINK       NUMBER,
  INLINK        NUMBER,
  TITLE         VARCHAR2(255),
  CLASSNAME     VARCHAR2(500) default 'UNKNOWN/',
  TEXT          VARCHAR2(4000),
  DOCSIZE       FLOAT default 0,
  PRIMARY KEY(ID) 
);

CREATE INDEX Documents_IndexID
    ON documents (ID)
;

CREATE TABLE feature(
    Term               varchar2(200)  NOT NULL,
    TF                 number         DEFAULT 0 ,
    TF_IDF             number         DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES Documents
      ON DELETE CASCADE
)
;

CREATE INDEX Feature_IndexID
    ON Feature (ID)
;

CREATE INDEX Feature_IndexTerm
    ON Feature (Term)
;

create table temp_user
(
 host varchar2(500) not null,
 sessionID integer not null,
 PRIMARY KEY (sessionID) 
)
;

CREATE INDEX TEMP_USER_IndexID
    ON temp_user (sessionid)
;

CREATE TABLE ORIGIN(
    Term               varchar2(200)  NOT NULL,
    TF                 number         DEFAULT 0 ,
    TF_IDF             number         DEFAULT 0 ,
    ID                 integer        NOT NULL,
    FOREIGN KEY (ID) REFERENCES Documents
      ON DELETE CASCADE
)
;


commit;

--- end of SQL 
