package schema;

import java.io.*;
import java.util.Enumeration.*;
import java.util.*;
import java.sql.*;
import oracle.jdbc.dbaccess.*;
import oracle.sql.*;
import oracle.jdbc.driver.*;
import java.lang.Math.*;


public class SchemaManager
{   
    private SessionBuffer session;
    private OracleConnection conn;
    private PreparedStatement SqlForm;
    private search.Error error; 

    public SchemaManager()
    {
      session = SessionBuffer.getInstance(true);
      error = new search.Error(session.userDir+ System.getProperty( "file.separator" ) +"schema_log.txt"); 
    }
    
    public void createSchema ()
    {
      try
      {  
               
        if (!openConnection()) System.exit(0);
        String dir = session.userDir;
        String path = dir + System.getProperty( "file.separator" ) + "schema" + System.getProperty( "file.separator" ) + "importer.sql";
        processFile (path);
        closeConnection();
        
        storeDescriptor();
      } 
      catch (Exception e)
      {e.printStackTrace();}
    }
    
    public void processFile (String myFile) throws Exception
    {
        BufferedReader reader = new BufferedReader (new FileReader(new File(myFile)));
        boolean finished = false;
        
        System.out.println("IMPORTER HAS STARTED");
        
        StringBuffer command = new StringBuffer();
        while (! finished)
        {
          String line = reader.readLine();
          if (line == null)
          {
             if (command.length() > 0) 
             {
               String befehl = command.toString().replaceAll(";","");
               try
               {
                 SqlForm = conn.prepareStatement(befehl);  
                 SqlForm.execute();                 
               }
               catch (Exception e)
               {
                e.printStackTrace();
               }
               SqlForm.close();
               System.out.println (error+"\n");
               befehl = null;
               command = null;
             }  
            finished = true;
          }    
          else if (line.trim().length() == 0 || line.startsWith("--"))
          {
             if (command.length() > 0) 
             {
               String befehl = command.toString().replaceAll(";","");
               try{
                   System.out.print (befehl);                   
                   SqlForm = conn.prepareStatement(befehl); 
                   SqlForm.execute();
                   System.out.println ("\n >> OK <<");
                  }
               catch (Exception e)
               {
                 error.set_error_message(e.toString());
                 if (befehl.toLowerCase().startsWith("drop") || befehl.toLowerCase().startsWith("create index")) 
                    System.out.println("\n >> BAD <<");
                 else
                   {
                    System.out.println("\n >> FATAL ERROR EXIT <<");
                    e.printStackTrace();
                    System.exit(0);
                   }
               }
               befehl = null;
               command = new StringBuffer();
               System.out.println("\n ========================= \n");
             }  
          } 
          else 
          {
            line = line.replaceAll ("\\$user\\$", session.defaultUser);
            line = line.replaceAll ("\\$password\\$", session.defaultUserPasswd);
            command.append(line);
            command.append (" ");
          }
        }    

    }
        
    private void closeConnection()
    {
      if (conn != null)  
      try{conn.close();}
      catch (Exception e)
      {
        e.printStackTrace();    
      }
      conn=null;
    }
    
    private boolean openConnection()
    {
     String path = "";   
     try 
     {   
      DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
      path="jdbc:oracle:thin:@(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST="+session.sHost+")(PORT=1521)))(CONNECT_DATA=(SERVICE_NAME="+session.sServiceName+")(SERVER=dedicated)))";
      conn = (OracleConnection)DriverManager.getConnection(path,session.defaultUser,session.defaultUserPasswd);   
      }
      catch(Exception e) 
      {
       e.printStackTrace();   
       error.set_error_message("dbconnect constructor S: "+path+"\n"+e.toString());       
       return false;
      }   
      return true;
    }
        
    void storeDescriptor() throws Exception
    {      
      String s = System.getProperty( "file.separator" );
      //File file = new File (session.userDir + s + "bingo" + s + "data" + s + "accounts.dat");
      File file = new File (error.ACCOUNT_PATH);
      String descriptor = session.defaultUser + ";" + session.defaultUserPasswd + ";" + session.sHost + ";" + session.sServiceName;
      BufferedWriter to = new BufferedWriter (new FileWriter (file));
      to.write (descriptor);
      to.flush(); 
      to.close();
    }
    
    public static void main(String[] s)
    {
      new SchemaManager().createSchema();
      System.exit(0);
    }
}
 