package search;

import java.lang.*;
import java.util.*;

public class FeedbackCluster extends Error
{    
 private HashMap    Dpos  = new HashMap();
 private HashMap    Dneg  = new HashMap();
 private HashSet    allDocs = new HashSet();
 private long       profileID;
 private profileDB  DB = new profileDB();;
 
 public FeedbackCluster(long _profileID) 
 {
   if (_profileID == 0) set_error_status();  
   {
    profileID = _profileID;
    DB.CloseConnect();
   }
 }

 public Long getProfileID()
 {
   if (get_error_status()) return null;  
   return new Long(profileID);  
 }
 
 public long getProfileID_longvalue()
 {
   if (get_error_status()) return 0;  
   return profileID;       
 }
 
 public void buildCluster()
 {
   DB.OpenConnect();  
   if (!get_error_status() && DB.createFeedbackCluster(this)) 
     {
      purge();
     }
    else set_error_status();
   DB.CloseConnect();
 }
 
 public void add(Long docid, int status)
 {
   if (!get_error_status() && docid != null && (status == NEGATIV || status == POSITIV))
    {
      allDocs.add(docid);  
      if (status == NEGATIV)
       {
         Integer I = (Integer)Dneg.get(docid);
         if (I == null) 
         {
           I = new Integer(0);
         }
         I = new Integer(I.intValue()+1);
         Dneg.put(docid, I);
       }
      
      if (status == POSITIV)
       {
         Integer I = (Integer)Dpos.get(docid);
         if (I == null) 
         {
           I = new Integer(0);
         }
         I = new Integer(I.intValue()+1);
         Dneg.put(docid, I);
       }
    }
 }
  
 public int getSizeDpos()
 {
   if (get_error_status()) return 0;
   return Dpos.size(); 
 }

 public int getSizeDneg()
 {
   if (get_error_status()) return 0;
   return Dneg.size(); 
 }
 
 public boolean purge()
 {
   if (!this.get_error_status())
    {
      
      Object[] os = Dneg.values().toArray(); 
      int n_I = os.length;
      Integer[] negI = null;
      if (n_I > 0)
       {
         negI = new Integer[n_I];      
         for (int j=0; j<n_I; j++) negI[j]=(Integer)os[j];
       }

      os = Dpos.values().toArray(); 
      n_I = os.length;
      Integer[] posI = null;
      if (n_I > 0)
       {
         posI = new Integer[n_I];
         for (int j=0; j<n_I; j++) posI[j]=(Integer)os[j];
       }
      
      double avgPos = 0;
      double avgNeg = 0;
      
      if (negI!= null && negI.length > 0)
       {  
        for (int i=0; i<negI.length; i++) avgNeg += negI[i].doubleValue();
        avgNeg = avgNeg/(double)negI.length;
       }
      else return false;
      
      if (posI!= null && posI.length > 0)
       {  
        for (int i=0; i<posI.length; i++) avgPos += posI[i].doubleValue();
        avgPos = avgPos/(double)posI.length;
       }
      else return false;
      
      Iterator It = allDocs.iterator();
      while (It.hasNext())
      {
        Long docid = (Long)It.next();
        int pos = 0;
        int neg = 0;
        Integer Ipos = (Integer)Dpos.get(docid);
        Integer Ineg = (Integer)Dneg.get(docid);
        if (Ipos != null) pos = Ipos.intValue();
        if (Ineg != null) neg = Ineg.intValue();
        
        /*
         *if total sum >= avgPos doc -> D++
         *if total sum <= avgNeg doc -> D--
         *otherwise doc -> rest (to remove)
         */        
        
        if ((pos-neg)>avgNeg) Dneg.remove(docid);          
        if ((pos-neg)<avgPos) Dpos.remove(docid);
      }
    }
   return true;
 }
 
 public double estimateResultList(Result_List RL)
 {
  if (get_error_status()) return 0.0;
  double score = 0.0;
  for (int i=0; i<RL.getLength() && i<TOPK; i++)
   {
    Long docid = RL.get_doc(i).get_doc_id();
    if (Dpos.containsKey(docid)) score = score + 1/this.getSizeDpos();
     else if (Dneg.containsKey(docid)) score = score - 1/this.getSizeDneg();
   }
  return score;
 }
}
