package search;

import java.util.*;
import java.lang.*;

import cern.colt.matrix.linalg.*;
import cern.colt.matrix.*;
import cern.colt.matrix.impl.*;

public class Linear_Regression extends Error
{
    private Result_List result_list;
    private Feedback feedback;
    private int NF = GLOBAL_NF;
    private double[][] A = new double[NF][NF];
    private double[][] B = new double[NF][1];
    
    public Linear_Regression(Result_List _result_list, Feedback _feedback) 
    {
      if (_result_list != null && _feedback != null && !_feedback.get_error_status() && !_result_list.get_error_status())
       {
         result_list = _result_list;
         feedback = _feedback;
         for (int i=0; i<NF; i++)
          {B[i][0]=0; for (int j=0; j<NF; j++) A[i][j]=0;}
       }
      else
        set_error_status();
    }
    
    public void set_NF(int _NF)
    {
      if (_NF>0)
       {
         NF = _NF;          
         A = new double[NF][NF];
         B = new double[NF][1];
         for (int i=0; i<NF; i++)
          {B[i][0]=0; for (int j=0; j<NF; j++) A[i][j]=0;}
       }
    }
    
    
    public double[] gauss(double[][] A, double[][] B)
    {
      double solution[] = new double[NF];  
      try 
      {

       double[][] A1 = new double[NF][NF]; 
       double[][] B1 = new double[NF][1];
       for (int i=0; i<NF; i++)
        {
         B1[i][0] = B[i][0];
         for (int j=0; j<NF; j++) A1[i][j] = A[i][j];
        }
       
      int[] orderY = new int[NF];
      int[] orderX = new int[NF];
      for (int i=0; i<NF; i++) orderY[i] = -1;
      for (int i=0; i<NF; i++) orderX[i] = i;
      
      for (int i=0; i<NF-1; i++)
       {
        int j = 0;
        boolean go = true;
        while (j<NF && go) 
         {
          int sum = -1;
          if (orderY[j] == -1)   
          for (int l=0; l<NF; l++)
           if (A1[j][l] == 0 && orderX[l] <= i) sum++; 
          if (sum == i-1) go = false;
          if (go) j++;
         }
        if (j<NF)
         {
           orderY[j] = i;  
           for (int l=0; l<NF; l++)
           if (orderY[l] == -1 && A1[l][orderY[j]] != 0)
           {
              double k = A1[l][orderY[j]]/A1[j][orderY[j]];
              for (int r=0; r<NF; r++)
               A1[l][r] -= A1[j][r]*k;
              B1[l][0] -= B1[j][0]*k;  
           }
         }
         else
         {
             
         }
       }
      
      int k=0;
       while (k<NF && orderY[k]!=-1) k++;
      if (k<NF && A1[k][NF-1]!=0) orderY[k]=NF-1;
      
      for (int i=NF-1; i>=0; i--)
       {
         int j=0;  
         while (j<NF && orderY[j]!=i) j++;
         if (j<NF && A1[j][orderX[i]]!=0)
          {
            double sum = B1[j][0];  
            for (int l=i+1; l<NF; l++)
             sum -= A1[j][orderX[l]]*solution[l];   
            solution[i] = sum/A1[j][orderX[i]];  
          }
         else solution[i]=0;   
       }
      }
      catch (Exception e)
      {
        set_error_message(e.toString());  
      }
            
      return solution;
    }
    
    public double[] another(double[][] A, double[][] B)
    {
      try
      {
       double[] solution = new double[NF];
       double[][] A1 = new double[NF][NF]; 
       double[][] B1 = new double[NF][1];
       for (int i=0; i<NF; i++)
        {
          B1[i][0] = B[i][0];
          for (int j=0; j<NF; j++) A1[i][j] = A[i][j];
        }
       Algebra Solver = new Algebra();
     
       DenseDoubleMatrix2D MA = new DenseDoubleMatrix2D(A1);
       DenseDoubleMatrix2D MB = new DenseDoubleMatrix2D(B1);


       
       LUDecomposition L = new LUDecomposition(MA);
       SingularValueDecomposition S = new SingularValueDecomposition(MA);
       int [] pivot = L.getPivot();
       int r = S.rank();
       
       if (!L.isNonsingular())
        {
          if (r == 0) return null;  
          MA = (DenseDoubleMatrix2D)Solver.permuteRows(MA, pivot, null);
          MA = (DenseDoubleMatrix2D)Solver.permuteColumns(MA, pivot, null);
          MA = (DenseDoubleMatrix2D)Solver.subMatrix(MA, 0, r-1, 0, r-1);
          MB = (DenseDoubleMatrix2D)Solver.permuteRows(MB, pivot, null);
          MB = (DenseDoubleMatrix2D)Solver.subMatrix(MB, 0, r-1, 0, 0);
       }
       
       //set_error_message(">>>>>>>>>>>>>>>>>>>>>>>>>>A: \n"+Solver.toVerboseString(MA));
       
       DenseDoubleMatrix2D MC = (DenseDoubleMatrix2D)Solver.solve(MA, MB);
       set_error_message(">>>>>>>>>>>>>>>>>>>>>>>>>C: \n"+MC.toString());
       double[] d = MC.viewColumn(0).toArray();
       if (!L.isNonsingular()) 
        for (int i=0; i<d.length; i++) solution[pivot[i]]=d[i];
       else solution = d;
       return solution;
      }
      catch (Exception e)
      {
        set_error_message("Linera_Reg_problems:\n"+e.toString());  
      }
      return null;  
    }
    
    public double[] computeProfile()
    {
      if (get_error_status()) return null;  
       for (int k=0; k<feedback.getLength(); k++)
       {
        Result_Document R_D = result_list.get_doc(feedback.getDOCID(k));
        double y = feedback.getDocScore(k); 
        for (int i=0; i<NF; i++)
         {
          double x = R_D.unit_scores[i];
          B[i][0] += y*x;
          for (int j=0; j<NF; j++)
           {
            double d = R_D.unit_scores[j];
            d = d*x;
            A[i][j] += d;
           }
         }
       }
       
       Random RD = new Random();
       for (int i=0; i<NF; i++) 
        {
         B[i][0] = B[i][0]*NF;    
         double d = (10.0 - RD.nextInt(20))/ 100.0;   
         if (B[i][0] == 0) B[i][0] = d; 
        }
      
     
      double[] solution = null;
            
//      solution = gauss(A,B);
      
      solution = another(A,B);
      
      for (int l=0; l<NF; l++)
       set_error_message("w"+l+" "+solution[l]);

      return solution;
    }        
    
    
    
}
