package search;

import java.util.*;
import java.lang.*;

public class Result_List extends Error
{ 
    private Result_Document[] list;
    private HashMap hash_list;
    private HashMap doc_pos;
    private boolean doc_pos_created                 = false;
    private boolean hash_list_created               = false;  
    private double[] average;
    
    public Result_List(Result_Document[] _list) 
    {
      if (_list == null) set_error_status();
       else
       {
         list = _list;
         this.computeAverage();
         //set_error_message("avg: "+this.vector_to_string(average, 4));
         
         //for (int i=0; i<list.length; i++)
         //  list[i].normalize_by_average(average);
         
         Arrays.sort(list);
       }
    }
    
    public Result_List(Result_Document[] _list, double[] _profile) 
    {
      if (_list == null || _profile == null) set_error_status();
       else
       {
         list = _list;
         set_profile(_profile);
       }
    }
    
    public boolean set_profile(double[] _profile)
    {
        if (_profile == null) return false; 
        
        for (int i=0; i<list.length; i++) list[i].set_profile(_profile);
          Arrays.sort(list);
          
        hash_list_created = false; 
        doc_pos_created = false;
        return true;
    }
    
    public void create_hash_list()
    {
      hash_list = new HashMap();
      for (int i=0; i<list.length; i++)
       hash_list.put(list[i].get_doc_id(), list[i]);
      hash_list_created = true;
    }
    
    public Result_Document get_doc(long _doc_id)
    {
     if (_doc_id == 0) return null;
     if (!hash_list_created) create_hash_list();
     if (!hash_list_created) return null;
     Long key = new Long(_doc_id);
     return (Result_Document)hash_list.get(key);
    }
    
    public Result_Document get_doc(int i)
    {
     if (i<0 || i>=list.length) return null;
     return list[i];
    }
    
    public int get_doc_position(long _doc_id)
    {
     if (_doc_id == 0) return -1;
     if (!doc_pos_created) create_doc_pos();
     Long key = new Long(_doc_id);
     Integer I = (Integer)doc_pos.get(key);
     if (I == null) return -1;
     return I.intValue()+1;
    }

    public void create_doc_pos()
    {
      doc_pos = new HashMap();
      for (int i=0; i<list.length; i++)
      {
        Long key = list[i].get_doc_id();
        Integer value = new Integer(i);
        doc_pos.put(key, value);
      }
      doc_pos_created = true;
    }
    
    public int getLength()
    {
     return list.length;   
    }
    
    public int getPositiveLength()
    {
      int n = 0;  
      for (int i=0; i<list.length && list[i].Sum()>0; i++) n++;
      if (n > list.length) n = list.length;
      return n;
    }
    
    public double estimateFeedback(double[] _profile, Feedback feedback)
    {
      if (this.set_profile(_profile))
       return estimateFeedback(feedback);
      return 0;
    }
    
    public double estimateFeedback(Feedback feedback)
    {
      if (feedback == null || feedback.get_error_status()) return 0;
      double score = 0;
      int np = this.getPositiveLength();
      int n  = this.getLength();
      double maxp = 1.0;
      double maxn = 1.0;
      int n_pos = feedback.getNumPos();
      int n_neg = feedback.getNumNeg();
      
      if (np > 0) maxp = 1;       
      for (int i=0; i<feedback.length(); i++)
       {
         long doc_id = feedback.getDOCID(i);
         int pos = this.get_doc_position(doc_id);
         int status = feedback.getStatus(i);
         if (status == POSITIV)
          {              
            if (pos <= np) score = score + maxp*(np - pos + 1)/np;
             else score = score - 1;
          }
         else if (status == NEGATIV)
         { 
           if (pos > np) score = score + 1;
            else score = score - maxp*(np - pos + 1)/np;
         }
       }
      Random R = new Random();
      if (score>0) score = score + R.nextDouble()/10000000;
      double med = (n - n_neg + n_pos)/2;
      double k1 = 0.7;
//      if (med < np) score +=   k1*np / med;
//       else score += k1 + (1-k1)*(np - med)/ np;
      return score;
    }
    
    
    public double[] getPosCentroid(Feedback feedback)
    {
      if (feedback == null || feedback.get_error_status()) return null;
      double[] centroid = new double[GLOBAL_NF];
      if (feedback.getNumPos() > 0)
       {
         int k = 0;  
         for (int i=0; i<feedback.getLength(); i++)
          if (feedback.getStatus(i) == POSITIV)
          {
            Result_Document R_D = this.get_doc(feedback.getDOCID(i));
            if (R_D != null)
            {   
             k++;  
             for (int j=0; j<GLOBAL_NF; j++)
               centroid[j] += R_D.unit_scores[j];  
            }              
          }        
         for (int j=0; j<GLOBAL_NF; j++)
          centroid[j] = centroid[j]/k;
       }
      else
       {
         int k = 0;  
         for (int i=0; i<this.getPositiveLength(); i++)
          if (feedback.getStatus(this.get_doc(i).get_doc_id_longValue()) != NEGATIV )
          {
            Result_Document R_D = this.get_doc(i);
            if (R_D != null)
            {   
             k++;  
             for (int j=0; j<GLOBAL_NF; j++)
               centroid[j] += R_D.unit_scores[j];  
            }              
          }        
         for (int j=0; j<GLOBAL_NF; j++)
          centroid[j] = centroid[j]/k;           
       }
      return centroid;
    }

    public double[] getNegCentroid(Feedback feedback)
    {
      if (feedback == null || feedback.get_error_status()) return null;
      double[] centroid = new double[GLOBAL_NF];
      if (feedback.getNumPos() > 0)
       {
         int k = 0;  
         for (int i=0; i<feedback.getLength(); i++)
          if (feedback.getStatus(i) == NEGATIV)
          {
            Result_Document R_D = this.get_doc(feedback.getDOCID(i));
            if (R_D != null)
            {   
             k++;  
             for (int j=0; j<GLOBAL_NF; j++)
               centroid[j] += R_D.unit_scores[j];  
            }              
          }        
         for (int j=0; j<GLOBAL_NF; j++)
          centroid[j] = centroid[j]/k;
       }
      else
       {
         int k = 0;  
         for (int i=0; i<this.getPositiveLength(); i++)
          if (feedback.getStatus(this.get_doc(i).get_doc_id_longValue()) != POSITIV )
          {
            Result_Document R_D = this.get_doc(i);
            if (R_D != null)
            {   
             k++;  
             for (int j=0; j<GLOBAL_NF; j++)
               centroid[j] += R_D.unit_scores[j];  
            }              
          }        
         for (int j=0; j<GLOBAL_NF; j++)
          centroid[j] = centroid[j]/k;           
       }
      return centroid;
    }
    
    public void computeAverage()
    {
      average = new double[GLOBAL_NF];
      Arrays.fill(average, 0.0);
      int n = this.getLength();
      for (int i=0; i<n; i++)
       {
        double[] temp = this.get_doc(i).unit_scores;
        if (temp!=null)
        for (int j=0; j<GLOBAL_NF; j++)
          average[j] += temp[j];
       }
      
     for (int j=0; j<GLOBAL_NF; j++) average[j] = average[j]/n;
    }
    
    public double[] getAverage()
    {
     if (average == null) this.computeAverage();   
     return average;   
    }
    
    public double maxBorder(Feedback feedback)
    {
       int n_pos = feedback.getNumPos();
       int n_neg = feedback.getNumNeg();
       int n = getLength();
       if (n_pos + n_neg > n) return -1;
       double np = (n - n_neg + n_pos)/2; 
       double d = ((n_pos)*(2*np-n_pos + 1)/(2*np))+n_neg;
       return d;
    }

}
